#!/bin/perl

#-----------------------------------------------------------
# Distributed Brokered Networking (DBNet) alert perl script
# author: Luis Cano
# date:   02/21/96
#-----------------------------------------------------------

$dbn_root = $ENV{'DBNROOT'};

if (!defined($dbn_root) || !$dbn_root) {
  print STDERR "\n$0: environment variable DBNROOT not set!\n";
  exit 1;
}

# rugged untainting. 
$ENV{PATH} = $dbn_env_path;

$dbn_config="${dbn_root}/table/dbn_config";
$dbn_config =~ /(.*)/;  # really bad untainting
require "$1";


unshift(@INC, '.');
unshift(@INC, "$include_dir");

require 'dbn_util.pl';


#local($type, $subtype, $path, $optional_file, @ct, $f, $dir);
local($type, $subtype, $path, $optional_file, $f, $dir);
local($size, $filedate, $alert_msg, $alert_msg_fn, $msgcode, $sitecode, @cmd_list); 

if (!&util'init()) {
  &util'msg($util'ERROR_MSG,"$0: init() failed!");
  exit 1;
}


if (@ARGV < 4 || @ARGV > 5) {
  &util'msg($util'ERROR_MSG,"usage: $0 type subtype jobname fullpath [optional_file]");
  exit 1;
}


$type=shift;
$subtype=shift;
$jobname=shift;
$path=shift;

&util'msg($util'INFO_MSG,"$0: entry type=$type subtype=$subtype job=$jobname path=$path $optional_file");

if ($optional_file && (! -r $optional_file)) {
  &util'msg($util'ERROR_MSG,"$0: optional_file $optional_file not readable: $!: disabling");
  $optional_file=""; 
}


($dir, $f)=&util'split_path($path);

if (!(($filedate, $size)=&util'build_filedate("$path"))) {
  &util'msg($util'ERROR_MSG,"$0: cannot build filedate");
  exit 1;
}
  
#@ct=gmtime(time);
#$ct[4]++;


if (!($msgcode=&util'parse("msgcode", "$parse_table",  "$type",  "$subtype",  "$path"))) {
  &util'msg($util'ERROR_MSG,"$0: parse(msgcode, ${parse_table}, ${type},  ${subtype},  ${path}) failed!");
  exit 1; 
}

if (!($sitecode=&util'parse("sitecode", "$parse_table",  "HOST", "host",  "$hostaddr"))) {
  &util'msg($util'ERROR_MSG,"$0: parse(sitecode, ${parse_table}, HOST, host, ${hostaddr}) failed!");
  exit 1;
}

$alert_msg_fn=&util'build_fn("$alert", "$sitecode", "$msgcode","0");
$data = $static_data;
$data=&util'build_timestamped_dataque;
$alert_msg="${data}/${alert_msg_fn}";

# realy bad way to untaint our variable
$alert_msg =~ /(.*)/;
$alert_msg = $1;

if (!open(ALERT, "> $alert_msg")) {
  &util'msg($util'ERROR_MSG,"$0: open $alert_msg failed: $!");
  exit 1;
}


#printf ALERT "AL%.4s %.4s %02d%02d%02d\n", $msgcode, $sitecode, $ct[3], $ct[2], $ct[1];
#printf ALERT "%s %s %02d%02d%02d%02d%02d%02d %s\n", $alert, $ver, $ct[5], $ct[4], $ct[3], $ct[2], $ct[1], $ct[0], $hostaddr;
printf ALERT "%s %s %s %s %s %s %s %s %s\n", $hostaddr, $path, $filedate, $size, $noop, $noop, $type, $jobname ,$subtype;

if ($optional_file) {
  if (!open(OPTIONAL_FILE, "$optional_file")) {
      &util'msg($util'ERROR_MSG,"$0: open optional_file $optional_file failed: $!: disabling");
      $optional_file="";
  }
  else {
    while (<OPTIONAL_FILE>) {
      print ALERT;
    }
    close(OPTIONAL_FILE) || &util'msg($util'ERROR_MSG,"$0: close optional_file $optional_file failed: $!");
  }
}

close ALERT || &util'msg($util'ERROR_MSG,"$0: close alert_msg $alert_msg failed: $!");
&util'msg($util'INFO_MSG,"$0: created alert msg: $alert_msg");


if (!(@cmd_list=&util'parse("quecmd", "$parse_table", "$quecmd", "$alert", "${sitecode}${msgcode}"))) {
  &util'msg($util'ERROR_MSG,"$0: parse(quecmd, $parse_table, $quecmd, $alert, ${sitecode}${msgcode}) failed!");
  &util'dbn_rename("$alert_msg", "${err_dir}/${alert_msg_fn}");
  exit 1;
}

if (!&util'quecmd(*cmd_list, $dbnet_que, $alert_msg, $alert_msg_fn, $parse_table, $msgcode)) {
  &util'msg($util'ERROR_MSG,"$0: quecmd(cmd_list) failed");
  &util'dbn_rename("$alert_msg", "${err_dir}/${alert_msg_fn}");
  exit 1;
}


exit 0;
