#!/bin/perl

#-----------------------------------------------------------
# Distributed Brokered Networking (DBNet) async perl script
#-----------------------------------------------------------

$dbn_root = $ENV{'DBNROOT'};

if (!defined($dbn_root) || !$dbn_root) {
  print STDERR "\n$0: environment variable DBNROOT not set!\n";
  exit 1;
}

$dbn_config="${dbn_root}/table/dbn_config";
require "$dbn_config";


unshift(@INC, '.');
unshift(@INC, "$include_dir");

require 'dbn_util.pl';

if (!&util'init()) {
  &util'msg($util'ERROR_MSG,"$0: init() failed!");
  exit 1;
}

use File::Copy;
use File::Basename;

if (@ARGV != 1)  {
  &usage();
  exit 1;
}

#
# globals
#
my($device)=shift;
#$path=shift;
#$cache=shift;
#$device=$async_device;
if (!defined $async_device_table{$device} ) {
  &util'msg($util'ERROR_MSG,"$0: init() failed! device: $device not found in async device table");
  exit 1;
}

my($path) = $async_device_table{$device}{queue_path};
my($cache) = $async_device_table{$device}{cache_time};
my($prefix) = $async_device_table{$device}{msg_format};
my($stop) = pack ("c","$async_device_table{$device}{stop}");
my($start) = pack ("c","$async_device_table{$device}{start}");


#$path=$async_prefix_path;
#$cache=$async_cache_time;
my($scan_rate) = 1;
my($tolerance) = $scan_rate+1;
my($readsz) = 1024;
my($def_cache) = $tolerance+1;
$debug=0;


if ($cache < $def_cache) {
  &util'msg($util'INFO_MSG,"$0: cache($cache) < $def_cache($scan_rate): resetting cache to $def_cache");
  $cache=$def_cache;
}

$tolerance = $cache - $tolerance;

&util'msg($util'INFO_MSG,"$0: entry device=${device} path_prefix=${path} cache_time=${cache}");
&util'msg($util'INFO_MSG,"$0: entry scan_rate=${scan_rate} tolerance=${tolerance} readsz=${readsz}");

my($len, $buf, $current_time, $offset, $prev_time, $written, $to_open);
my($idle_time, $idle_threshold, $idle_delta, $device_open, $max_not_scan, $nbr_not_scan);
my($time_to_die);
my($sitecode) = "0000";
my($msgcode) = "0000";
my($tunnel) = $async_device_table{$device}{tunnel};

$prev_time=0;
$len=0;
$buf=0;
$offset=0;
$to_open=0;
$device_open=0;
$idle_time=time();
$current_time=$idle_time;
$idle_threshold=5 * 60;
$idle_delta=0;
$max_not_scan=0;
$nbr_not_scan=0;
$time_to_die=0;


&init_vars();
&main'init();


if (!open(DEVICE, "+<${device}") ) {
  &util'msg($util'ERROR_MSG,"$0: open $device: $!");
  &exit_hdlr;
}
`$async_device_table{$device}{device_settings}`;
$written=0;
while ($start && ! $written) {
  $written = syswrite( DEVICE, $start, 1, 0);
  if (! defined $written ) {
    &util'msg($util'ERROR_MSG,"$0: written=syswrite(): write start !defined: $!: exiting");
    &exit_hdlr;
  }
}

$device_open=1;
$paged = 0;

while ( 1 ) {

  &exit_hdlr if $time_to_die;

#  if ( $scan_rate > 0 ) {
#   if ( ($current_time != $idle_time) || ($nbr_not_scan >= $max_not_scan) ) {
#      sleep($scan_rate);
#      $nbr_not_scan=0
#    }
#    else {
#      $nbr_not_scan++;
#    }
#  }

#  sleep($scan_rate) if ( ($scan_rate > 0) && ($current_time != $idle_time) );

  
  $len = sysread(DEVICE, $buf, $readsz);
  if (!defined $len) {
    if ($! =~ /^Interrupted/) {
      &util'msg($util'ERROR_MSG,"$0: len=sysread(): len !defined: $!: continuing");
      next;
    }
    &util'msg($util'ERROR_MSG,"$0: sysread() error: len !defined: $!: terminating");
    &exit_hdlr;
  }
  
  $current_time=time();
  if ( ($to_open) && ($current_time >= ($prev_time + $tolerance)) ) {
    close(TO) || &util'msg($util'ERROR_MSG,"$0: close ${path}.${prev_time}.${cache} failed: $!");
    &util::dbn_rename("${path}/.$fn", "${path}/${fn}");
    &buildRAWmessage("${path}/${fn}") if ($tunnel);
    $to_open = 0;
    if ( $debug ) {
      &util'msg($util'DEBUG_MSG,"$0: closed ${path}.${prev_time}.${cache}");
    }
  }

  if ( $len < 1 ) {
    $idle_delta=$current_time - $idle_time;
    if ( $idle_delta > $idle_threshold ) {
      callPager("dbnoper","Line IDLE") if ($paged == 0);
      $paged = 1;
      &util'msg($util'WARN_MSG,"$0: idle_threshold exceeded: idle_delta=${idle_delta} idle_threshold=${idle_threshold}");
      $idle_time=$current_time;
    }
    next;
  }

  if ($debug) {
    $idle_delta=$current_time - $idle_time;
    &util'msg($util'DEBUG_MSG,"$0: lenread=${len} scan=${scan_rate} idle_delta=${idle_delta}");
  }

  $idle_time=$current_time;
  $paged = 0;  

  if (! $to_open) {
     $fn = &util::build_fn($prefix,$sitecode,$msgcode,0);
#   if ( ! open( TO, "> ${path}.${current_time}.${cache}") ) {
    if ( ! open( TO, "> ${path}/.${fn}") ) {
      &util'msg($util'ERROR_MSG,"$0: open ${path}/.${fn} failed: $!");
      &exit_hdlr;
    }
    $prev_time = $current_time;
    $to_open=1;
    if ( $debug ) {
      &util'msg($util'DEBUG_MSG,"$0: opened ${path}/.${fn}");
    }
  } 
  $offset=0;
  while ($len) {
    $written = syswrite( TO, $buf, $len, $offset);
    if (! defined $written ) {
      &util'msg($util'ERROR_MSG,"$0: written=syswrite(): written !defined: $!: terminating");
      &exit_hdlr;
    }
    if ($debug) {
      &util'msg($util'DEBUG_MSG,"$0: written=${written} lenread=${len} offset=${offset}");
    }
    $len -= $written;
    $offset += $written;
#   select((select(TO), $| = 1)[$[]);  # flush
  } 
           
}

sub usage {
  &util'msg($util'ERROR_MSG,"$0 usage: $0 async_device_table_index");
}

sub init {
  $SIG{'INT'} = 'sig_hdlr';
  $SIG{'HUP'} = 'init_vars';
  $SIG{'QUIT'} = 'sig_hdlr';   # action being taken
# $SIG{'ILL'} = 'sig_hdlr';
  $SIG{'TRAP'} = 'sig_hdlr';
# $SIG{'ABRT'} = 'sig_hdlr';
  $SIG{'IOT'} = 'sig_hdlr';
# $SIG{'BUS'} = 'sig_hdlr';
# $SIG{'FPE'} = 'sig_hdlr';
# $SIG{'KILL'} = 'sig_hdlr';
  $SIG{'USR1'} = 'sig_hdlr';
# $SIG{'SEGV'} = 'sig_hdlr';
  $SIG{'USR2'} = 'sig_hdlr';
  $SIG{'PIPE'} = 'sig_hdlr';
  $SIG{'ALRM'} = 'sig_hdlr';
  $SIG{'TERM'} = 'sig_hdlr';   # action being taken
  $SIG{'STKFLT'} = 'sig_hdlr';
  $SIG{'CHLD'} = 'sig_hdlr';
  $SIG{'CONT'} = 'sig_hdlr';
  $SIG{'STOP'} = 'sig_hdlr';
#  $SIG{'TSTP'} = 'sig_hdlr';
  $SIG{'TTIN'} = 'sig_hdlr';
  $SIG{'TTOU'} = 'sig_hdlr';
  $SIG{'URG'} = 'sig_hdlr';
  $SIG{'XCPU'} = 'sig_hdlr';
  $SIG{'XFSZ'} = 'sig_hdlr';
  $SIG{'VTALRM'} = 'sig_hdlr';
  $SIG{'PROF'} = 'sig_hdlr';
# $SIG{'WINCH'} = 'sig_hdlr';
  $SIG{'IO'} = 'sig_hdlr';
  $SIG{'POLL'} = 'sig_hdlr';
  $SIG{'PWR'} = 'sig_hdlr';
  $SIG{'UNUSED'} = 'sig_hdlr';

  return 1; 
}

sub sig_hdlr {
  my($sig) = @_;
  &util'msg($util'ERROR_MSG,"$0: INTERUPT: $sig");
  $time_to_die=1 if ($sig eq "QUIT") || ($sig eq "TERM") || ($sig eq "INT");
}

sub exit_hdlr {
  &util'msg($util'ERROR_MSG,"$0: exit");
  if ( $device_open ) {
    $written=0;
    while ($stop && ! $written) {
      $written = syswrite( DEVICE, $stop, 1, 0);
      if (! defined $written ) {
        &util'msg($util'ERROR_MSG,"$0: written=syswrite(): write stop !defined: $!: exiting");
        $written=1;
      }
    }
    &util'msg($util'ERROR_MSG,"$0: exit: closing $device");
    close(DEVICE) || &util'msg($util'ERROR_MSG,"$0: exit: close $device failed: $!");
  }
  if ( $to_open ) {
    &util'msg($util'ERROR_MSG,"$0: exit: closing ${path}/.${fn}");
    close(TO) || &util'msg($util'ERROR_MSG,"$0: exit: close ${path}/.${fn} failed: $!");
  }
  exit 1;
} 



sub buildRAWmessage {
my ($filename) = @_;
my (@cmdlist,$fn,$timestamped_datadir);

  $timestamped_datadir=$static_data;
 
  if (! ($timestamped_datadir=&util'build_timestamped_dataque) ) {
    &util'msg($util'ERROR_MSG,"$0: Cannot create timestamped dbnet directory: cannot tunnel");
    return 0; 
  }

  $fn = basename($filename); 
  copy("$filename", "${timestamped_datadir}/${fn}");

  if (!(@cmd_list=&util'parse("quecmd", "$parse_table", "$quecmd", "$prefix", "${sitecode}${msgcode}"))) {
    &util'msg($util'ERROR_MSG,"$0: parse(quecmd, $parse_table, $quecmd, $prefix, $sitecode$msgcode) failed!");
    &util'dbn_rename("${timestamped_datadir}/${fn}", "${err_dir}/${fn}");
    return;
  }

  if (!&util'quecmd(*cmd_list, $dbnet_que, "${timestamped_datadir}/${fn}", $fn, $parse_table, $msgcode)) {
    &util'msg($util'ERROR_MSG,"$0: quecmd(cmd_list) failed");
    &util'dbn_rename("${timestamped_datadir}/${fn}", "${err_dir}/${fn}");
    return;
  }
}

sub init_vars {
  &util'parse_cfg_set_hup();

  &util'msg($util'INFO_MSG,"$0: Reparsing sitecode and message code.");
  
  if (!($msgcode=&util'parse("msgcode", "$parse_table",  "$prefix",  "$device",  "*"))) {
    &util'msg($util'ERROR_MSG,"$0: parse(msgcode, ${parse_table}, $prefix,  ${device}, *) failed!");
#    return; 
  }

  if (!($sitecode=&util'parse("sitecode", "$parse_table",  "HOST", "host",  "$hostaddr"))) {
    &util'msg($util'ERROR_MSG,"$0: parse(sitecode, ${parse_table}, HOST, host, ${hostaddr}) failed!");
#    return;
  }
}

sub callPager {
  my($who, $message) = @_;
  my($fname) = "$timestamped_datadir/Pager.${who}.$$";

  if (!open(PAGERFILE, ">$fname")) {
    &util'msg($util'ERROR_MSG,"$0: open failed in callPager: $!");
    return;
  }
  my ($h) = `hostname -s`;
  chomp($h);
  print PAGERFILE "$h $message\n" or
    &util'msg($util.ERROR_MSG,"$): unable to write to pager file:$!");

  if (!close(PAGERFILE)) {
    &util'msg($util'ERROR_MSG,"$0: close failed in callPager: $!");
    return;
  }

  &util'build_alert("$hostaddr","$fname","ASYNC","$device","dbn_async.pl") ;

}

