#!/bin/perl

#-----------------------------------------------------------
# Distributed Brokered Networking (DBNet) bufr perl script
#-----------------------------------------------------------

$dbn_root=$ENV{'DBNROOT'};

if (!defined($dbn_root) || !$dbn_root) {
  print STDERR "\n$0: environment variable DBNROOT not set!\n";
  exit 1;
}

require "${dbn_root}/table/dbn_config";

unshift(@INC, '.');
unshift(@INC, "$include_dir");

require 'dbn_util.pl';
  

if (!&util'init()) {
  &util'msg($util'ERROR_MSG, "$0: init() failed!");
  exit 1;
}

exit 1 if ! &util'is_running($tmp_dir, $bufr);

local(@bfiles, @sfiles, @bundle, $bfile, $rfile, $i, $subtype, $fsn, $pid, $ftime, @cmd_list, $btime);
local($bufr_pattern)="${bufr}\\.0+\\.([\\w]+)\\.([\\d]+)\\.([\\d]+)\\.([\\d]+)\\.([\\d]+)";
local($bufr_idle_time)=time;
local($sitecode, $bufr_cache_time);
local($subtype_a, $subtype_b, $fsn_a, $fsn_b, $pid_a, $pid_b, $ftime_a, $ftime_b, $msgcode_a, $msgcode_b, $bfile_a, $bfile_b);
local($amt_bundled);

&util'msg($util'WARN_MSG, "$0: entry: DBNROOT=${dbn_root}");

if (!($sitecode=&util'parse("sitecode", "$parse_table",  "HOST", "host",  "$hostaddr"))) {
  &util'msg($util'ERROR_MSG, "$0: parse(sitecode, ${parse_table}, HOST, host, ${hostaddr}) failed!");
  kill 'TERM', $$;
}

while (1) {
  exit 1 if &util'isit_time_to_die($tmp_dir, $bufr);

  $data = $static_data;
   if (! ($data=&util'build_timestamped_dataque) ) {
    &util'msg($util'ERROR_MSG,"$0: Cannot create timestamped dbnet directory");
    exit 1;
  }
   $dbnet_que = $static_dbnet_que;
  if (! ($dbnet_que=&util'build_timestamped_dbnet_que) ) {
    &util'msg($util'ERROR_MSG,"$0: Cannot create timestamped dbnet_que  directory");
    exit 1;
  }

  &scan();
  exit 1 if &util'isit_time_to_die($tmp_dir, $bufr);
  sleep $bufr_scan_time;
}

sub scan {
  
  $btime=time, @bfiles=(), @sfiles=(), @cmd_list=();

  if (!opendir(DIR, $tmp_dir)) {
    &util'msg($util'ERROR_MSG, "$0: opendir($tmp_dir) failed: $!");
    kill 'TERM', $$;
    return;
  }

  @bfiles=readdir(DIR);
  closedir(DIR);
  
  for ($i=@bfiles; $i > 0; $i--) {
    ($bfile) = splice(@bfiles, $i-1, 1);
    next if $bfile !~ /$bufr_pattern/;
    $subtype=$1, $fsn=$2, $pid=$3, $ftime=$4, $bufr_cache_time=$5;
    next if ($btime - $ftime) < ($bufr_cache_time + $bufr_tolerance);
    push(@bfiles, $bfile);
  }

  if ( ! @bfiles ) {
    return if ! $bufr_max_idle_time;
    if ( ($btime - $bufr_idle_time) > $bufr_max_idle_time ) {
      &util'msg($util'ERROR_MSG, "$0: exceeded bufr_max_idle_time=$bufr_max_idle_time");
      kill 'TERM', $$;
    }
    return;
  }
  $bufr_idle_time = $btime;

  @sfiles = sort scan_sort @bfiles;

  while (@sfiles) {
    $bfile = pop(@sfiles);
    $bfile =~ /$bufr_pattern/;
    $subtype_a=$1, $fsn_a=$2, $pid_a=$3, $ftime_a=$4;
    if (!($msgcode_a=&util'parse("msgcode", "$parse_table",  "$bufr",  "$subtype_a",  "$bfile"))) {
      &util'msg($util'ERROR_MSG, "$0: msgcode_a=parse(msgcode, ${parse_table}, ${bufr},  ${subtype_a},  ${bfile}) failed!");
      &util'dbn_rename("${tmp_dir}/${bfile}", "${err_dir}/${bfile}");
      next;
    }
    @bundle=($bfile);

    for ($i=@sfiles ; $i > 0 ; $i--) {
      $bfile = $sfiles[$i-1];
      $bfile =~ /$bufr_pattern/;
      $subtype_b=$1, $fsn_b=$2, $pid_b=$3, $ftime_b=$4;
      if (!($msgcode_b=&util'parse("msgcode", "$parse_table",  "$bufr",  "$subtype_b",  "$bfile"))) {
        &util'msg($util'ERROR_MSG, "$0: msgcode_b=parse(msgcode, ${parse_table}, ${bufr},  ${subtype_b},  ${bfile}) failed!");
        &util'dbn_rename("${tmp_dir}/${bfile}", "${err_dir}/${bfile}");
        next;
      }
      unshift(@bundle, splice(@sfiles, $i-1, 1) ) if ($msgcode_a eq $msgcode_b);
    }

    $bfile_a="";
    while (@bundle) {
      $bfile_a = shift(@bundle);
      last if open(BUFR_A, ">> ${tmp_dir}/${bfile_a}");
      &util'msg($util'ERROR_MSG, "$0: bfile_a = open ${tmp_dir}/${bfile_a} error: $!");
      &util'dbn_rename("${tmp_dir}/${bfile_a}", "${err_dir}/${bfile_a}");
      $bfile="";
    }
    next if ! $bfile_a;
    $amt_bundled = -s "${tmp_dir}/${bfile_a}";
    
    while (@bundle) {
      last if $amt_bundled > $bufr_max_amt_bundled;
      $bfile_b = shift(@bundle);
      if (!open(BUFR_B, "${tmp_dir}/${bfile_b}")) {
        &util'msg($util'ERROR_MSG, "$0: bfile_b = open ${tmp_dir}/${bfile_b} error: $!");
        &util'dbn_rename("${tmp_dir}/${bfile_b}", "${err_dir}/${bfile_b}");
      }
      else {
	$amt_bundled += -s "${tmp_dir}/${bfile_b}";
        while (<BUFR_B>) {
          print BUFR_A;
        }
        close (BUFR_B) || &util'msg($util'WARN_MSG, "$0: bfile_b = close(${tmp_dir}/${bfile_b}) failed: $!");
        if ( ! unlink("${tmp_dir}/${bfile_b}") ) {
          &util'msg($util'ERROR_MSG, "$0: bfile_b = unlink(${tmp_dir}/${bfile_b}) failed: $!");
          kill 'TERM', $$;
          return;
        }
      }
    } 

    close(BUFR_A) || &util'msg($util'WARN_MSG, "$0: bfile_a = close(${tmp_dir}/${bfile_a}) failed: $!"); 

    $bfile_a =~ /$bufr_pattern/;
    $fsn_a=$2, $pid_a=$3, $ftime_a=$4;

    $rfile="${bufr}.${sitecode}.${msgcode_a}.${fsn_a}.${pid_a}.${ftime_a}";
    &util'dbn_rename("${tmp_dir}/${bfile_a}", "${data}/${rfile}") || &util'msg($util'ERROR_MSG, "$0: data queue error: rename(${tmp_dir}/${bfile_a}, ${data}/${rfile}) failed: $!");
    if (!(@cmd_list=&util'parse("quecmd", "$parse_table", "$quecmd", "$bufr", "${sitecode}${msgcode_a}"))) {
      &util'msg($util'ERROR_MSG, "$0: parse(quecmd, $parse_table, $quecmd, $bufr, ${sitecode}${msgcode_a}) failed!");
      &util'dbn_rename("${data}/${rfile}", "${err_dir}/${rfile}");
      next;
    }
    if (!&util'quecmd(*cmd_list, $dbnet_que, "${data}/${rfile}", $rfile, $parse_table, $msgcode_a)) {
      &util'msg($util'ERROR_MSG, "$0: quecmd(cmd_list) failed");
      &util'dbn_rename("${data}/${rfile}", "${err_dir}/${rfile}");
      next;
    }
  }
}


sub scan_sort {

  $a =~ /$bufr_pattern/;
  $subtype_a=$1, $fsn_a=$2, $pid_a=$3, $ftime_a=$4;
  $b =~ /$bufr_pattern/;
  $subtype_b=$1, $fsn_b=$2, $pid_b=$3, $ftime_b=$4;
  $ftime_a <=> $ftime_b;
}
