#!/bin/perl


$dbn_root=$ENV{'DBNROOT'};

if (!defined($dbn_root) || !$dbn_root) {
  print STDERR "\n$0: environment variable DBNROOT not set!\n";
  exit 1;
}

require "${dbn_root}/table/dbn_config";

unshift(@INC, '.');
unshift(@INC, "$include_dir");

require 'dbn_util.pl';

# the util'sort routine uses command pattern
$cmd_pattern='^(\w+)\.([\w]+)\.([\w]+)\.([\d]+)\.([\d]+)\.([\d]+)';
# end of sort stuff

if (!&util'init()) {
  &util'msg($util'ERROR_MSG,"$0: init() failed!");
  exit 1;
}

exit 1 if ! &util'is_running($tmp_dir, "${dbn_external}");

&util'msg($util'INFO_MSG,"$0:Initializing, scan time=$external_scan_time, wait=$foreign_wait_time");

my (@pfiles);



while (1) {

  &exit_hdlr if &util'isit_time_to_die($tmp_dir,"${dbn_external}");

  $data = $static_data;
  if (! ($data=&util'build_timestamped_dataque) ) { 
    &util'msg($util'ERROR_MSG,"$0: Cannot create timestamped dbnet directory");
    exit 1;
  }

  &process_foreign();
  &process_pending();
  sleep($external_scan_time);
}

exit 1;



sub process_pending  {

  my(@cmds, $i,$index,$mtype, $hostid, $datatype, $seq);
  my($rpid,$buildtime);
  local(@cmd_list);

  my($j)=0;

  if (!defined @pfiles ) {
    if (!opendir(DIR, $pending_dir)) {
      &util'msg($util'ERROR_MSG,"$0: opendir($pending_dir) failed: $!");
      return;
    }

    @pfiles=readdir(DIR);
    closedir(DIR);

    &util'sort_cmds(\@pfiles,$cmd_pattern);
  }

  for ($index=0;$index <= $#pfiles; $index++) {
    $i = $pfiles[$index];

#    if ($j++ > 100) {
#      splice(@pfiles,0,$index);
#      return;
#    }

    ($mtype, $hostid, $datatype, $seq , $rpid, $buildtime) = split(/\./,$i);
    &util'dbn_rename("${pending_dir}/${i}","${data}/${i}") || next; 

    if (!(@cmd_list=&util'parse("quecmd", "$parse_table", "$quecmd", "$mtype", "${hostid}${datatype}"))) {
      &util'msg($util'ERROR_MSG,"$0: parse(quecmd, $parse_table, $quecmd, $mtype, $hostid$datatype) failed!");
      &util'dbn_rename("${data}/${i}", "${err_dir}/${i}");
      next;
    }

    if (!&util'quecmd(*cmd_list, $dbnet_que, "${data}/${i}", $i, $parse_table, $datatype, $hostid)) {
        &util'msg($util'ERROR_MSG,"$0: quecmd(cmd_list) failed");
        &util'dbn_rename("${data}/${i}", "${err_dir}/${i}");
        next;
    }
  }  
  undef @pfiles;
}


sub process_foreign  {

  my ($i,$filename,$mtime, $host, $type, $subtype, $job);
  my ($input_line, @files);

  if (!opendir(DIR, $foreign_dir)) {
    &util'msg($util'ERROR_MSG,"$0: opendir($foreign_dir) failed: $!");
    return;
  }

  @files=readdir(DIR);
  closedir(DIR);

  foreach $i (@files) {
    next if (($i =~ /^\./));

    $mtime = (stat("${foreign_dir}/${i}"))[9];

    next if ($mtime+$foreign_wait_time > time);

      if (!open(IFILE,"$foreign_dir/$i")) {
      &util'msg($util'ERROR_MSG,"Unable to open foreign file: $foreign_dir/$i");
      next;
    }

    $input_line = <IFILE>;
    close (IFILE);

    if (!($input_line =~ /(\S+)[ \t]+(\S+)[ \t]+(\S+)[ \t]+(\S+)[ \t]+(\S+)[ \t]+(\S+)[ \t]+(\S+)[ \t]+(\S+)[ \t]+(\S+)/)) {
      &util'msg($util'ERROR_MSG,"Unable to match pattern in foreign file: $foreign_dir/$i"
);
      &util'dbn_rename("${foreign_dir}/${i}", "${err_dir}/${i}");
      next;
    }
    $host = $1; $filename = $2; $type = $7; $job = $8; $subtype = $9;

    if (&build_alert($host,$filename,$type,$subtype,$job)) {
      &util'dbn_copy_unlink("${foreign_dir}/${i}","${data}/${i}") if $foreign_use_copy_unlink;
      &util'dbn_rename("${foreign_dir}/${i}","${data}/${i}") if ! $foreign_use_copy_unlink;

    }else {;
      &util'dbn_copy_unlink("${foreign_dir}/${i}","${err_dir}/${i}") if $foreign_use_copy_unlink;
      &util'dbn_rename("${foreign_dir}/${i}","${err_dir}/${i}") if ! $foreign_use_copy_unlink;
   }
  }
}

sub build_alert {
  my ($host,$path,$type,$subtype,$jobname,$optional_file) = @_;
  my ($filedate, $size, $msgcode, $sitecode, $alert_msg_fn);
  my ($alert_msg);

  if ($optional_file && (! -r $optional_file)) {
    &util'msg($util'ERROR_MSG,"$0: optional_file $optional_file not readable: $!: disabling");
    $optional_file="";
  }


  if (!($msgcode=&util'parse("msgcode", "$parse_table",  "$type",  "$subtype",  "$path")))
  {
    &util'msg($util'ERROR_MSG,"$0: parse(msgcode, ${parse_table}, ${type},  ${subtype}, ${path}) failed!");
    return 0;
  }

  if (!($sitecode=&util'parse("sitecode", "$parse_table",  "HOST", "host",  "$host"))) { 
    &util'msg($util'ERROR_MSG,"$0: parse(sitecode, ${parse_table}, HOST, host, ${host}) failed!");
     return 0; 
  } 
 
  if (!($localsitecode=&util'parse("sitecode", "$parse_table",  "HOST", "host",  "$hostaddr"))) { 
    &util'msg($util'ERROR_MSG,"$0: parse(sitecode, ${parse_table}, HOST, host, ${hostaddr}) failed!");
    $localsitecode = "0000";
  }

  if ($sitecode eq $localsitecode) {
    if (!(($filedate, $size)=&util'build_filedate("$path"))) {
      &util'msg($util'ERROR_MSG,"$0: cannot build filedate");
      return 0;
    }
  } else {
    $filedate = 0;
    $size = 0;
  }

  $alert_msg_fn=&util'build_fn("$alert", "$sitecode", "$msgcode","0");
  $alert_msg="${data}/${alert_msg_fn}";

  if (!open(ALERT, "> $alert_msg")) {
    &util'msg($util'ERROR_MSG,"$0: open $alert_msg failed: $!");
    return 0;
  }


  printf ALERT "%s %s %s %s %s %s %s %s %s\n", $host, $path, $filedate, $size, $noop, $noop, $type, $jobname ,$subtype;

  if ($optional_file) {
    if (!open(OPTIONAL_FILE, "$optional_file")) {
      &util'msg($util'ERROR_MSG,"$0: open optional_file $optional_file failed: $!: disabling");
      $optional_file="";
    }

   else {
      while (<OPTIONAL_FILE>) {
        print ALERT;
      }
      close(OPTIONAL_FILE) || &util'msg($util'ERROR_MSG,"$0: close optional_file $optional_file failed: $!");
    }
  }

  close ALERT || &util'msg($util'ERROR_MSG,"$0: close alert_msg $alert_msg failed: $!");
  &util'msg($util'INFO_MSG,"$0: created alert msg: $alert_msg");


  if (!(@cmd_list=&util'parse("quecmd", "$parse_table", "$quecmd", "$alert", "${sitecode}${msgcode}"))) {
    &util'msg($util'ERROR_MSG,"$0: parse(quecmd, $parse_table, $quecmd, $alert, $msgcode) failed!");
    &util'dbn_rename("$alert_msg", "${err_dir}/${alert_msg_fn}");
    return 0;
  }

  if (!&util'quecmd(*cmd_list, $dbnet_que, $alert_msg, $alert_msg_fn, $parse_table, $msgcode)) {
    &util'msg($util'ERROR_MSG,"$0: quecmd(cmd_list) failed");
    &util'dbn_rename("$alert_msg", "${err_dir}/${alert_msg_fn}");
    return 0;
  }
  return 1;
}


sub exit_hdlr {
  &util'msg($util'ERROR_MSG,"$0: exit");
  exit 1;
}
