#!/bin/perl

#-----------------------------------------------------------
#
# dbn_x224.pl
#
# Distributed Brokered Networking (DBNet) X224 perl script
#
# author: Luis Cano
# date:   07/24/96
#-----------------------------------------------------------

#use strict;
my($dbn_root) = $ENV{'DBNROOT'};

if (!defined($dbn_root) || !$dbn_root) {
  print STDERR "\n$0: environment variable DBNROOT not set!\n";
  exit 1;
}

my($dbn_config)="${dbn_root}/table/dbn_config";
require "$dbn_config";


unshift(@INC, '.');
unshift(@INC, "$include_dir");

require 'dbn_util.pl';
 
if (!&util'init()) {
  &util'msg($util'ERROR_MSG,"$0: init() failed!");
  exit 1;
}

if (@ARGV != 1)  {
  &util'msg($util'ERROR_MSG,"$0: usage - $0 device_table_index ");
  exit 1;
}

my($device_table_index) = shift;
my($servicehost) = $::async_device_table{$device_table_index}{site};
my($prefix) = $::async_device_table{$device_table_index}{msg_format};
my($hold_file) = "X224.0000.0000.0.0.0.0";
my($msgcode) = "0000";
my($sitecode) = "0000";
my($type) = $::async_device_table{$device_table_index}{type};

if (!($msgcode=&util'parse("msgcode", "$parse_table",  "$prefix",  "$device_table_index",  "*"))) {
  &util'msg($util'ERROR_MSG,"$0: parse(msgcode, ${parse_table}, $prefix,  ${device_table_index}, *) failed!");
  $msgcode = "0000";
}

if (!($sitecode=&util'parse("sitecode", "$parse_table",  "HOST", "host",  "$servicehost"))) {
  &util'msg($util'ERROR_MSG,"$0: parse(sitecode, ${parse_table}, HOST, host, ${servicehost}) failed!");
  $sitecode = "0000";
}

exit 1 if ! &util'is_running($tmp_dir, "${prefix}.${msgcode}");

# the next few lines should be in dbn_config
#my($wmo) = "WMO";
my($x224_dir) = "$::async_device_table{$device_table_index}{queue_path}";
my($use_x224_bytecount) = "$::async_device_table{$device_table_index}{use_x224_bytecount}";
#my($filename_pattern) = 'x25\.raw\.(\d+)\.(\d+)';
my($filename_pattern) = "^$async_device_table{$device_table_index}{msg_format}\\.${sitecode}\\.${msgcode}\\.";
# end of stuff that should be in dbn_config

# define patterns for CCB.  each field has a seperate pattern,
# they are then combined for a complete ccb

my($ccb_bytecount) = '\@\014'; # I really mean this =0x400c ;
my($ccb_mode) = '\000';
my($ccb_submode) = '\001';
my($ccb_precedence) = '[RPOZ]';
my($ccb_classification) ='[UCSTE]';
my($ccb_m_originator) = '....';
my($ccb_catagory) = '.';
my($ccb_subcatagory) = '.';
my($ccb_userdefined1) = '.';
my($ccb_userdefined2) = '.';
my($ccb_year) = '[\00-\150]';
my($ccb_month) = '[\000-\014]';
my($ccb_torday) = '[\000-\037]';
my($ccb_torhour) = '[\000-\030]';
my($ccb_torminute) = '[\000-\074]';
my($ccb_numdest) = '\001';
my($ccb_dest) = '....';

my($ccb_pattern) = "${ccb_bytecount}${ccb_mode}${ccb_submode}${ccb_precedence}${ccb_classification}${ccb_m_originator}${ccb_catagory}${ccb_subcatagory}(${ccb_userdefined1})(${ccb_userdefined2})${ccb_year}${ccb_month}${ccb_torday}${ccb_torhour}${ccb_torminute}${ccb_numdest}${ccb_dest}";

#$eom_pattern = '\015\015\012\015\015\012\003';
my($eom_pattern) = '\015\012\003';
my($ccb_len) = 24;
my($eom_len) = 3;
my($seq_number) = 0; # sequence number for error files, in case we get more than 1/sec
my($valid_ccb_messages) = 0;  # stats
my($ccb_received_byte_count) = 0;  # stats
my($valid_wmo_messages) = 0;  # stats
my($invalid_wmo_msg) = 0;  # stats

my($time_to_die);

&util'msg($util'DEBUG_MSG, "$0: Starting the WMO message parser now.");
my($wmo_TTAA00) = '\w{4,6}';
my($wmo_Kxxx) = '\w{4}';
my($wmo_day) = '[0-3][0-9]';
my($wmo_hour) = '[0-2][0-9]';
my($wmo_min) = '[0-5][0-9]';
my($wmo_date) = "$wmo_day$wmo_hour$wmo_min";
my($wmo_date_Z) = '(?:Z|UTC)?';
my($wmo_BBB) = 'RR[A-Z]|CC[A-Z]|AA[A-Z]|COR|AMD|RTD\d{0,3}|\w{3} {1,2}\d ';
my($wmo_seg) = 'P[A-Z][A-Z]';
my($wmo_eol) = '(?:\015)*\012';
my($wmo_line2) = '(?:(?:(\w{5}[\w ]?)\015)|(METAR))';

my($wmo_pattern) = "($wmo_TTAA00)[ ]+($wmo_Kxxx)[ ]*(?:($wmo_date)$wmo_date_Z)?[ ]*($wmo_BBB)?[ ]*(?:($wmo_seg)[ ]*)?$wmo_eol(?:$wmo_line2)?";

my(@available_files);
my($i);
my($j);
my($bytes_read);
my($input);

# setup signal handlers first...
# &init();

#loop forever
while (1) {


  &exit_hdlr if &util'isit_time_to_die($tmp_dir,"${prefix}.${msgcode}");

  # check to see what files are available.
  if ( @available_files = &check_files()) { 
    if (!($msgcode=&util'parse("msgcode", "$parse_table",  "$prefix",  "$device_table_index",  "*"))) {
      &util'msg($util'ERROR_MSG,"$0: parse(msgcode, ${parse_table}, $prefix,  ${device_table_index}, *) failed!");
      $msgcode = "0000";
    }

    if (!($sitecode=&util'parse("sitecode", "$parse_table",  "HOST", "host",  "$servicehost"))) {
      &util'msg($util'ERROR_MSG,"$0: parse(sitecode, ${parse_table}, HOST, host, ${servicehost}) failed!");
      $sitecode = "0000";
    }

    $hold_file = "${prefix}.${sitecode}.${msgcode}.0.0.0.0";

    $j = $i = $[; 
    $bytes_read=0;
    undef $input;$input="";
    while ($i <= $#available_files) {
      # read all available files upto 1 megabyte
      # init the global varaibles...  
      # read some data
      while (($bytes_read < 500000) && ($i <= $#available_files)) {
        &read_file($available_files[$i++]);
      }

      # find valid messages in buffer, save them as DBNet WMO messages
      $use_x224_bytecount ? &process_buffer_by_bytecount() : &process_buffer_by_CCB();
      # remove processed files and write the remaining data back out.
 
      $bytes_read = length($input); 

      for ($j=$j;$j<$i;$j++) {
        if ($available_files[$j] ne "$hold_file") {
          #&util'dbn_rename("$x224_dir/$available_files[$j]","$data/$available_files[$j]") ;
          unlink("$x224_dir/$available_files[$j]") ;
          &util'msg($util'INFO_MSG,"$0: unlink file $x224_dir/$available_files[$j]");
        }

      }

    }

    if ($input ne "") {
      if (!open(_FILE, ">$x224_dir/${hold_file}")) {
        &util'msg($util'ERROR_MSG,"$0: open file $x224_dir/${hold_file} failed: $!:");
      }

      print _FILE $input;

      if (!close(_FILE) ) {
        &util'msg($util'ERROR_MSG,"$0: close file $x224_dir/${hold_file} failed: $!:");
        return 0;
      }   
    } else {
      unlink("$x224_dir/${hold_file}");
    }


  } # no files to process


  &util'msg($util'DEBUG_MSG,"$0: completed scan: ccb_messages=$valid_ccb_messages, wmo_messages=$valid_wmo_messages, errors=$seq_number, WMO-errors=$invalid_wmo_msg, ccb_bytes=$ccb_received_byte_count");

  sleep ($async_device_table{$device_table_index}{ingest_time});

  # sleep for a little while and do it again

}


# exit 1;

sub read_file {
  my ($filename) = @_;

  if (!open(_FILE, "$x224_dir/$filename")) {
    &util'msg($util'ERROR_MSG,"$0: open file $filename failed: $!:");
    return 0;
  }
  else {
    while ($j = read(_FILE, $input,4096,$bytes_read)) {
      $bytes_read += $j;
    }

    if (!close(_FILE) ) {
      &util'msg($util'ERROR_MSG,"$0: close file $filename failed: $!");
      return 0;
    }   
  }
}


sub check_files {
  my(@file_list,@new_list, $fn);
  my (%keylist);
#  my ($curr_time) = time;

  if (!opendir(DIR, $x224_dir)) {
    &util'msg($util'ERROR_MSG,"$0: opendir($x224_dir) failed: $!");
    return ();
  }

  @file_list=readdir(DIR);
  closedir(DIR);

  foreach $fn (@file_list) {
    if ($fn =~ /$filename_pattern/) {
#      if (( $1+$2) < $curr_time) {
        push(@new_list,$fn);
#      }
    }
  }  

  foreach $fn (@new_list) {
    $keylist{$fn} = (split(/\./,$fn))[5];
  }
  return sort { $keylist{$a} <=> $keylist{$b} }  @new_list;

}

sub process_buffer_by_bytecount {
  my ($str1, $str2, $offset1, $offset2);
  my ($bytecount, $b1, $b2);
  
  while (1) {

  if (!($input =~ /($ccb_pattern)/gos) ) {
    &util'msg($util'DEBUG_MSG,"$0: no matches");
    return;
  }
  $b1 = $2;
  $b2 = $3;

  $offset1 = length($`);

  $bytecount = unpack("C",$2)*256 + unpack("C",$3);

  if ($offset1 != 0) {
    &util'msg($util'ERROR_MSG,"$0: first pattern is a ccb at offset $offset1, not the begining of the buffer. File number=$seq_number");
    &saveERRORmessage($offset1);
    substr($input, 0, $offset1) = "";
  }

  if (length($input)<$bytecount) {
    &util'msg($util'DEBUG_MSG,"$0:Reached end of buffer");
    return;
  }

  $msgtext = substr($input,$ccb_len,$bytecount-$ccb_len);

  $offset2 = 0;
  if ($msgtext =~ /$ccb_pattern/) {
    $offset2 = length($`);
    &util'msg($util'ERROR_MSG,"$0: CCB inside the data at $offset2, bytecount=$bytecount. File number:$seq_number");
    &saveERRORmessage($bytecount);
  #  substr($input, 0, $offset2+$ccb_len) = "";
  #  next;
  }
  
  if (!(substr($msgtext,$bytecount-$eom_len-$ccb_len,$eom_len) =~ /$eom_pattern/)) {
    &util'msg($util'ERROR_MSG,"$0: CCB does not have a matching EOM. File number:$seq_number");
    &saveERRORmessage($ccb_len+$offset2);
    substr($input, 0, $ccb_len+$offset2) = "";
    next;
  }


  # I must have a good message now.. save it off
  $valid_ccb_messages++;
  $ccb_received_byte_count = $ccb_received_byte_count + $bytecount;
  &saveWMOmessage($msgtext);
  substr($input,0,$bytecount) = "";

 }

}

sub process_buffer_by_CCB {
  my ($str1, $str2, $offset1, $offset2);

  while (1) {

  if (!($input =~ /($ccb_pattern)|($eom_pattern)/gom) ) {
    &util'msg($util'DEBUG_MSG,"$0:no matches");
    return;
  }
  $str1 = $&;
  $offset1 = length($`);
#  $offset1 = pos();
  if ($str1 =~ /$eom_pattern/) {
    &util'msg($util'ERROR_MSG,"$0: first pattern is an eom not a ccb");
    &saveERRORmessage($offset1 + $eom_len);
    substr($input, 0, $offset1 + $eom_len) = "";
    next;
  }
  if ($offset1 != 0) {
    &util'msg($util'ERROR_MSG,"$0: first pattern is a ccb, but is not at the begining of the buffer");
    &saveERRORmessage($offset1);
    substr($input, 0, $offset1) = "";
  }
  if (!($input =~ /($ccb_pattern)|($eom_pattern)/gom) ) {
    &util'msg($util'DEBUG_MSG,"$0: Reached the end of the Buffer");
    return;
  }
  $str2 = $&;
  $offset2 = length($`);
#  $offset2 = pos();
  if ($str2 =~ /$ccb_pattern/) {
    &util'msg($util'ERROR_MSG,"$0: two ccb's with no eom inbetween");
    &saveERRORmessage($offset2);
    substr($input, 0, $offset2) = "";
    next;
  }

  # I must have a good message now.. save it off
  $valid_ccb_messages++;
  $ccb_received_byte_count = $ccb_received_byte_count + ($offset2+$eom_len - $ccb_len);
  &saveWMOmessage(substr($input,$ccb_len,$offset2 + $eom_len - $ccb_len));
  substr($input,0,$offset2+$eom_len) = "";

 }
}

sub saveERRORmessage {
  my($mlen) = @_;

  my($fname) = "$err_dir/x224.error." . time . ".$seq_number";
  &util'msg($util'DEBUG_MSG, "$0: Saving discarded data in file: $fname");
  open (BADDATA, ">$fname");
  print BADDATA substr($input,0,$mlen);
  close (BADDATA);
#  substr($input, 0, $mlen) = "";
 
  $seq_number++; 
  return;
}


sub saveWMOmessage {
  my ($msg) = @_;
  my ($subtype,$ssubtype);
  my ($msgcode,$sitecode,$wmo_msg_fn,$wmo_msg);
  my($data) =&util'build_timestamped_dataque;
# desegmentation variables
  my ($prod_date, $prod_seg, $prod_seg_n, $deseg_wlist, @fs_stat, $fs_mtime_delta, @deseg_time);
  my ($deseg_msg, $deseg_msg_fn, @deseg_wlist_mem, $deseg_pat, $deseg_i, $deseg_j, @deseg_dup);
  my ($deseg_saveA, $deseg_saveB, $deseg_head, $deseg_eol, $deseg_itb, $deseg_eom);

#    &util'msg($util'DEBUG_MSG, "Message= $msg" );
    if (! ($msg =~ /^$wmo_pattern/)) {
      my($fname) = "$err_dir/wmo.error." . time . ".$seq_number";
      &util'msg($util'ERROR_MSG,"$0: WMO Header not found in msg. Data saved in $fname" );
      open (BADDATA, ">$fname");
      print BADDATA $msg;
      close (BADDATA);
      $seq_number++; $invalid_wmo_msg++;
      return;
    }
    $deseg_saveA=$1; $deseg_saveB=$2;    # need them for desegmentation
    $subtype=$1 . $2 . $4 ;
    $ssubtype = $6 . $7;
    $prod_date = $3; $prod_seg = $5;

    $valid_wmo_messages++;
    
    if (!($msgcode=&util'parse("msgcode", "$parse_table",  "$type",  "$subtype",  "$ssubtype"))) {
      &util'msg($util'ERROR_MSG,"$0: parse(msgcode, ${parse_table}, ${type},  ${subtype},  $ssubtype) failed!");
      return; 
    }

    if (!($sitecode=&util'parse("sitecode", "$parse_table",  "HOST", "host",  "$hostaddr"))) {
      &util'msg($util'ERROR_MSG,"$0: parse(sitecode, ${parse_table}, HOST, host, ${hostaddr}) failed!");
      return;
    }

    $wmo_msg_fn=&util'build_fn("$type", "$sitecode", "$msgcode","0");
    $wmo_msg="${data}/${wmo_msg_fn}";
  
    if (!open(WMO, "> $wmo_msg")) {
      &util'msg($util'ERROR_MSG,"$0: open $wmo_msg failed: $!");
      return;
    }

#
#   desegmentation processing - PART I
#
#    &util'msg($util'DEBUG_MSG, "$0: type=$type, subtype=$subtype ssubtype=$ssubtype prod_date=$prod_date prod_seg=$prod_seg $&" );
    if ( $prod_seg ) {
      # determine and valid working list filename
      $prod_date = 999999 if ! defined($prod_date);
      $deseg_wlist = "DESEG.X224.$type.$subtype.$prod_date";
      if ( @fs_stat=stat("$tmp_dir/$deseg_wlist") ) {
        $fs_mtime_delta=(time - $fs_stat[9]) / 60; 
        if ($fs_mtime_delta > $x224_max_deseg_wlist_age) {
          &util'msg($util'WARN_MSG, "$0: $deseg_wlist exceeded legal age: $fs_mtime_delta > $x224_max_deseg_wlist_age" );
          &util'dbn_rename("$tmp_dir/$deseg_wlist", "$err_dir/$deseg_wlist");
        }
      } 

      # append new segment info to working list
      if (!open(DES_WLIST, "+>> $tmp_dir/$deseg_wlist")) {
        &util'msg($util'ERROR_MSG,"$0: open $tmp_dir/$deseg_wlist failed: $!");
        return;
      }
      @deseg_time=gmtime(time);
      printf (DES_WLIST "%s %s %02d%02d%02d%02d%02d\n", $prod_seg, $wmo_msg, ++$deseg_time[4], $deseg_time[3], $deseg_time[2], $deseg_time[1], $deseg_time[0]);
      # We will keep the DES_WLIST file handle open,
      # use it more later.
      #
      # debug - Before fiddling with the contents of message, lets save an unadultered
      # copy into the data queue.
      if (!open(PREDESEG, "> $wmo_msg.PREDESEG")) {
        &util'msg($util'ERROR_MSG,"$0: open $wmo_msg.PREDESEG failed: $!");
        return;
      }
      print PREDESEG $msg;
      close PREDESEG || &util'msg($util'ERROR_MSG,"$0: close $wmo_msg.PREDESEG failed: $!");
      # example - the pattern below resolves to :
      # '^(SXUS41[ ]+KTIR[ ]+.*?)(?:P[A-Z][A-Z][ ]*(\\015(?:\\015)*\\012))(?:(?:(?:(\\w{5}[\\w ])\\015)|(METAR))[ ]*\\015(?:\\015)*\\012)?(.*)(\\015\\015\\012\\003)$' 
      $deseg_pat = "^($deseg_saveA" . '[ ]+' . "$deseg_saveB" . '[ ]+.*?)(?:' . "$wmo_seg" . "[ ]*($wmo_eol))(?:$wmo_line2" . "[ ]*$wmo_eol)?(.*)(\\015$eom_pattern)\$";
      ($deseg_head, $deseg_eol, $deseg_i, $deseg_j, $deseg_itb, $deseg_eom) = $msg =~ /$deseg_pat/s;
      # The next line of code removes cr cr lf from the end of the ITB. 
      # This is being done to accomandate AWIPS segmentation which
      # places double cr cr lf sequence followed by ETX. The OSO gateway
      # only places a single cr cr lf followed by ETX. 
      $deseg_itb = $1 if $deseg_itb =~ /(.*)\r\r\n$/s;
      if ( $prod_seg =~ /^PAA$/ ) {                 # first segment
        if ( $deseg_i || $deseg_j ) {
          $msg = "$deseg_head$deseg_eol$deseg_i$deseg_j$deseg_eol$deseg_itb";
        }
        else {
          $msg = "$deseg_head$deseg_eol$deseg_itb";
        }
      }
      elsif ( $prod_seg =~ /^P[^Z][A-Z]$/ ) {       # ITB segment
        $msg = "$deseg_itb";
      }
      else  {                                       # last segment
        $msg = "$deseg_itb$deseg_eom";
      }
    }
#
#   end desegmentation processing - PART I
#

    print WMO $msg;

    close WMO || &util'msg($util'ERROR_MSG,"$0: close wmo_msg $wmo_msg failed: $!");
    &util'msg($util'INFO_MSG,"$0: created wmo msg ty=$type, subty=$subtype, ssubty=$ssubtype: $wmo_msg");


#
#   desegmentation processing - PART II
#
    if ( $prod_seg ) {
      # We could figure out if a piece is missing, but
      # we have been told that the pieces should always arrive in order. 
      # So, let's assume that this is the case, for now.
      # First order of business, get out if not the last segment
      if ( $prod_seg =~ /^P[^Z][A-Z]/ ) {
         close DES_WLIST || &util'msg($util'ERROR_MSG,"$0: deseg (PZ.): close $tmp/$deseg_wlist failed: $!");
         return;
      }
      # move the whence to the start of the file and slurp it up
      if (!seek(DES_WLIST, 0, 0)) {
        &util'msg($util'ERROR_MSG,"$0: deseg (PZ.): seek $tmp_dir/$deseg_wlist failed: $!");
        close DES_WLIST || &util'msg($util'ERROR_MSG,"$0: deseg (PZ.): close $tmp/$deseg_wlist failed: $!");
        &util'dbn_rename("$tmp_dir/$deseg_wlist", "$err_dir/$deseg_wlist");
        return;
      }
      while (<DES_WLIST>) {
        chomp;
        push(@deseg_wlist_mem, $_);
      } 
      close DES_WLIST || &util'msg($util'ERROR_MSG,"$0: deseg (PZ.): close $tmp/$deseg_wlist failed: $!");
      # get rid of the dups
      @deseg_dup = ();
      for ($deseg_i=0 ; $deseg_i < @deseg_wlist_mem; $deseg_i++) {
        ($prod_seg, $deseg_msg_fn) = split(/ /, $deseg_wlist_mem[$deseg_i], 2);  
        for ( $deseg_j=$deseg_i+1; $deseg_j < @deseg_wlist_mem; $deseg_j++ ) {
          ($prod_seg_n, $deseg_msg_fn) = split(/ /, $deseg_wlist_mem[$deseg_j], 2);
          if ( $prod_seg eq $prod_seg_n ) {
            unshift (@deseg_dup, $deseg_i);
            $deseg_j = @deseg_wlist_mem;
          }
        } 
      }
      while (@deseg_dup) {
        $prod_seg_n = shift(@deseg_dup);
        splice( @deseg_wlist_mem, $prod_seg_n, 1);        
      }
      # start of change to handle segmented products from awips:
      # we need to build the fn with the
      # message code of the very first segment. we have to do this 
      # becuase messages are starting to come from awips that do 
      # not include the first and second line of the WMO message in
      # all segments, the 2nd line of the WMO is only the first segment. 12/9/99 Lou.

      ($msgcode) = $deseg_wlist_mem[0] =~ /.*\.(\w+)(\.\d+){4}\s+\d+$/o; 
      # end of change to handle segmented products from awips.

      # create a new file to append all of the segments together.
      $wmo_msg_fn=&util'build_fn("$type", "$sitecode", "$msgcode","0");
      $wmo_msg="${data}/${wmo_msg_fn}";
      if (!open(WMO, "> $wmo_msg")) {
        &util'msg($util'ERROR_MSG,"$0: deseg (PZ.): open $wmo_msg failed: $!");
        return;
      }
      while ($deseg_msg_fn=shift(@deseg_wlist_mem)) {
        &util'msg($util'DEBUG_MSG,"$0: deseg (PZ.): processing $deseg_msg_fn");
        ($prod_seg, $deseg_msg_fn, $deseg_i) = split(' ', $deseg_msg_fn, 3);
        if (!open(DESEG, "$deseg_msg_fn")) {
          &util'msg($util'ERROR_MSG,"$0: deseg (PZ.): open $deseg_msg_fn failed: $!");
          return;
        }
        while (<DESEG>) {
          print WMO;
        }
        close DESEG || &util'msg($util'ERROR_MSG,"$0: deseg (PZ.): close $deseg_msg_fn failed: $!"); 
      }
      &util'dbn_rename("$tmp_dir/$deseg_wlist", "$data/$deseg_wlist");  
    }        
          
#
#   end of desegmentation processing - PART II
#

    if (!(@cmd_list=&util'parse("quecmd", "$parse_table", "$quecmd", "$type", "${sitecode}${msgcode}"))) {
      &util'msg($util'ERROR_MSG,"$0: parse(quecmd, $parse_table, $quecmd, $type, $sitecode$msgcode) failed!");
      &util'dbn_rename("$wmo_msg", "${err_dir}/${wmo_msg_fn}");
      return;
    }

    if (!&util'quecmd(*cmd_list, $dbnet_que, $wmo_msg, $wmo_msg_fn, $parse_table, $msgcode, $sitecode)) {
      &util'msg($util'ERROR_MSG,"$0: quecmd(cmd_list) failed");
      &util'dbn_rename("$wmo_msg", "${err_dir}/${wmo_msg_fn}");
      return;
    }
}

sub exit_hdlr {
  &util'msg($util'ERROR_MSG,"$0: exit");
  exit 1;
}

