#-----------------------------------------------------------
# read in directory config file
# config file format:
# dirname  filepattern  TYPE SubTYPE
#    * filepattern is a regular expression
#
# read in list from last time, perform error checking
#
# get new list of files
# delete old files
# make list of completed files
# generate alerts
# make list of files to check next time
# all done......
# init the necessary vars,  this will be deleted when it is part of dbn_qh.pl
#-----------------------------------------------------------

########################################################
# read in the list of completed files for the disk
########################################################
sub scan_dir {
  my($fname,$mtime,$size,@scan_list,$dirpattern,$filepattern,$type,$subtype);
  my($dirname,@dirlist,@newfiles,$f,$dev,$ino,$mode,$nlink,$uid,$gid,$rdev);
  my($atime,$ctime,$blksize,$blocks,$fs,$ft,$ftype,$stats,@files_deleted);
  my(%current,%completed,$legal_age,$curr_time,$jobname);
  my($stored_pattern) = "^\\S+[ ]+\\d+[ ]+\\d+";

  &util'msg($util'INFO_MSG,"$0: FIle scan starting");

  if (!open(COMPLETED, "< ${scan_table_completed}")) {
    &util'msg($util'ERROR_MSG,"$0: open (COMPLETED,<${scan_table_completed}) failed. $!");
#    return 1;
  } else {
    while (<COMPLETED>) {
      if ( /"^#"/ ) { next };
      if ( ! /$stored_pattern/ ) {
        &util'msg($util'ERROR_MSG,"$0: Invalid Format in $scan_table_completed file, line=$_!");
	next;
#        close(COMPLETED);
#        return 1;
      }
      ($fname,$mtime,$size) = split;
      $completed{$fname} = "${mtime}:${size}";
    }

    if ( !close(COMPLETED) ) {
      &util'msg($util'ERROR_MSG,"$0: close (COMPLETED) failed. $! during init");
#      return 1;
    }
  }

########################################################
# read in the list of current in process files from disk
########################################################

  if (!open(CURRENT, "< ${scan_table_current}")) {
    &util'msg($util'ERROR_MSG,"$0: open (CURRENT,<${scan_table_current}) failed. $!");
#    return 1;
  } else  {
    while (<CURRENT>) {
      if ( ! /$stored_pattern/ ) {
        &util'msg($util'ERROR_MSG,"$0: Invalid Format in $scan_table_current file, line=$_!");
	next;
#        close(CURRENT);
#        return 1;
      }

      ($fname,$mtime,$size) = split;
      $current{$fname} = "${mtime}:${size}";
    }

    if ( !close(CURRENT) ) {
      &util'msg($util'ERROR_MSG,"$0: close (CURRENT) failed. $! during init");
#      return 1;
    }
  }

  $scan_pattern = "/\\S+[ \\t]+\\S+[ \\t]+\\S+[ \\t]+\\S+[ \\t]+\\S+[ \\t]+\\d+";

########################################################
# read in the scan table of files to look for
########################################################

  if (!open(SCAN, "< ${scan_table}")) {
    &util'msg($util'ERROR_MSG,"$0: open (SCAN,<${scan_table}) failed. $!");
    return 1;
  }

  undef @scan_list;

  while (<SCAN>) {
    push(@scan_list,$_) if /$scan_pattern/;
  }

  if ( !close(SCAN) ) {
    &util'msg($util'ERROR_MSG,"$0: close (SCAN) failed. $!");
#    return 1;
  }

########################################################
# build a list of the matching files in each directory
########################################################
  $curr_time = time;

  foreach (@scan_list) {
    ($dirpattern, $filepattern, $type, $subtype, $jobname,  $legal_age) = split(/[ \t]+/,$_);
    chomp $legal_age;

    @dirlist = glob($dirpattern);

    foreach $dirname (@dirlist) {
      next if !-d $dirname;
      if (!opendir(DIR, $dirname)) {
        &util'msg($util'ERROR_MSG,"$0: opendir($dirname) failed: $!");
        next;
      }

      @newfiles=readdir(DIR);
      closedir(DIR);
  
      foreach $f (@newfiles) {
	$fname = "$dirname/$f";
	next if !-f $fname;
        if ($fname =~ /$filepattern/) {
          ########################################################
          # now check the time/size of each file, to see if it is 
          # different from what I have in the current list. If the
          # time/size match and the time is younger $legal_age then 
	  # generated an ALERT. if the file is not in the current 
	  # list add it, if the file is in the completed list, 
	  # skip it.
          ########################################################

	  # do a stat, if the stat fails, delete the file name 
	  # from current and continue to the next file.
          if (! (($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
		  $atime,$mtime,$ctime,$blksize,$blocks) = stat($fname))) {
	     &util'msg($util'WARN_MSG,"$0: stat failed on file $fname");
	     delete $current{$fname};
	     next;
	  }
         
          if (exists $completed{$fname}) {
             ($ft,$fs) = split(/:/,$completed{$fname});
              if (($ft != $mtime) || ( $fs != $size)) { 
                delete $completed{$fname};
              } else {
               next ;
              }
          }

          #next if (defined ($completed{$fname}) );

	  if ( ! exists ($current{$fname}) ) {
            $current{$fname} = "${mtime}:${size}";
            next;
          }
    
          ($ft,$fs) = split(/:/,$current{$fname});
  
          if (($ft == $mtime) && ( $fs == $size)) {
            if (($legal_age != 0) && ($curr_time < ($legal_age * 60 + $ft))) {
	      system("$dbn_root/bin/dbn_alert.pl", $type, $subtype, $jobname, $fname);
            } else {
              &util'msg($util'WARN_MSG,"$0: File $fname is outside legal age for SCANNER");
	    }
	    delete $current{$fname};
            $completed{$fname} = "${mtime}:${size}";
          }
          else {
            $current{$fname} = "${mtime}:${size}";
          }
        } #if filepattern && !-d
      } # foreach newfiles
    }
  }

  ########################################################
  # now clean up the completed array.  Delete any entries in which the
  # file no longer exists.  The two loops below could probably be combined
  # into to one loop, but I don't like changeing an array as I am 
  # interating over it.
  ########################################################

  while (($fname,$stats) = each %completed) {
    if (! -e $fname ) {push(@files_deleted,$fname);}
  }

  foreach (@files_deleted) {
    delete $completed{$_};
  }

  if (!open(COMPLETED, "> ${scan_table_completed}")) {
    &util'msg($util'ERROR_MSG,"$0: open (COMPLETED,<${scan_table_completed}) failed. $!");
    return 1;
  }

  while (($fname,$stats) = each %completed) {
    ($mtime,$size) = split(/:/,$stats);
    print COMPLETED "$fname  $mtime  $size\n";
  }

  if ( !close(COMPLETED) ) {
    &util'msg($util'ERROR_MSG,"$0: close (COMPLETED) failed. $!");
    return 1;
  }

  ############################################################################
  # clean up the current list, delete any files that no longer
  # exist.  This may not happen vary often but, better safe the
  # sorry.  
  ###########################################################################

  undef @files_deleted;
  while (($fname,$stats) = each %current) {
    if (! -e $fname ) {push(@files_deleted,$fname);}
  }

  foreach (@files_deleted) {
    delete $current{$_};
  }

  if (!open(CURRENT, "> ${scan_table_current}")) {
    &util'msg($util'ERROR_MSG,"$0: open (CURRENT,<${scan_table_current}) failed. $!");
    return 1;
  }

  while (($fname,$stats) = each %current) {
    ($mtime,$size) = split(/:/,$stats);
    print CURRENT "$fname  $mtime  $size\n";
  }

  if ( !close(CURRENT) ) {
    &util'msg($util'ERROR_MSG,"$0: close (CURRENT) failed. $!");
    return 1;
  }

  &util'msg($util'INFO_MSG,"$0: File scan completed successfully");

  return 0;

}
1;
