#!/bin/sh -- # perl using gnu/perl
eval 'exec ${DBNROOT}/bin/perl $0 ${1+"$@"}'
  if 0;
##
# fsl_ruc2_surface_slink.pl
#
#
# author: Luis Cano
# date:   10/07/99
#
#
#

$dbn_root = $ENV{'DBNROOT'};

if (!defined($dbn_root) || !$dbn_root) {
  print STDERR "\n$0: environment variable DBNROOT not set!\n";
  exit 1;
}

my ($old_path) = $ENV{PATH};
$old_path =~ /(.*)/;  # really bad untainting
$ENV{PATH} = $1;

$dbn_config="${dbn_root}/table/dbn_config";
$dbn_config =~ /(.*)/;  # really bad untainting
require "$1";

unshift(@INC, '.');
unshift(@INC, "$include_dir");

require 'dbn_util.pl';

local($orig_path, $ddir, $sdir, $sfn, $gribdate, $ruc2type, $type, $stype, $jobname);
local($gyear, $gmonth, $gday, $gcycle, $gfhour, $gcentury );

if (!&util'init()) {
  &util'msg($util'ERROR_MSG,"$0: init() failed!");
  exit 1;
}

if (@ARGV != 6) {
  &util'msg($util'ERROR_MSG,"usage: $0 fsl_ruc2_gribpath rename_ruc2_dest_dir isobaric|native|surface type subtype jobname");
  exit 1;
}

$orig_path=shift;
$ddir=shift;
$ruc2type=shift;
$type=shift;
$stype=shift;
$jobname=shift;

if    ( $ruc2type  eq "native" ) { 
  $ruc2type="bgrb";
}
elsif ( $ruc2type  eq "isobaric" ) {
  $ruc2type="pgrb";
}
elsif ( $ruc2type  eq "surface" ) {
  $ruc2type="sgrb";
}
else {
  &util'msg($util'ERROR_MSG,"usage: $0 fsl_ruc2_gribpath rename_ruc2_dest_dir isobaric|native|surface type subtype jobname");
  &util'msg($util'ERROR_MSG,"usage: invalid: ruc2type=$ruc2type");
  exit 1;
}


&util'msg($util'INFO_MSG,"$0: entry $orig_path $ddir $ruc2type $type $stype $jobname");

$gribdate=`$dbn_root/user/gribdate $orig_path`;

if ( $? != 0) {
  util'msg($util'ERROR_MSG,"$0: can't execute $dbn_root/user/gribdate $orig_path: $?");
  exit 1; 
} 

if ( $gribdate !~ /^([\d]+)\ ([\d]+)\ ([\d]+)\ ([\d]+)\ [\d]+\ [\d]+\ ([\d]+)\ [\d]+\ [\d]+\ ([\d]+)/ ) {
  &util'msg($util'ERROR_MSG,"$0: cannot determine gribdate: $orig_path");
  exit 1;
}

$gyear=$1; $gmonth=$2; $gday=$3; $gcycle=$4; $gfhour=$5; $gcentury=$6;

if ( $gyear eq "100" ) {
  $gyear = substr($gyear, 1, 2);
}

if ( $gcentury > 20 ) {
  $gcentury = $gcentury - 1;
}

&util'msg($util'INFO_MSG, "Grib Info: year=$gyear, month=$gmonth, day=$gday, cycle=$cycle, fhour=$gfhour, century=$gcentury");
if ( $gfhour == 0 ) {
  $gfhour = "anl";
}
else {
  $gfhour = "f" . $gfhour;
}

$sdir="${ddir}/ruc2a.${gcentury}${gyear}${gmonth}${gday}";
$sfn="ruc2.t${gcycle}z.${ruc2type}${gfhour}";

&util'msg($util'INFO_MSG,"$0: sdir=$sdir sfn=$sfn");

if (! &util'mkdirs($sdir)) {
  &util'msg($util'ERROR_MSG,"$0: cannot create dirs $sdir");
  exit 1;
}

if ( -l "${sdir}/${sfn}" ) {
  &util'msg($util'WARN_MSG,"$0: ${sdir}/${sfn} exits, deleting it");
  if ( ! unlink("${sdir}/${sfn}") ) {
    &util'msg($util'ERROR_MSG,"$0: cannot unlink symlink ${sdir}/${sfn}: $!");
    exit 1;
  }
}
 

if (! symlink($orig_path, "${sdir}/${sfn}") ) {
  &util'msg($util'ERROR_MSG,"$0: cannot symlink $orig_path ${sdir}/${sfn}: $!");
  exit 1;
}

system "$dbn_root/bin/dbn_alert.pl", $type, $stype, $jobname, "${sdir}/${sfn}";

exit 0;
