#!/bin/csh
#########################################################################
# send_awips
#
# This script sends a product to the AWIPS WAN for distribution.
#
# Inputs:
#	$1	Full path for file
#	$2	10 character AWIPS ID
#	$3	Type of product
#		    Valid types are:
#			text[_list]	(handleOUP.pl)
#					(distributeProduct w/list)
#			graphic		(msg_send)
#			grib		(msg_send)
#			vgf[_xx]	(ftp script)
#			work		(textdb)
#
##
# Log:
# S. Danz	 3/00	Created
# S. Jacobs	 3/00	Cleaned up for delivery
# S. Jacobs	 4/00	Added processing of vgf files
# S. Jacobs	 5/00	Added option for text sent to a list of sites
# S. Jacobs	 5/00	Added work keyword
# S. Jacobs	 5/00	Removed grib keyword
# S. Jacobs	 8/00	Added chmod and chown for product files
# S. Jacobs	 9/00	Added grib keyword
#########################################################################

# Sleep to make sure that the file has been written.
sleep 5

# Make sure that the permissions and owner are set properly for 
# the message handling system.
# chmod 775 $1 >&! /dev/null
# chown fxa $1 >&! /dev/null

# Split the type into a keyword and an extension
set keyword = `echo $3 | awk -F_ '{print $1}'`
set exten   = `echo $3 | awk -F_ '{print $2}'`

# Process the given keyword.
if  ( $keyword == "text" )  then

    if  ( $exten == "" )  then
	# Process text products that are sent to the DEFAULTNCF.

	handleOUP.pl $2 $1 >& /dev/null

    else
	# Process text products with an addressee list.

	set afosid = `getAFOSid $2`
	set class = "AFOS_STORE_TEXTDB"

	distributeProduct -s $afosid -a $exten -c $class $2 $1 >& /dev/null

    endif

else if  ( $keyword == "work" )  then
    # Process the work files.

    textdb -w $2 < $1

else if  ( $keyword == "graphic" )  then
    # Process the graphic products.

    set class = 0

    # Get the WMO ID info from the file
    set hdr   = "`od -A n -j 24 -N 6 -t c $1 | sed 's/[ 	]*//g'`"
    set site  = "`od -A n -j 31 -N 4 -t c $1 | sed 's/[ 	]*//g'`"
    set time  = "`od -A n -j 36 -N 6 -t c $1 | sed 's/[ 	]*//g'`"

    msg_send -e $1 -i "$hdr $site $time" -c $class -a defaultncf

else if  ( $keyword == "grib" )  then
    # Process the grib products.

    set class = 0

    # Get the WMO ID info from the file
    set hdr   = "`od -A n -N 6 -t c $1 | sed 's/[ 	]*//g'`"
    set site  = "`od -A n -j  7 -N 4 -t c $1 | sed 's/[ 	]*//g'`"
    set time  = "`od -A n -j 12 -N 6 -t c $1 | sed 's/[ 	]*//g'`"

    msg_send -e $1 -i "$hdr $site $time" -c $class -a defaultncf

else if  ( $keyword == "vgf" )  then
    # Process the VGF files.
    # Call a script to ftp the files to the RFCs.

    if  ( $exten == "" )  then
	set type = `echo "${keyword}" | tr "[a-z]" "[A-Z]"`
    else
	set type = `echo "${keyword}${exten}" | tr "[a-z]" "[A-Z]"`
    endif

    set logdate = `date -u "+%Y%m%d%H"`

    ftp_vg_hpc2rfc $type $1 >>& $HOME/logs/${logdate}_$$.ftpvgf

endif

# Expire the product from the LDM product queue,
# in case there is a resend.
pqexpire -q $LDMHOME/data/ldm.pq -p ${2}':'${3} -i 0 -a .001
