#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

#define BUFSIZE 4095

int main(int argc, char *argv[]) {

  int n, fd;
  int i = BUFSIZE;
  int j = 0;
  int state;

  char buf[BUFSIZE+1]; 

  if ( argc != 2 ) {
    fprintf(stderr, "usage: gribdate gribfile\n");
    fprintf(stderr, "PDS:   Year Month Day HourOfDay MinuteOfHour FsctTimeUnit\n");
    fprintf(stderr, "PDS:   P1 P2 TimeRangeIndicator Century\n");
    exit(1);
  }

  if ( (fd = open(argv[1], O_RDONLY, 0)) == -1) {
    fprintf(stderr, "gribdate: cannot open %s: %s\n", argv[1], strerror(errno)); 
    exit(1); 
  }
 
  while  ( ((n = read(fd, buf+j, i)) > 0) && i > 0 )  {
    j += n;
    i -= n;
  } 

  if ( j < 40 ) {
    fprintf(stderr, "gribdate: %s not at least 40 bytes\n", argv[1]);
    close(fd);
    exit(1);
  } 

  close(fd);
  
  for (i = 0; i < j ; i++ ) {
/*
 *  state 0 = Looking for G
 *  state 1 = Looking for R
 *  state 2 = Looking for I
 *  state 3 = Looking for B
 *  state 4 = Process date
*/
    switch (state) {
      case 0: 
        if ( buf[i] == 'G' ) 
          state++;
          break;
      case 1:
        if ( buf[i] == 'R' ) 
          state++;
        else
          state=0; 
        break;
      case 2: 
        if ( buf[i] == 'I' ) 
          state++; 
        else
          state=0;
        break;
      case 3: 
        if ( buf[i] == 'B' ) 
          state++; 
        else
          state=0;
        break;
      case 4: 
        if ( (j - i) < 40 ) { 
          fprintf(stderr, "gribdate: found GRIB in %s, but remainder file not at least 40 bytes\n", argv[1]);
          exit(1);
        }
/*
 * Year Month Day Cycle ForecastHour Century
 */
        fprintf(stdout, "%02d %02d %02d %02d %02d %02d %02d %02d %02d %02d\n", 
                                  buf[i+16],
                                  buf[i+17],
                                  buf[i+18],
                                  buf[i+19],
                                  buf[i+20],
                                  buf[i+21],
                                  buf[i+22],
                                  buf[i+23],
                                  buf[i+24],
                                  buf[i+28] );
         exit(0);
    }
  }
  fprintf(stderr, "gribdate: did not find GRIB in %s\n", argv[1]);
  exit(1); 
}
