#define _POSIX_SOURCE 1 

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <dirent.h>
#include <unistd.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>
#include "../util/support.h"

#define HDRSZ 11
#define CSNOFF 4
#define CSNLEN 3
#define TRAILERSZ 7
#define SOH 0x01
#define CR  0x0d
#define LF  0x0a
#define SP  0x20
#define RS  0x1e
#define ETX 0x03
#define NIL 0x00
#define EQ '='
#define EOB -2
#define TRUE 1
#define FALSE 0
#define DBNROOT_ENV "DBNROOT"


int main (int argc, char *argv[] ) {

  char modname[32]="dbn_ldm";

  static char *dbn_root=NULL;
  static char default_dbn_root[]="/usr/DBNet";

  extern int errno;

  FILE *fds, *fps;
  unsigned char *msg;
  int pid, pipefds[2], fd;
  struct stat sbuf; 
  unsigned i, xoff;
  static unsigned csn=0;

  unsigned char hdr[]={ SOH, CR, CR, LF, '0', '0', '0', SP, CR, CR, LF };
  const unsigned char trailer[]={ CR, CR, LF, CR, CR, LF, ETX };

  DIR *my_que;
  struct dirent *this_entry;
  struct stat status;
  
  static logbfrT logbfr;
  static char logstr[MAX_LOG_MSG_LEN+1];
  static char loggerpath[MAX_PATH+1];
  static char logstage[2048];
  const char logprefix[]="/log/dbn_ldm.log.";

  logstr[0]=0x0;
  logbfr.str=logstr;
  logbfr.str_sz=MAX_LOG_MSG_LEN;

/*
 * setup dbn_root
 */
  dbn_root=getenv(DBNROOT_ENV);
  if (dbn_root==NULL) {
    (void)fprintf(stderr, "%s environment variable %s not set!\n", modname, DBNROOT_ENV); 
    exit(EXIT_FAILURE);
  }

/*
 * setup logger path
 */
  (void)strcpy(loggerpath, dbn_root);
  (void)strcat(loggerpath, (*(dbn_root+strlen(dbn_root)-1)=='/') ? logprefix+1 : logprefix);


  if (argc<4) {
    fprintf(stderr,"\nusage: %s pqing_path feed_type dbnet_ldm_que\n", argv[0]);
    exit(EXIT_FAILURE);
  }

  strcpy(modname, argv[2]);
  sprintf(logstage, "%s %s %s", argv[1], argv[2], argv[3]);
  (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
           logstage, (int *)NULL,
           (char *)NULL, (int *)NULL,
           (char *)NULL, (int *)NULL,
           (char *)NULL, (int *)NULL,
           &logbfr );
  utilLogger(loggerpath, modname, 0, &logbfr, MAX_LOG_MSG_LEN); 

  if (pipe(pipefds)<0) {
    (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
             "pipe(pipefds) failed: ", (int *)NULL,
             strerror(errno), (int *)NULL,
             (char *)NULL, (int *)NULL,
             (char *)NULL, (int *)NULL,
             &logbfr );
    utilLogger(loggerpath, modname, errno, &logbfr, 0);
    exit(EXIT_FAILURE);
  }

  if ((pid=fork())<0) {
    (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
             "pid=fork() failed: ", (int *)NULL,
             strerror(errno), (int *)NULL,
             (char *)NULL, (int *)NULL,
             (char *)NULL, (int *)NULL,
             &logbfr );
    utilLogger(loggerpath, modname, errno, &logbfr, 0);
    exit(EXIT_FAILURE);
  }
  
  if (pid==0) {

    if (close(0)<0) {
      (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
               "close(0) failed: ", (int *)NULL,
               strerror(errno), (int *)NULL,
               (char *)NULL, (int *)NULL,
               (char *)NULL, (int *)NULL,
               &logbfr );
      utilLogger(loggerpath, "child process", errno, &logbfr, 0);
      exit(EXIT_FAILURE);
    }
    if (dup(pipefds[0])<0) {
      sprintf(logstage, "dup(pipefds[0]=%i) failed: ", pipefds[0]);
      (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
               logstage, (int *)NULL,
               strerror(errno), (int *)NULL,
               (char *)NULL, (int *)NULL,
               (char *)NULL, (int *)NULL,
               &logbfr );
      utilLogger(loggerpath, "child process", errno, &logbfr, 0);
      exit(EXIT_FAILURE);
    }
    if (close(pipefds[0])<0){
     sprintf(logstage, "close(pipefds[0]=%i) failed: ", pipefds[0]);
      (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
               logstage, (int *)NULL,
               strerror(errno), (int *)NULL,
               (char *)NULL, (int *)NULL,
               (char *)NULL, (int *)NULL,
               &logbfr );
      utilLogger(loggerpath, "child process", errno, &logbfr, 0);
      exit(EXIT_FAILURE);
    }
    if (close(pipefds[1])<0) {
      sprintf(logstage, "close(pipefds[1]=%i) failed: ", pipefds[1]);
      (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
               logstage, (int *)NULL,
               strerror(errno), (int *)NULL,
               (char *)NULL, (int *)NULL,
               (char *)NULL, (int *)NULL,
               &logbfr );
      utilLogger(loggerpath, "child process", errno, &logbfr, 0);
      exit(EXIT_FAILURE);
    }

    sprintf(logstage, "executing execlp(%s, %s, -v, %s, -, NULL)", 
                       argv[1], argv[1], argv[2]);
    (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
             logstage, (int *)NULL,
             (char *)NULL, (int *)NULL,
             (char *)NULL, (int *)NULL,
             (char *)NULL, (int *)NULL,
             &logbfr );
    utilLogger(loggerpath, "child process", 0, &logbfr, MAX_LOG_MSG_LEN);

    execlp(argv[1], argv[1], "-v", argv[2], "-", NULL);
      sprintf(logstage, "execlp(%s, %s, -v, %s, -, NULL) failed: ", 
                          argv[1], argv[1], argv[2]);
      (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL, 
               logstage, (int *)NULL,
               strerror(errno), (int *)NULL,
               (char *)NULL, (int *)NULL,
               (char *)NULL, (int *)NULL,
               &logbfr );
      utilLogger(loggerpath, "child process", errno, &logbfr, 0);
      exit(EXIT_FAILURE);
  }
  
   
  sleep(1);
  
  if (close(pipefds[0])<0) {
    sprintf(logstage, "close(pipefds[0]=%i) failed: ", pipefds[0]);
    (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
             logstage, (int *)NULL,
             strerror(errno), (int *)NULL,
             (char *)NULL, (int *)NULL,
             (char *)NULL, (int *)NULL,
             &logbfr );
    utilLogger(loggerpath, modname, errno, &logbfr, 0);
    exit(EXIT_FAILURE);
  }

  if ((fps=fdopen(pipefds[1], "w"))==NULL) {
    sprintf(logstage, "fps=fdopen(pipefds[1]=%i, w) failed: ", pipefds[1]);
    (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
             logstage, (int *)NULL,
             strerror(errno), (int *)NULL,
             (char *)NULL, (int *)NULL,
             (char *)NULL, (int *)NULL,
             &logbfr );
    utilLogger(loggerpath, modname, errno, &logbfr, 0);
    exit(EXIT_FAILURE);
  }

  if (chdir(argv[3]) != 0) {
    sprintf(logstage, "chdir(%s) failed: ", argv[3]);
    (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
             logstage, (int *)NULL,
             strerror(errno), (int *)NULL,
             (char *)NULL, (int *)NULL,
             (char *)NULL, (int *)NULL,
             &logbfr );
    utilLogger(loggerpath, modname, errno, &logbfr, 0);
    exit(EXIT_FAILURE);
  }


  while ( 1 ) {
    
    sleep (2);  

    my_que = opendir(argv[3]);
    if ( my_que == NULL ) {
      sprintf(logstage, "opendir(%s) failed: ", argv[3]);
      (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
               logstage, (int *)NULL,
               strerror(errno), (int *)NULL,
               (char *)NULL, (int *)NULL,
               (char *)NULL, (int *)NULL,
               &logbfr );
      utilLogger(loggerpath, modname, errno, &logbfr, 0);
      exit(EXIT_FAILURE);
    }
    
    while ( (this_entry = readdir(my_que)) != NULL ) {
      if ( (strcmp(this_entry->d_name, ".") == 0) ||
           (strcmp(this_entry->d_name, "..") == 0) ) 
        continue;  
      fds=NULL;
      msg=NULL;
    
      if ((fd=open(this_entry->d_name, O_RDONLY))<0) {
        sprintf(logstage, "fd=open(%s, O_RDONLY) failed: ",
                           this_entry->d_name);
        (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                 logstage, (int *)NULL,
                 strerror(errno), (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 &logbfr );
        utilLogger(loggerpath, modname, errno, &logbfr, MAX_LOG_MSG_LEN);
      }

      else if ((fds=fdopen(fd, "r"))==NULL) {
        sprintf(logstage, "%s: fds=fdopen(fd=%l, r) failed: ", 
                           this_entry->d_name, fd);
        (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                 logstage, (int *)NULL,
                 strerror(errno), (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 &logbfr );
        utilLogger(loggerpath, modname, errno, &logbfr, MAX_LOG_MSG_LEN);
      }

      else if (fstat(fd, &sbuf)<0) {
        sprintf(logstage, "%s: fstat(fd=%l, &sbuf) failed: ", 
                           this_entry->d_name, fd);
        (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                 logstage, (int *)NULL,
                 strerror(errno), (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 &logbfr );
        utilLogger(loggerpath, modname, errno, &logbfr, MAX_LOG_MSG_LEN);
      }

      else if ( sbuf.st_size < 10 ) {
        sprintf(logstage, "%s: sbuf.st_size=%i < 10  ", 
                           this_entry->d_name, sbuf.st_size);
        (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                 logstage, (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 &logbfr );
        utilLogger(loggerpath, modname, 0, &logbfr, MAX_LOG_MSG_LEN);
      }
     
      else if ((msg=(unsigned char *)malloc(sbuf.st_size))==NULL) {
        sprintf(logstage, "%s: malloc(sbuf.st_size=%i) failed: ", 
                           this_entry->d_name, sbuf.st_size);
        (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                 logstage, (int *)NULL,
                 strerror(errno), (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 &logbfr );
        utilLogger(loggerpath, modname, errno, &logbfr, 0);
        exit(EXIT_FAILURE);
      }

      else if (fread(msg, sbuf.st_size, 1, fds)<1) {
        sprintf(logstage, "%s: fread(msg=%p, sbuf.st_size=%i, 1, fds=%l) failed: ", 
                           this_entry->d_name, (void *)msg, sbuf.st_size, fds);
        (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                 logstage, (int *)NULL,
                 strerror(errno), (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 &logbfr );
        utilLogger(loggerpath, modname, errno, &logbfr, MAX_LOG_MSG_LEN);
      }
    
      else {
        csn=csn%1000;
        sprintf((char *)hdr+CSNOFF, "%03d", csn);
        *(hdr+CSNOFF+CSNLEN)=SP; 
        csn++;
        if (fwrite(hdr, HDRSZ, 1, fps)!=1) {
          sprintf(logstage, "%s: fwrite(hdr, HDRSZ, 1, fps=%l) failed: ", 
                             this_entry->d_name, fps);
          (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                   logstage, (int *)NULL,
                   strerror(errno), (int *)NULL,
                   (char *)NULL, (int *)NULL,
                   (char *)NULL, (int *)NULL,
                   &logbfr );
          utilLogger(loggerpath, modname, errno, &logbfr, 0);
          exit(EXIT_FAILURE);
        }
        for (xoff=sbuf.st_size; (xoff > 0)  &&
               (*(msg+xoff-1)==ETX ||
                *(msg+xoff-1)==CR  ||
                *(msg+xoff-1)==LF)    ; xoff--)
              ;
        if (xoff==0) {
          sprintf(logstage, "%s: xoff(%i)==0: invalid message ", 
                             this_entry->d_name, xoff);
          (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                   logstage, (int *)NULL,
                   (char *)NULL, (int *)NULL,
                   (char *)NULL, (int *)NULL,
                   (char *)NULL, (int *)NULL,
                   &logbfr );
          utilLogger(loggerpath, modname, 0, &logbfr, MAX_LOG_MSG_LEN);
        }
        else if (fwrite(msg, xoff, 1, fps)!=1) {
          sprintf(logstage, "%s: fwrite(msg=%p, xoff=%i, 1, fps=%l failed: ",
                             this_entry->d_name, (void *)msg, xoff, fps);
          (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                   logstage, (int *)NULL,
                   strerror(errno), (int *)NULL,
                   (char *)NULL, (int *)NULL,
                   (char *)NULL, (int *)NULL,
                   &logbfr );
          utilLogger(loggerpath, modname, errno, &logbfr, 0);
          exit(EXIT_FAILURE);
        }
        else if (fwrite(trailer, TRAILERSZ, 1, fps)!=1) {
          sprintf(logstage, "%s: fwrite(trailer, TRAILERSZ, 1, fps=%l failed: ",
                             this_entry->d_name, fps);
          (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                   logstage, (int *)NULL,
                   strerror(errno), (int *)NULL,
                   (char *)NULL, (int *)NULL,
                   (char *)NULL, (int *)NULL,
                   &logbfr );
          utilLogger(loggerpath, modname, errno, &logbfr, 0);
          exit(EXIT_FAILURE);
        }
        else {
          fflush(fps);
        }
        sprintf(logstage, "%s ", this_entry->d_name);
        xoff = strlen(logstage);
        for (i=0; i<20; i++) {
         logstage[xoff+i] = *(msg+i); 
         if (! isprint(logstage[xoff+i]))
           logstage[xoff+i] = ' ';
        }
        logstage[xoff+i] = 0x0; 
        (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                 logstage, (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 &logbfr );
        utilLogger(loggerpath, modname, 0, &logbfr, MAX_LOG_MSG_LEN);

        if (fds!=NULL && fclose(fds)==EOF) {
          sprintf(logstage, "%s: fclose(fds=%l) failed: ",
                             this_entry->d_name, fds);
          (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                   logstage, (int *)NULL,
                   strerror(errno), (int *)NULL,
                   (char *)NULL, (int *)NULL,
                   (char *)NULL, (int *)NULL,
                   &logbfr );
          utilLogger(loggerpath, modname, errno, &logbfr, MAX_LOG_MSG_LEN);
        }

        if (msg!=NULL) 
          free(msg);

      }
      if ( unlink(this_entry->d_name) != 0 ) {
        sprintf(logstage, "unlink(this_entry->d_name=%s) failed: ",
                           this_entry->d_name);
        (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
                 logstage, (int *)NULL,
                 strerror(errno), (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 (char *)NULL, (int *)NULL,
                 &logbfr );
        utilLogger(loggerpath, modname, errno, &logbfr, MAX_LOG_MSG_LEN);
        exit(EXIT_FAILURE); 
      }
    }
    if (closedir(my_que) != 0) {
      sprintf(logstage, "%s: %s:  closedir(my_que=%l) failed: ",
                         this_entry->d_name, argv[3], my_que);
      (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
               logstage, (int *)NULL,
               strerror(errno), (int *)NULL,
               (char *)NULL, (int *)NULL,
               (char *)NULL, (int *)NULL,
               &logbfr );
      utilLogger(loggerpath, modname, errno, &logbfr, 0);
      exit(EXIT_FAILURE);
    } 
  }
}
