#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>

#define BUFSIZE 4095

int main(int argc, char *argv[]) {

  int n, fd;
  int i = BUFSIZE;
  int j = 0;
  int state;

  char buf[BUFSIZE+1]; 

  if ( argc != 2 ) {
    fprintf(stderr, "usage: printPDS gribfile\n");
    fprintf(stderr, "From PDS:  octet 5  - Identification center\n");
    fprintf(stderr, "           octet 13 - Year of century\n");
    fprintf(stderr, "           octet 14 - Month of year\n");
    fprintf(stderr, "           octet 15 - Day of month\n");
    fprintf(stderr, "           octet 16 - Hour of day\n");
    fprintf(stderr, "           octet 17 - Minute of hour\n");
    fprintf(stderr, "           octet 18 - Forecast time unit\n");
    fprintf(stderr, "           octet 19 - P1\n");
    fprintf(stderr, "           octet 20 - P2\n");
    fprintf(stderr, "           octet 21 - Time range indicator\n");
    fprintf(stderr, "           octet 25 - Century of initial reference\n");
    fprintf(stderr, "           octet 26 - ID of sub-center\n");
    exit(1);
  }

  if ( (fd = open(argv[1], O_RDONLY, 0)) == -1) {
    fprintf(stderr, "gribdate: cannot open %s: %s\n", argv[1], strerror(errno)); 
    exit(1); 
  }
 
  while  ( ((n = read(fd, buf+j, i)) > 0) && i > 0 )  {
    j += n;
    i -= n;
  } 

  if ( j < 40 ) {
    fprintf(stderr, "gribdate: %s not at least 40 bytes\n", argv[1]);
    close(fd);
    exit(1);
  } 

  close(fd);
  
  for (i = 0; i < j ; i++ ) {
/*
 *  state 0 = Looking for G
 *  state 1 = Looking for R
 *  state 2 = Looking for I
 *  state 3 = Looking for B
 *  state 4 = Process date
*/
    switch (state) {
      case 0: 
        if ( buf[i] == 'G' ) 
          state++;
          break;
      case 1:
        if ( buf[i] == 'R' ) 
          state++;
        else
          state=0; 
        break;
      case 2: 
        if ( buf[i] == 'I' ) 
          state++; 
        else
          state=0;
        break;
      case 3: 
        if ( buf[i] == 'B' ) 
          state++; 
        else
          state=0;
        break;
      case 4: 
        if ( (j - i) < 40 ) { 
          fprintf(stderr, "gribdate: found GRIB in %s, but remainder file not at least 40 bytes\n", argv[1]);
          exit(1);
        }
/*      
 *     Bump pass section 0 (GRIB Indicator Section) and point to the start of the PDS.
 *     The GRIB documentation is not zero relative, so it will not be here as well. 
 */
        i += 3;

/*
 * From PDS:    octet 5  - Identification center
 *              octet 13 - Year of century
 *              octet 14 - Month of year
 *              octet 15 - Day of month
 *              octet 16 - Hour of day
 *              octet 17 - Minute of hour
 *              octet 18 - Forecast time unit
 *              octet 19 - P1
 *              octet 20 - P2
 *              octet 21 - Time range indicator
 *              octet 25 - Century of initial reference
 *              octet 26 - ID of sub-center
 */
        fprintf(stdout, "%03d %02d %02d %02d %02d %02d %02d %02d %02d %02d %02d %03d\n", 
                                  buf[i+5],
                                  buf[i+13],
                                  buf[i+14],
                                  buf[i+15],
                                  buf[i+16],
                                  buf[i+17],
                                  buf[i+18],
                                  buf[i+19],
                                  buf[i+20],
                                  buf[i+21],
                                  buf[i+25],
                                  buf[i+26] );
         exit(0);
    }
  }
  fprintf(stderr, "gribdate: did not find GRIB in %s\n", argv[1]);
  exit(1); 
}
