#
# dbn_util.pl
#
# Distributed Brokered Networking (DBNet) utilities
#
# Author: Luis J. Cano
# Date:   02/20/96 
#
# 
# buglist
##
# sub cleanup_by_pattern
#
# for the YY... patterns, it is not going to work
# right (initially) after the year 2000 (since 00
# is less than 99). Possible solution is to modify
# cleanup_ptest_rex for the YY... entries to build
# $1 as 0099 and 9900, where YY is 99 and 00 respectfully.
##


package util;

use File::Path;
use File::Find;
use File::Copy;

#
# inited by the user of utils
#
$logfiledate=0;
$logfilename="";
$logfiledir="";
$logfile="";
$logfilestokeep=7;
$verbose=0;
$parsing_table_cache_time=0;
$dbnlogdate=0;
$monitorlogdate=0;



#
# is_running globals
#
$graceful_death=0;

# msg function info
$ERROR_MSG =1;
$WARN_MSG = 3;
$INFO_MSG = 5;
$DEBUG_MSG = 6;

#
# parser interface
#
@parse_cfg_cache=();
$parse_cfg_cache_size=0;
$parse_lasttime=0;

#
# parser globals
#
$parse_cfg_hup=1;

#
# build_fn globals
#
$build_fn_csn=0;

#
# cleanup interface to the howtos
#
@cleanup_match=();
$cleanup_howto="";

#
# cleanup jump table
#      0 = use file date
#      1 = use YYMMDD in file name
#      2 = use YYMMDD.*HH in file name
#      3 = use YYMMDD.*HH.*MI in file name
#      4 = use YYMMDD.*HH.*MI.*SS in file name
#      5 = use MMDD in file name
#      6 = use MMDDHH in file name
#      7 = use MMDDHHMI in file name
#      8 = use MMDDHHMISS in file name
#      9 = use DDHH in file name
#     10 = use DDHHMI in file name
#     11 = use DDHHMISS in file name
#     12 = use HHMI in file name
#     13 = use HHMISS in file name
#     14 = use YYYYMMDD in file name
#     15 = use YYYYMMDDHH in file name
#     16 = use YYYYMMDDHHMI in file name
#     17 = use YYYYMMDDHHMISS in file name
#     18 = use YYYYMM in file name
#
#
$cleanup_nbr_howtos=18;
$pattern_YY="[0-9][0-9]";
$pattern_YYYY="[12][0-9][0-9][0-9]";
$pattern_MM="[01][0-9]";
$pattern_DD="[0-3][0-9]";
$pattern_HH="[0-2][0-9]";
$pattern_MI="[0-5][0-9]";
$pattern_SS="[0-5][0-9]";
$pattern_F="[^0-9]*";
$pattern_HHMI="${pattern_HH}${pattern_MI}";
$pattern_HHMISS="${pattern_HHMI}${pattern_SS}";
$pattern_DDHH="${pattern_DD}${pattern_HH}";
$pattern_DDHHMI="${pattern_DDHH}${pattern_MI}";
$pattern_DDHHMISS="${pattern_DDHHMI}${pattern_SS}";
$pattern_MMDD="${pattern_MM}${pattern_DD}";
$pattern_MMDDHH="${pattern_MMDD}${pattern_HH}";
$pattern_MMDDHHMI="${pattern_MMDDHH}${pattern_MI}";
$pattern_MMDDHHMISS="${pattern_MMDDHHMI}${pattern_SS}";
$pattern_YYMMDD="${pattern_YY}${pattern_MMDD}";
$pattern_YYYYMMDD="${pattern_YYYY}${pattern_MMDD}";
$pattern_YYYYMM="${pattern_YYYY}${pattern_MM}";

%cleanup_ptest_rex = (
'0',  '"";',
'1',  '$_ =~ /(${pattern_YYMMDD})/, 
                                    $tmp1=$1; 
                                    $tmp1 =~ /^(\d\d)/ ; 
                                     if ( $1 < 50 ) 
                                        { "20${tmp1}" }  
                                     else  
                                        { "19${tmp1}" } ;',
'2',  '$_ =~ /(${pattern_YYMMDD})${pattern_F}(${pattern_HH})/, 
                                    $tmp1="${1}${2}"; 
                                    $tmp1 =~ /^(\d\d)/ ; 
                                     if ( $1 < 50 ) 
                                        { "20${tmp1}" }  
                                     else  
                                        { "19${tmp1}" } ;',
'3',  '$_ =~ /(${pattern_YYMMDD})${pattern_F}(${pattern_HH})${pattern_F}(${pattern_MI})/, 
                                    $tmp1="${1}${2}${3}";
                                    $tmp1 =~ /^(\d\d)/ ; 
                                     if ( $1 < 50 ) 
                                        { "20${tmp1}" }  
                                     else  
                                        { "19${tmp1}" } ;',
'4',  '$_ =~ /(${pattern_YYMMDD})${pattern_F}(${pattern_HH})${pattern_F}(${pattern_MI})${pattern_F}(${pattern_SS})/, 
                                    $tmp1="${1}${2}${3}${4}";
                                    $tmp1 =~ /^(\d\d)/ ; 
                                     if ( $1 < 50 ) 
                                        { "20${tmp1}" }  
                                     else  
                                        { "19${tmp1}" } ;',
'5',  '$_ =~ /(${pattern_MMDD})/, "$1";',
'6',  '$_ =~ /(${pattern_MMDDHH})/, "$1";',
'7',  '$_ =~ /(${pattern_MMDDHHMI})/, "$1";',
'8',  '$_ =~ /(${pattern_MMDDHHMISS})/, "$1";',
'9',  '$_ =~ /(${pattern_DDHH})/, "$1";',
'10', '$_ =~ /(${pattern_DDHHMI})/, "$1";',
'11', '$_ =~ /(${pattern_DDHHMISS})/, "$1";',
'12', '$_ =~ /(${pattern_HHMI})/, "$1";',
'13', '$_ =~ /(${pattern_HHMISS})/, "$1";',
'14',  '$_ =~ /(${pattern_YYYYMMDD})/, "$1";',
'15',  '$_ =~ /(${pattern_YYYYMMDD})${pattern_F}(${pattern_HH})/, "$1$2";',
'16',  '$_ =~ /(${pattern_YYYYMMDD})${pattern_F}(${pattern_HH})${pattern_F}(${pattern_MI})/, "$1$2$3";',
'17',  '$_ =~ /(${pattern_YYYYMMDD})${pattern_F}(${pattern_HH})${pattern_F}(${pattern_MI})${pattern_F}(${pattern_SS})/, "$1$2$3$4";',
'18',  '$_ =~ /(${pattern_YYYYMM})/, "$1";',
);

%cleanup_jmptbl = (
'0',  '&cleanup_by_filetimestamp;',
'1',  '&cleanup_by_pattern;',
'2',  '&cleanup_by_pattern;',
'3',  '&cleanup_by_pattern;',
'4',  '&cleanup_by_pattern;',
'5',  '&cleanup_by_pattern;',
'6',  '&cleanup_by_pattern;',
'7',  '&cleanup_by_pattern;',
'8',  '&cleanup_by_pattern;',
'9',  '&cleanup_by_pattern;',
'10', '&cleanup_by_pattern;',
'11', '&cleanup_by_pattern;',
'12', '&cleanup_by_pattern;',
'13', '&cleanup_by_pattern;',
'14', '&cleanup_by_pattern;',
'15', '&cleanup_by_pattern;',
'16', '&cleanup_by_pattern;',
'17', '&cleanup_by_pattern;',
'18', '&cleanup_by_pattern;',
);

#
# parser jmptable 
#  0 = msginfo
#  1 = msgcode
#  2 = sitecode
#  3 = account
#  4 = passwd
#  5 = localpath
#  6 = permissions
#  7 = cleanup
#  8 = msginfobycode
#  9 = siteinfobycode 
#  10= quecmd
#
#
%parsejmptable = (
'msginfo',         '&parse_msginfo;',
'msgcode',         '&parse_msgcode',
'sitecode',        '&parse_msgcode;',
'account',         '&parse_account;',
'passwd',          '&parse_account;',
'localpath',       '&parse_local_path;',
'permissions',     '&parse_permissions;',
'cleanup',         '&parse_cleanup;',
'msginfobycode',   '&parse_msginfo_bycode;',
'siteinfobycode',  '&parse_account_bycode;',
'quecmd',          '&parse_quecmd;',
);


#%parametertable = (
#'0' ,  '',
#'1' ,  '/([\S]+)/,eval qq/"$exec_args"/',
#'2' ,  '/([\S]+)[ \t]+([\S]+)/,eval qq/"$exec_args"/',
#'3' ,  '/([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)/,eval qq/"$exec_args"/',
#'4' ,  '/([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)/,eval qq/"$exec_args"/',
#'5' ,  '/([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)/,eval qq/"$exec_args"/',
#'6' ,  '/([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)/,eval qq/"$exec_args"/',
#'7' ,  '/([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)/,eval qq/"$exec_args"/',
#'8' ,  '/([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)/,eval qq/"$exec_args"/',
#'9' ,  '/([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)/,eval qq/"$exec_args"/',
#);

1;





#sub case {
#  local(*assoc,$_) = @_;
#  for (split(/\n/)) {
#    /^(\S+)\s+(.*)/;
#    for (eval $1) {
#      $assoc{$_} = $2;
#    }
#  }
#}


sub init {
  local($logfile_tmp, $tmp_dir, $que, $data, $pending, $foreign);
  local($parsingtbl);

# 
# init from main
#
  $logfile_tmp=$main'logfile;
  $logfilestokeep=$main'logfilestokeep;
  $tmp_dir=$main'tmp_dir;
  $verbose=$main'verbose;
  $que=$main'dbnet_que;
  $data=$main'data;
  $pending=$main'pending_dir;
  $foreign=$main'foreign_dir;
  $err_dir=$main'err_dir;
  $parsing_table_cache_time=$main'parsing_table_cache_time;
  $parsingtbl=$main'parsingtbl;

#
# end init from main
#

  local(@logfilearray)=split(/\//, $logfile_tmp);

  $logfilename=pop(@logfilearray);
  $logfilename=~s/(\.)/\\$1/g;
  $logfilename="$logfilename.*";

  $logfiledir="";
  while (@logfilearray) {
    $logfiledir .= shift(@logfilearray);
    $logfiledir .= "/" if @logfilearray;
  }
  $logfile=$logfile_tmp;
#
# should we create dbnet directories?
#

  if ( (! defined $main'init_create_dirs) || (! $main'init_create_dirs) ) {
#    &msg($INFO_MSG,"init: init_create_dirs: $main'init_create_dirs: bypass mkdirs");
    return 1;
  }

#
# Yes, let's create them.
#

  &msg($INFO_MSG,"init: init_create_dirs: $main'init_create_dirs: entering mkdirs");
  
  if (!&mkdirs($logfiledir)) {
    &msg($ERROR_MSG,"init: mkdirs(${logfiledir}) failed");
    return 0;
  }
# &cleanup_logs;
  if (!&mkdirs($tmp_dir)) {
    &msg($ERROR_MSG,"init: mkdirs(${tmp_dir}) failed");
    return 0;
  }
  if (!&mkdirs($que)) {
    &msg($ERROR_MSG,"init: mkdirs(${que}) failed");
    return 0;
  }
  if (!&mkdirs($data)) {
    &msg($ERROR_MSG,"init: mkdirs(${data}) failed");
    return 0;
  }
  if (!&mkdirs($foreign)) {
    &msg($ERROR_MSG,"init: mkdirs(${foreign}) failed");
    return 0;
  }
  if (!&mkdirs($pending)) {
    &msg($ERROR_MSG,"init: mkdirs(${pending}) failed");
    return 0;
  }
  if (!&mkdirs($err_dir)) {
    &msg($ERROR_MSG,"init: mkdirs(${err_dir}) failed");
    return 0;
  }
  return 1;
}

sub mapstreamtolog {
  local (*s) = @_;

  return if !$logfile;

  $s = 'util\'LOGFILE';
}

sub msg {
  local($severity, $s ) = @_;
  return if !$logfile;
  local(@t, $timestamp);

  @t = gmtime(time);
  $t[4]++;
  $timestamp=sprintf("%02d%02d%02d", $t[5]+1900, $t[4], $t[3]);
  if ($logfiledate ne $timestamp) {
    if ($logfiledate) {
      close(LOGFILE) || warn "$0: msg: cannot close ${logfile}.${logfiledate}: $!";
    }
#    &cleanup_logs;
    $logfiledate=$timestamp;
    if (! open(LOGFILE, ">> ${logfile}.${logfiledate}") ) {
      warn "$0: msg: cannot open ${logfile}.${logfiledate}: $!";
      $logfiledate=0;
      return 0;
    }
    select((select(LOGFILE), $| = 1)[$[]);  # flush 
  } 
  printf LOGFILE ("%02d%02d%02d: [%d] [Severity - %d] %s\n", $t[2], $t[1], $t[0], $$,$severity, $s);
  printf STDERR  ("%02d%02d%02d: [%d] [Severity - %d] %s\n", $t[2], $t[1], $t[0], $$,$severity, $s) if $verbose;
}


# logmsg function to replace msg function: write to specified log file:
# $main'dbnlogfile or $main'monitorlogfile.

sub logmsg {
  my ($logname, $severity, $s) = @_;
  return if !$logfile;

  local(@t, $timestamp);

  @t = gmtime(time);
  $t[4]++;
  $timestamp=sprintf("%02d%02d%02d", $t[5]+1900, $t[4], $t[3]);

# Get last log file timestamp and file handle for specified logfile.

  if ($logname eq $main'dbnlogfile) {
    $lastlogdate = \$utils'dbnlogdate;
    $LOGFILE = "DBNLOGFILE";
    }
  elsif ($logname eq $main'monitorlogfile) {
    $lastlogdate = \$utils'monitorlogdate;
    $LOGFILE = "MONLOGFILE";
    }
  else {
    print "$0: logmsg:  Unknown logfile: $logname "; return 0;
  }


# If this is a new day, or last log date empty, 
# create or reopen new log file.

  if ($$lastlogdate ne $timestamp) {
    if ($$lastlogdate) {
      close($LOGFILE) || warn "$0: msg: cannot close ${logname}.${$lastlogdate}:
 $!";
    }
    $$lastlogdate=$timestamp;
    $utils'monitorlogdate=$timestamp;
    if (! open($LOGFILE, ">> ${logname}.${$lastlogdate}") ) {
      warn "$0: msg: cannot open ${logname}.${$lastlogdate}: $!";
      $$lastlogdate=0;
      return 0;
    }
  }
  printf $LOGFILE ("%02d%02d%02d: [%d] [Severity - %d] %s\n", $t[2], $t[1], $t[0], 
        $$,$severity, $s);
  printf STDERR  ("%02d%02d%02d: [%d] [Severity - %d] %s\n", $t[2], $t[1], $t[0],
         $$,$severity, $s) if $verbose;
  select((select($LOGFILE), $| = 1)[$[]);  # flush
}

sub cleanup_logs {
#  &cleanup($logfiledir, "IFS\.log.*", $logfilestokeep, 0);
  return if !$logfile;
  &cleanup($logfiledir, $logfilename, $logfilestokeep, 0);
}

sub clean_all {
  my(@clean_list, $curr_time, $dir, $filepattern, $sort_func, $qty);
  my($clean_table) = @_;
  my($clean_pattern) = "\\S+[ \\t]+\\S+[ \\t]+\\S+[ \\t]+\\S+\\s*\\S*";

########################################################
# read in the clean table of files to look for
########################################################

  if (!open(CLEAN, "< ${clean_table}")) {
    &util'msg($util'ERROR_MSG,"$0: open (CLEAN,<${clean_table}) failed. $!");
    return 0;
  }
  #&util'msg($util'ERROR_MSG,"$0: open (CLEAN,<${clean_table}) successful. $!");

  undef @clean_list;

  while (<CLEAN>) {
    chomp;
    next if /^#/;    # this should probably be in clean_pattern
    push(@clean_list,$_) if /$clean_pattern/;
  }

  if ( !close(CLEAN) ) {
    &util'msg($util'ERROR_MSG,"$0: close (CLEAN) failed. $!, not cleaning");
    return 0;
  }

  foreach (@clean_list) {
    ($dir, $filepattern, $sort_type, $qty, $use_subdir) = split(/[\t ]+/,$_);
  #  &util'msg($util'ERROR_MSG,"Clean::: $dir,$filepattern,$qty,$sort_type,$use_subdir");
    &cleanup($dir,$filepattern,$qty,$sort_type,$use_subdir);
  }
}
##
# wanted sub for cleanup
#
# $use_subdirs must be a global variable for the wanted routine to find it
##

sub wanted {

  #&util'msg($util'DEBUG_MSG,"$0: WANTED: $File::Find::dir, $_, $File::Find::name, $File::Find::prune, ${gl_cleanup_dir}, $gl_cl_pattern,$use_subdir");

# skip the direcotries . and ..
  if ($_ =~ /^\.\.?$/) {
    return;
  }

  $File::Find::prune = !$use_subdir if -d ;

#  if ( ($File::Find::dir =~ /${gl_cleanup_dir}/) && ($_ =~ /$gl_cl_pattern/) ) {
  if ($File::Find::name =~ /${gl_cleanup_dir}\/$gl_cl_pattern/) { 
    @stat_rtn=();
    if (! -l && !(@stat_rtn=stat("$File::Find::name"))) {
      &msg($ERROR_MSG,"cleanup: cannot stat $File::Find::name: $!");
      return;
    }
    if (-l (@stat_rtn=lstat("$File::Find::name"))) {
      &msg($ERROR_MSG,"cleanup: cannot lstat $File::Find::name: $!");
      return;
    }
    $mtime=splice(@stat_rtn, 9, 1);
    push(@cleanup_match, "${mtime};$File::Find::name");
  #  &util'msg($util'DEBUG_MSG,"$0: wanted: ${mtime};$File::Find::name");
  }
}

#
# $dir = directory where cleanup is to take place
# $pattern = file pattern in directory to match
# $clean_val = the amount of patterns to keep
# $cleanup_howto = method in which cleanup is performed
# 
sub cleanup {
  local($cleanup_dir, $pattern, $clean_val);
  ($gl_cleanup_dir, $gl_cl_pattern, $clean_val, $cleanup_howto, $use_subdir)=@_;
  msg($DEBUG_MSG, "sub cleanup: $gl_cleanup_dir $gl_cl_pattern $clean_val $cleanup_howto $use_subdir");
  local(@files, $f, @stat_rtn, $mtime, $i, $entry);

  $cleanup_howto = 0 if !defined($cleanup_howto);

  @cleanup_match = (); 
  #msg($DEBUG_MSG, "Cleanup : $gl_cleanup_dir, $gl_cl_pattern, $clean_val, $cleanup_howto");
  if ($cleanup_howto > $cleanup_nbr_howtos) {
    &msg($WARN_MSG,"cleanup: cleanup_howto(${cleanup_howto}) > cleanup_nbr_howtos(${cleanup_nbr_howtos}): using default cleanup");
    $cleanup_howto = 0;
  }
  
  find(\&wanted,"$gl_cleanup_dir");

  return 1 if @cleanup_match <= $clean_val;
  eval $cleanup_jmptbl{$cleanup_howto};
  return 1 if @cleanup_match <= $clean_val;
  #&msg($DEBUG_MSG,"files :@cleanup_match");
  for ($i=0; @cleanup_match > 0;  $i++) {
    $entry=shift(@cleanup_match);
    if ($i >= $clean_val) {
      ($mtime, $f)=split(/;/, $entry, 2); 
      #&msg($INFO_MSG,"cleanup: calling cleanfile $f");
      &cleanfile("$f");
    }
  }
  return 1; 
}

sub cleanfile {
  my ($f) = @_;
  if (!(($f =~ /\/\.$/) || ($f =~ /\/\.\.$/))) {
  if ((-f $f) || (-l $f)) { 
    &msg($INFO_MSG,"$0: unlinking file $f");
    if (! defined $main::dbn_clean_test || $main::dbn_clean_test != 1) {
      unlink("$f") || &msg($ERROR_MSG,"cleanup: cannot unlink $f: $!");
    }
  } else {
    &msg($INFO_MSG,"$0: rmtree $f");
    if (! defined $main::dbn_clean_test || $main::dbn_clean_test != 1) {
      rmtree($f) || &msg($ERROR_MSG,"cleanfile cannot remove directory $f: $!");
    }
  }
  }
}


sub cleanup_by_filetimestamp {
  local(@temp, $f, $d_time, $d_f, $q_time, $q_f, $i);
  my (%timestamphash);

  foreach $f (@cleanup_match) {
    $timestamphash{$f} = (split(/;/,$f))[0];
  }

  @cleanup_match = sort { $timestamphash{$b} <=> $timestamphash{$a} } @cleanup_match;  

}


sub cleanup_by_pattern {
  my (%sortlist) = ();
  local($f, $d_time, $d_f, $d_pattern, $q_time, $q_f, $i);
  #&util'msg($ERROR_MSG,"Cleanup by pattern");

  foreach $f (@cleanup_match) {
    ($d_time, $d_f) = split(/;/, $f, 2);
    $_=$d_f;
    next if !($d_pattern = eval $cleanup_ptest_rex{$cleanup_howto});

    $sortlist{$f} = $d_pattern;
    #&util'msg($ERROR_MSG,"Adding file to sort list: $d_pattern");
  }
  @cleanup_match = sort { $sortlist{$b} cmp $sortlist{$a} } @cleanup_match;


}



# return 0 on error, 1 on success
sub make_dir
{
  local( $dir, $mode ) = @_;
  local( $val );

  # make a local directory
  if( -e $dir || -l $dir ){
    unlink( $dir );
  }
  if (!($val = mkdir( $dir, $mode ))) {
    &msg($ERROR_MSG,"make_dir: mkdir($dir, $mode) failed: $!");
  } 
  return $val;
}

 
sub mkdirs
{
  local( $dir ) = @_;
  local( @dirs, $d, $path );

  # Very often the directory does exist - so return now
  return 1 if -d $dir; 

  # Make sure that the target directory exists
  @dirs = split( '/', $dir );

  # the root directory always exists
  $path = '';
  if( $dirs[ 0 ] eq '' ){
    shift( @dirs );
    $path = '/';
  }

  foreach $d ( @dirs ){
    $path = $path . $d;
    if( ! -d $path ){
       &msg($INFO_MSG, "mkdirs: $path" );
       return 0 if  !&make_dir($path, 0755);
#      &set_attribs( $path, 'd' );
     }
     $path .= "/";
  }
  return 1;
}


sub split_path {
  local($path) = @_;
  local(@work_array, $f, $dir);
  $dir = "";

  @work_array=split(/\//, $path);
  $f=pop(@work_array);

  while (@work_array) {
    $dir .= shift(@work_array);
    $dir .= "/" if @work_array;
  }
  return ($dir, $f);
}


sub build_filedate {
  local($path)=@_;
  local(@fs, @ft, $filedate);

  if (!(@fs=stat("${path}"))) {
    &msg($ERROR_MSG,"build_filedate: cannot stat ${path}: $!");
    return ();
  }
  @ft=gmtime($fs[9]);
  $ft[4]++;
  $filedate = sprintf("%02d%02d%02d%02d%02d%02d", $ft[5], $ft[4], $ft[3], $ft[2], $ft[1], $ft[0]); 
  return ($filedate, $fs[7]);
}



#
# parser
#

sub parse {

  local($rtn_type, $cfg_name, $msg_type, $msg_sub, $arg) = @_;

  if ($parse_cfg_hup || (!@parse_cfg_cache) ) {
    $parse_cfg_hup=0;
#    &msg($WARN_MSG,"parse: caching $cfg_name");
    @parse_cfg_cache=();
    if (!open(CFG, $cfg_name)) {
      &msg($ERROR_MSG,"parse: open $cfg_name failed");
      return 0;
    }
    while (<CFG>) {
      next if /(^[ \t]*\#)|(^[ \t]*$)/;
      chop;
      push(@parse_cfg_cache, $_);
    }
    close (CFG) || &msg($ERROR_MSG,"parse: close($cfg_name) failed!");
    $parse_cfg_cache_size=@parse_cfg_cache;
  }

  return "" if ! defined($parsejmptable{$rtn_type});
  eval $parsejmptable{$rtn_type};

}




sub parse_msgcode {
  local($ptype, $psub, $parg, $pval, $i);

  for ($i=0; $i < $parse_cfg_cache_size; $i++) {
    ($ptype, $psub, $parg, $pval) = split(/[ \t]+/, $parse_cfg_cache[$i]);
    return $pval if ( $ptype && $msg_type =~ /$ptype/ &&
                      $psub  && $msg_sub  =~ /$psub/  &&
                      $parg  && $arg      =~ /$parg/  && $pval );
  }
  "";
} 




sub parse_account {
  local($ptype, $psub, $parg, $pval, $paccount, $ppasswd, $pprefsrchostname, $i);

  for ($i=0; $i < $parse_cfg_cache_size; $i++) {
    ($ptype, $psub, $parg, $pval, $paccount, $ppasswd, $pprefsrchostname) = split(/[ \t]+/, $parse_cfg_cache[$i], 7);
    if ($ptype && $msg_type =~ /$ptype/ &&
        $psub  && $msg_sub  =~ /$psub/  &&
        $parg  && $arg      =~ /$parg/  && $paccount && $ppasswd) {
      chop ($ppasswd = `whoami`) if ($ppasswd eq "`whoami`");
      return ($paccount, $ppasswd, $pprefsrchostname);
    }
  }
  ();
}



sub parse_account_bycode {
  local($ptype, $psub, $parg, $pval, $paccount, $ppasswd, $pprefsrchostname, $i);

  for ($i=0; $i < $parse_cfg_cache_size; $i++) {
    ($ptype, $psub, $parg, $pval, $paccount, $ppasswd, $pprefsrchostname) = split(/[ \t]+/, $parse_cfg_cache[$i], 7);
    if ($ptype && $msg_type =~ /$ptype/ && 
        $psub  && $msg_sub  =~ /$psub/  &&
        $parg  && "$pval" eq $arg && $paccount && $ppasswd) {
      chop ($ppasswd = `whoami`) if ($ppasswd eq "`whoami`");
      return ($paccount, $ppasswd, $pprefsrchostname);
    }
  }
  ();
}



sub parse_local_path {
  local($ptype, $psub, $parg, $pval, $plocaldir, $plocalfile, $i);

  for ($i=0; $i < $parse_cfg_cache_size; $i++) {
    ($ptype, $psub, $parg, $pval, $plocaldir, $plocalfile) = split(/[ \t]+/, $parse_cfg_cache[$i]);
    if ($ptype && $msg_type =~ /$ptype/ &&
        $psub  && $msg_sub  =~ /$psub/  &&
        $parg  && $arg      =~ /$parg/  && $plocaldir && $plocalfile) {
      $arg =~ /$parg/;
      return (eval qq/"$plocaldir"/, eval qq/"$plocalfile"/);
    }
  }
  ();
}



sub parse_permissions {
  local($ptype, $psub, $parg, $pval, $plocaldir, $plocalfile, $ppermissions, $powner, $pgroup, $i);

  for ($i=0; $i < $parse_cfg_cache_size; $i++) {
    ($ptype, $psub, $parg, $pval, $plocaldir, $plocalfile, $ppermissions, $powner, $pgroup) = split(/[ \t]+/, $parse_cfg_cache[$i]);
    return ($ppermissions, $powner, $pgroup)  if ($ptype && $msg_type =~ /$ptype/ &&
                                                  $psub  && $msg_sub  =~ /$psub/  &&
                                                  $parg  && $arg      =~ /$parg/  && 
                                                  $ppermissions && $powner && $pgroup) 
  }
  ();
}




sub parse_cleanup {
 local($ptype, $psub, $parg, $pval, $plocaldir, $plocalfile, $ppermissions, $powner, $pgroup, $pcleanup, $i);

  for ($i=0; $i < $parse_cfg_cache_size; $i++) {
    ($ptype, $psub, $parg, $pval, $plocaldir, $plocalfile, $ppermissions, $powner, $pgroup, $pcleanup) = split(/[ \t]+/, $parse_cfg_cache[$i]);
    return ($pcleanup) if ($ptype && $msg_type =~ /$ptype/ &&
                           $psub  && $msg_sub  =~ /$psub/  &&
                           $parg  && $arg      =~ /$parg/  && $pcleanup);
  }
  ();
}




sub parse_msginfo {
 local($ptype, $psub, $parg, $pval, $plocaldir, $plocalfile, $ppermissions, $powner, $pgroup, $pcleanup, $i);

  for ($i=0; $i < $parse_cfg_cache_size; $i++) {
    ($ptype, $psub, $parg, $pval, $plocaldir, $plocalfile, $ppermissions, $powner, $pgroup, $pcleanup) = split(/[ \t]+/, $parse_cfg_cache[$i]);
    if ($ptype && $msg_type =~ /$ptype/ &&
        $psub  && $msg_sub  =~ /$psub/ &&
        $parg  && $arg      =~ /$parg/ && 
        $pval  && $plocaldir && $plocalfile && $ppermissions && $powner && $pgroup ) {
      $arg =~ /$parg/;
      return ($pval, eval qq/"$plocaldir"/, eval qq/"$plocalfile"/, $ppermissions, $powner, $pgroup, $pcleanup);
    }
  }
  ();
}



sub parse_msginfo_bycode {
  local($ptype, $psub, $parg, $pval, $plocaldir, $plocalfile, $ppermissions, $powner, $pgroup, $pcleanup, $i);

  for ($i=0; $i < $parse_cfg_cache_size; $i++) {    
    ($ptype, $psub, $parg, $pval, $plocaldir, $plocalfile, $ppermissions, $powner, $pgroup, $pcleanup) = split(/[ \t]+/, $parse_cfg_cache[$i]);
    if ($ptype && 
        $psub  && 
        $parg  && $arg =~ /$parg/ &&  
        "$pval" eq $msg_type && $plocaldir && $plocalfile && $ppermissions && $powner && $pgroup ) {
      $arg =~ /$parg/;
      return ($ptype, $psub, $parg, eval qq/"$plocaldir"/, eval qq/"$plocalfile"/, $ppermissions, $powner, $pgroup, $pcleanup);
    }
  }
  ();
}


sub parse_quecmd {
  local( $ptype, $psite, $pmsg, @cmd_list, $i,$gname, $mode );

  @cmd_list=();
  for ($i=0; $i < $parse_cfg_cache_size; $i++) {
    ($ptype, $psite, $pmsg)=split(/[ \t]+/, $parse_cfg_cache[$i]);
    if ($ptype && $msg_type =~ /$ptype/ &&
        $psite && $msg_sub  =~ /$psite/ &&
        $pmsg  && $arg      =~ /$pmsg/) {
      push(@cmd_list, $1) if $parse_cfg_cache[$i] =~ /[\S]+[ \t]+[\S]+[ \t]+[\S]+[ \t]+(.+)/;
    }
    if ($ptype && "GROUP "  =~ /$ptype/ &&
        $psite && $msg_sub  =~ /$psite/ &&
        $pmsg  && $arg      =~ /$pmsg/) {
      ($ptype, $psite, $pmsg, $gname, $mode,$exp_time)=split(/[ \t]+/, $parse_cfg_cache[$i]);
      $g = "GROUP $psite $pmsg $mode $gname $exp_time";
      $i++;
      ($ptype, $psite, $pmsg)=split(/[ \t]+/, $parse_cfg_cache[$i]);
       while ($ptype eq "GCMD") {
         $g .= "!!${1}" if $parse_cfg_cache[$i] =~ /[\S]+[ \t]+(.+)/;
         $i++;
         ($ptype)=split(/[ \t]+/, $parse_cfg_cache[$i]);
      }
      push(@cmd_list,$g);
#      &msg($DEBUG_MSG,"GROUP CMD LIST $g");
    } 
         
  }
  return @cmd_list;
}


sub quecmd {
  local(*cmds, $que, $local_path, $local_fn, $parse_table, $msg, $msg_sitecode) = @_;
  local($i, $nbr_of_entries, $cmd_type, $dest_host, $cmd_fn, $clnt_args, $nbr_cmds_proc);
  local($sitecode, $exec_path, $exec_mode, $exec_arg,$ind,$ind2,$evalmap);
  local($inputcommands,@incommands,@returnvalue, $cmd_time);
  my (@commandlist,$untainted_file,$untainted_file2,$rsitecode,$desthost,$junk);
  $nbr_cmds_proc =0;

  @returnvalue =();
  for ($i=0, $nbr_of_entries=@cmds, $nbr_cmds_proc=0; $i < $nbr_of_entries; $i++) {
    ($cmd_type)=split(/[ \t]+/, $cmds[$i]);
    next if ! $cmd_type;
    if ($cmd_type eq "CLNT") {
      $cmds[$i] =~ /[\S]+[ \t]+([\S]+)[ \t]+([\S]+)[ \t]*(.*)/;
      $cmd_time = $1; $dest_host=$2,  $clnt_args=$3;
      if ( !$dest_host ) {
        &msg($ERROR_MSG,"quecmd: incomplete cmd: cmd_type=${cmd_type} dest_host=${dest_host} clnt_args=${clnt_args}");
        next;
      }
      if (!($sitecode=&util'parse("sitecode", "$parse_table",  "HOST", "host",  "$dest_host"))) {
        &msg($WARN_MSG,"quecmd: parse(sitecode, ${parse_table}, HOST, host, ${dest_host}) failed!: using default");
        $sitecode="0000"; 
      }
      $cmd_fn=&build_fn($cmd_type, $sitecode, $msg,$cmd_time);
      "${que}/.${cmd_fn}" =~ /(.*)/;
      $untainted_file = $1;
      if (!open(CMD, "> ${untainted_file}")) {
        &msg($ERROR_MSG,"quecmd: open ${que}/.${cmd_fn} failed: $!");
        return 0;
      }
      printf CMD "%s %s %s", "$dest_host", "$local_path", "$clnt_args";
      &msg($INFO_MSG,"quecmd: qitem: ${untainted_file}: qcont: $dest_host $local_path $clnt_args"); 
      close(CMD) || &msg($ERROR_MSG,"quecmd: close ${que}/.${cmd_fn} failed: $!");
      "${que}/${cmd_fn}" =~ /(.*)/;
      $untainted_file2 = $1;
      if (!rename("${untainted_file}", "${untainted_file2}")) {
        &msg($ERROR_MSG,"quecmd: rename(${que}/.${cmd_fn}, ${que}/${cmd_fn}) failed");
        return 0;
      }
      push(@returnvalue,"${cmd_fn}");
      $nbr_cmds_proc++;
    }
    elsif ($cmd_type eq "GROUP") {
      (@commandlist) = split(/!!/,$cmds[$i]);
      $commandlist[0] =~ /[\S]+[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+(.+)/;
      $d=$1,  $c=$2; $exp_time = $5;
#      &msg($DEBUG_MSG,"command list:$cmds[$i]::$d,$c");
#      &msg($DEBUG_MSG,"site/msg:$d,$c");

      $exec_mode = $3;

      $cmd_fn=&build_fn($cmd_type, $exec_mode , $msg, $exp_time);

      "${que}/.${cmd_fn}" =~ /(.*)/;
      $untainted_file = $1;
      if (!open(CMD, "> ${untainted_file}")) {
        &msg($ERROR_MSG,"quecmd: open ${que}/.${cmd_fn} failed: $!");
        return 0;
      }

      printf CMD "%s %s %s\n", $commandlist[0], $msg,$local_path;
      &msg($INFO_MSG,"quecmd: qitem: ${untainted_file}: qcont: $commandlist[0] $msg $local_path");
      for ($ii=1;$ii<@commandlist;$ii++) {
        printf CMD "%s\n", $commandlist[$ii]; 
        &msg($INFO_MSG,"quecmd: qitem: ${untainted_file}: qcont: $commandlist[$ii]");
      }

      close(CMD) || &msg($ERROR_MSG,"quecmd: close ${que}/.${cmd_fn} failed: $!");

      "${que}/${cmd_fn}" =~ /(.*)/;
      $untainted_file2 = $1;
      if (!rename("${untainted_file}", "${untainted_file2}")) {
        &msg($ERROR_MSG,"quecmd: rename(${que}/.${cmd_fn}, ${que}/${cmd_fn}) failed");
        return 0;
      }
      
      $nbr_cmds_proc++;
      push(@returnvalue,"${cmd_fn}");

    }
    elsif ($cmd_type eq "DPROC") {
      next;         # implementation needed for DPROC 
    }
    elsif ($cmd_type eq "LDM") {
      next;         # implementation needed for LDM
    }

    elsif ($cmd_type eq "EXEC") {
      $cmds[$i] =~ /[\S]+[ \t]+([\S]+)[ \t]+([\S]+)[ \t]+([\S]+)[ \t]*(.*)/;
      $cmd_time = $1; $exec_mode=$2,  $exec_path=$3, $exec_args=$4;
#      &msg($DEBUG_MSG,"quecmd: status:$cmds[$i],msg:$msg, cmd_type=${cmd_type} exec_path=${exec_path} exec_args=${exec_args}");

      if ( !$exec_mode || !$exec_path || !cmd_time) {
        &msg($ERROR_MSG,"quecmd: incomplete cmd: cmd_type=${cmd_type} exec_path=${exec_path} exec_args=${exec_args}");
        next;
      }

     # decide if we need to open and parse the file.  ie are there any $\d command line
     # parameters.
     # this $f is my first attempt at passing parameters into the EXEC command,
     # other than the internal parameters
     $f = $local_path;

     if ($exec_args =~ /\$\d/) {                # there are internal parameters
#       &util'msg($util'DEBUG_MSG,"Processing Internal EXEC parameters");
       if (!open(CMDIN, "$local_path")) {
          &msg($ERROR_MSG,"quecmd: open $local_path failed: $!");
          return 0;
        }
#        while (<CMDIN>) {
          $_ = (<CMDIN>);
#        }

        close(CMDIN) || &msg($ERROR_MSG,"quecmd: close $local_path failed: $!");
       $evalmap="";   
       $inputcommands = "";
       $ind = scalar(split);
       if ($exec_args ne "" ) {
         $evalmap = "/([\\S]+)";
         for ($ind2=0;$ind2<($ind-1);$ind2++) {
	   $evalmap .= "[ \\t]+([\\S]+)";
         }
         $evalmap .= "/,eval qq/\"\$exec_args\"/";
#         &util'msg($DEBUG_MSG,"EXEC: parameter string: $evalmap");
         if (!($inputcommands=eval $evalmap)) {
            &msg($ERROR_MSG,"quecmd: Parameter match failed, input=$_");
          } 
#         if (!($inputcommands=eval $parametertable{$ind})) {
#            &msg($ERROR_MSG,"quecmd: Parameter match failed, input=$_");
#          } 
#          &msg($DEBUG_MSG,"quecmd: status-2 " . scalar(@a) . ",i=$inputcommands,\np=$parametertable{scalar(split)}:\n_=$_");
#          &msg($DEBUG_MSG,"quecmd: status-3: i=$inputcommands, e=$exec_args");
        }
      } #end of internal parameter processing
      else {
        $inputcommands = $exec_args;
      }

      $inputcommands =~ s/\$f/$f/;
#      &msg($DEBUG_MSG,"Command line options i=$inputcommands, e=$exec_args");

      $cmd_fn=&build_fn($cmd_type, $exec_mode, $msg, $cmd_time);
      "${que}/.${cmd_fn}" =~ /(.*)/;
      $untainted_file = $1;
      if (!open(CMD, "> ${untainted_file}")) {
        &msg($ERROR_MSG,"quecmd: open ${que}/.${cmd_fn} failed: $!");
        return 0;
      }
      printf CMD "%s %s %s","$exec_mode", "$exec_path", "$inputcommands";
      &msg($INFO_MSG,"quecmd: qitem: ${untainted_file}: qcont: $exec_mode $exec_path $inputcommands");
      close(CMD) || &msg($ERROR_MSG,"quecmd: close ${que}/.${cmd_fn} failed: $!");
      "${que}/${cmd_fn}" =~ /(.*)/;
      $untainted_file2 = $1;
      if (!rename("${untainted_file}", "${untainted_file2}")) {
        &msg($ERROR_MSG,"quecmd: rename(${que}/.${cmd_fn}, ${que}/${cmd_fn}) failed");
        return 0;
      }
      $nbr_cmds_proc++;
      push(@returnvalue,"${cmd_fn}");

#      next;         # implementation needed for exec
    }
    elsif ($cmd_type eq "FORK") {
      next;         # implementation needed for fork
    }
    elsif ($cmd_type eq "FTPGET") {
      $cmds[$i] =~ /[\S]+[ \t]+(.+)/;
      $cmd_time=$1;
#      &msg($DEBUG_MSG,"quecmd: status:$cmds[$i],msg:$msg, cmd_type=${cmd_type} exec_path=${exec_path} exec_args=${exec_args}");

      if (!open(CMDIN, "$local_path")) {
        &msg($ERROR_MSG,"quecmd: open $local_path failed: $!");
        return 0;
      }
#      while (<CMDIN>) {
        $_ = (<CMDIN>);
#      }

      close(CMDIN) || &msg($ERROR_MSG,"quecmd: close $local_path failed: $!");
      
#     if (!($inputcommands=eval $parametertable{scalar(split) > 9 ? 9 : scalar(split)})) {
#        &msg($ERROR_MSG,"quecmd: Parameter match failed, input=$_");
#      } 
#      &msg($DEBUG_MSG,"quecmd: status-2 " . scalar(@a) . ",i=$inputcommands,\np=$parametertable{scalar(split)}:\n_=$_");
#      &msg($DEBUG_MSG,"quecmd: status-3: i=$inputcommands, e=$exec_args");
# figure out the host code fo the remote site.

      ($desthost,$junk)=split(/ /,$_);

      if (!($rsitecode=&util'parse("sitecode", "$parse_table",  "HOST",  "host",  "$desthost"))) {
        &util'msg($util'ERROR_MSG,"$0: parse(hostid, ${parse_table}, HOST,  host,  ${desthost}) failed!");
        $rsitecode = "0000";
      }

      $cmd_fn=&build_fn($cmd_type, $rsitecode, $msg, $cmd_time);
      "${que}/.${cmd_fn}" =~ /(.*)/;
      $untainted_file = $1;
      if (!open(CMD, "> ${untainted_file}")) {
        &msg($ERROR_MSG,"quecmd: open ${que}/.${cmd_fn} failed: $!");
        return 0;
      }
      printf CMD "%s","$_";
      &msg($INFO_MSG,"quecmd: qitem: ${untainted_file}: qcont: $_");
      close(CMD) || &msg($ERROR_MSG,"quecmd: close ${que}/.${cmd_fn} failed: $!");
      "${que}/${cmd_fn}" =~ /(.*)/;
      $untainted_file2 = $1;
      if (!rename("${untainted_file}", "${untainted_file2}")) {
        &msg($ERROR_MSG,"quecmd: rename(${que}/.${cmd_fn}, ${que}/${cmd_fn}) failed");
        return 0;
      }
      $nbr_cmds_proc++;
      push(@returnvalue,"${cmd_fn}");

    }
    elsif ($cmd_type eq "FILE") {
      $cmds[$i] =~ /[\S]+[ \t]+([\S]+)[ \t]+(.*)/;
      $cmd_time = $1; $cmd_args = $2;
      $msg_sitecode = "0000" if ! defined $msg_sitecode;
      $cmd_fn=&build_fn($cmd_type, $msg_sitecode, $msg, $cmd_time);
      "${que}/.${cmd_fn}" =~ /(.*)/;
      $untainted_file = $1;
      if (!open(CMD, "> ${untainted_file}")) {
        &msg($ERROR_MSG,"quecmd: open ${que}/.${cmd_fn} failed: $!");
        return 0;
      }
      print CMD "-i $local_path $cmd_args";
      &msg($INFO_MSG,"quecmd: qitem: ${untainted_file}: qcont: -i $local_path $cmd_args");
      close(CMD) || &msg($ERROR_MSG,"quecmd: close ${que}/.${cmd_fn} failed: $!");
      "${que}/${cmd_fn}" =~ /(.*)/;
      $untainted_file2 = $1;
      if (!rename("${untainted_file}", "${untainted_file2}")) {
        &msg($ERROR_MSG,"quecmd: rename(${que}/.${cmd_fn}, ${que}/${cmd_fn}) failed");
        return 0;
      }
      $nbr_cmds_proc++;
      push(@returnvalue,"${cmd_fn}");
    }  
    elsif ($cmd_type eq "PIPE") {
      next;         # implementation needed for pipe
    }
    else {
      &msg($ERROR_MSG,"quecmd: unknown cmd_type=${cmd_type}");
      next; 
    }
  }
  &msg($ERROR_MSG,"quecmd: zero cmds found!") if !$nbr_cmds_proc;
#  return $nbr_cmds_proc;
  return @returnvalue;
}


sub build_fn {
  local($prefix1, $prefix2, $prefix3, $f_time)=@_;
  local ($t) = time;
  local ($exp_t);

  if (($f_time == 0) || !defined($f_time)) {
    $exp_t = "0";
  }
  else {
    $exp_t = $t + ($f_time * 60);
  }
  $build_fn_csn=($build_fn_csn + 1) % 10000;
  "$prefix1" . "."  . "$prefix2" . "." . "$prefix3" . "." . "$build_fn_csn" . "." . "$$" . ".${t}.${exp_t}"; 
}



sub is_running {
  local($tmp_dir, $who) = @_;

  if (-e "${tmp_dir}/.${who}") {
    &msg($ERROR_MSG,"$0: is_running: already running - ${tmp_dir}/.${who} exists");
    return 0;
  }

  if ( ! open(RUNNING, "> ${tmp_dir}/.${who}") ) {
    &msg($ERROR_MSG,"is_running: open(${tmp_dir}/.${who}) failed: $!");
    return 0;
  }

  print RUNNING $$;
  close(RUNNING) || &msg($ERROR_MSG,"is_running: close(${tmp_dir}/.${who}) failed: $!");

  $SIG{'INT'} = "util'die_handler";
  $SIG{'QUIT'} = "util'die_handler";
  $SIG{'TERM'} = "util'die_handler";
  $SIG{'HUP'} = "util'parse_cfg_set_hup";

  return 1;
} 

sub die_handler {
  $graceful_death=1;
}

sub parse_cfg_set_hup {
  $parse_cfg_hup=1;
}

sub isit_time_to_die {
  local($tmp_dir, $who) = @_;

  if ($graceful_death) {
    &msg($ERROR_MSG,"isit_time_to_die: graceful death!");
    unlink("${tmp_dir}/.${who}") || &msg($ERROR_MSG,"isit_time_to_die: unlink(${tmp_dir}/.${who}) failed: $!");
  }
  $graceful_death;
}

sub dbn_rename {
  local($src, $dest) = @_;

  # Untaint the source filename.
  $src =~ /(.*)/;
  $src=$1;

  # Untaint the destination filename.
  $dest =~ /(.*)/;
  $dest=$1;

  # Rename the source file to the destination file.  If the rename fails,
  # then try copying the source file to the destination file.  This allows for
  # renaming across filesystems.

  &util'msg($INFO_MSG,"$0: dbn_rename: renaming(${src}, ${dest})");
  if ( ! rename("${src}", "${dest}") ) {
    &util'msg($ERROR_MSG,"$0: dbn_rename: rename(${src}, ${dest}) failed: $! : trying copy()");
    if ( ! copy("${src}", "${dest}") ) {
      &util'msg($ERROR_MSG,"$0: dbn_rename: rename() and copy() failed: $!");
      return 0;
    }
    if ( ! unlink("${src}") ) {
      &util'msg($ERROR_MSG,"dbn_rename:  unlink(${src}) failed: $!");
      return 0;
    }
  }
  return 1;
}


# this is a option parsing sub.  It is used to pull args out
# of an array, and build a hash of these values.  this code
#is really just a hack of the the getopts module.  
#
# The sub requires three arguments, 1: the argument definition
# string,  2: a hash ref for the results, 3: the arg array
#


sub parseopts {
    local($argumentative,$hashref,@arglist) = @_;
    local(@args,$_,$first,$rest);
    local($errs) = 0;

    @args = split( / */, $argumentative );
    while(@arglist && ($_ = $arglist[0]) =~ /^-(.)(.*)/) {
	($first,$rest) = ($1,$2);
	$pos = index($argumentative,$first);
	if($pos >= 0) {
	    if(defined($args[$pos+1]) and ($args[$pos+1] eq ':')) {
		shift(@arglist);
		if($rest eq '') {
		    ++$errs unless @arglist;
		    $rest = shift(@arglist);
		}
                $$hashref{$first} = $rest;
	    }
	    else {
                $$hashref{$first} = 1;
		if($rest eq '') {
		    shift(@arglist);
		}
		else {
		    $arglist[0] = "-$rest";
		}
	    }
	}
	else {
#	    print STDERR "Unknown option: $first\n";
	    ++$errs;
	    if($rest ne '') {
		$arglist[0] = "-$rest";
	    }
	    else {
		shift(@arglist);
	    }
	}
    }
    $errs == 0;
}

sub sort_cmds {
  my($source,$cmd_pattern,$order) = @_;
  my($time_field, $fsn_field, $fn, $junk);
  my(%keylist);

  for ($i=$#$source;$i>=0;$i--) {
    if (!(@$source[$i] =~ /$cmd_pattern/)) {
      splice(@$source,$i,1);
      next;
    }
    ($junk,$junk,$junk,$fsn_field,$junk,$time_field,$junk) = split(/\./,@$source[$i]);
    $keylist{@$source[$i]} = "${time_field}${fsn_field}";
  }
 
  if ($order == 1) {
    @$source = sort { $keylist{$b} <=> $keylist{$a} }  @$source;
  } else {
    @$source = sort { $keylist{$a} <=> $keylist{$b} }  @$source;
  }

}


sub build_alert {
  my ($host,$path,$type,$subtype,$jobname,$optional_file) = @_;
  my ($filedate, $size, $msgcode, $sitecode, $alert_msg_fn);
  my ($alert_msg);

  if ($optional_file && (! -r $optional_file)) {
    &util'msg($util'ERROR_MSG,"$0: optional_file $optional_file not readable: $!: disabling");
    $optional_file="";
  }


  if (!($msgcode=&util'parse("msgcode", "$parse_table",  "$type",  "$subtype",  "$path")))
  {
    &util'msg($util'ERROR_MSG,"$0: parse(msgcode, ${parse_table}, ${type},  ${subtype}, ${path}) failed!");
    return 0;
  }

  if (!($sitecode=&util'parse("sitecode", "$parse_table",  "HOST", "host",  "$host"))) {
    &util'msg($util'ERROR_MSG,"$0: parse(sitecode, ${parse_table}, HOST, host, ${host}) failed!");
     return 0;
  }

  if (!($localsitecode=&util'parse("sitecode", "$parse_table",  "HOST", "host",  "$::hostaddr"))) {
    &util'msg($util'ERROR_MSG,"$0: parse(sitecode, ${parse_table}, HOST, host, ${hostaddr}) failed!");
    $localsitecode = "0000";
  }

  if ($sitecode eq $localsitecode) {
    if (!(($filedate, $size)=&util'build_filedate("$path"))) {
      &util'msg($util'ERROR_MSG,"$0: cannot build filedate");
      return 0;
    }
  } else {
    $filedate = 0;
    $size = 0;
  }

  $alert_msg_fn=&util'build_fn("$::alert", "$sitecode", "$msgcode","0");
  $alert_msg="${::data}/${alert_msg_fn}";

  if (!open(ALERT, "> $alert_msg")) {
    &util'msg($util'ERROR_MSG,"$0: open $alert_msg failed: $!");
    return 0;
  }

  printf ALERT "%s %s %s %s %s %s %s %s %s\n", $host, $path, $filedate, $size, $noop, $noop, $type, $jobname ,$subtype;

  if ($optional_file) {
    if (!open(OPTIONAL_FILE, "$optional_file")) {
      &util'msg($util'ERROR_MSG,"$0: open optional_file $optional_file failed: $!: disabling");
      $optional_file="";
    }

   else {
      while (<OPTIONAL_FILE>) {
        print ALERT;
      }
      close(OPTIONAL_FILE) || &util'msg($util'ERROR_MSG,"$0: close optional_file $optional_file failed: $!");
    }
  }

  close ALERT || &util'msg($util'ERROR_MSG,"$0: close alert_msg $alert_msg failed: $!");
  &util'msg($util'INFO_MSG,"$0: created alert msg: $alert_msg");


  if (!(@cmd_list=&util'parse("quecmd", "$::parse_table", "$::quecmd", "$::alert", "${sitecode}${msgcode}"))) {
    &util'msg($util'ERROR_MSG,"$0: parse(quecmd, $parse_table, $quecmd, $alert, $msgcode) failed!");
    &util'dbn_rename("$alert_msg", "${err_dir}/${alert_msg_fn}");
    return 0;
  }

  if (!&util'quecmd(*cmd_list, $::dbnet_que, $alert_msg, $alert_msg_fn, $parse_table, $msgcode)) {
    &util'msg($util'ERROR_MSG,"$0: quecmd(cmd_list) failed");
    &util'dbn_rename("$alert_msg", "${err_dir}/${alert_msg_fn}");
    return 0;
  }
  return 1;
}

