/*
 * @(#)support.h
 *
 * Author: Luis J. Cano
 * Date:   01/22/96
 *
 * Description: support. 
 *
 *
 */

 
#ifndef SUPPORT__H
#define SUPPORT__H 

#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <errno.h>


/*
 *
 *
 * macros def
 *
 *
 */

#define MAX_LOG_MSG_LEN 2048
#define MAX_LOG_VERB_LEN 128
#define MAX_EXEC_PIPES 16
#define MAX_EXEC_FORKS 16
#define ERR_SVR(x)  (x * -1)
#define ERR_CLNT(x) (x * 1000 * -1)
#define ERR_GEN(x)  (x * 2000 * -1)
#define IF_SUCCESS 0
#ifdef _POSIX_PATH_MAX
#define MAX_PATH _POSIX_PATH_MAX
#endif
#ifndef MAX_PATH
#define MAX_PATH 1024
#endif
#ifdef _POSIX_NAME_MAX
#define MAX_NAME _POSIX_NAME_MAX
#endif
#ifndef MAX_NAME
#define MAX_NAME 256
#endif

/* execution interface types */

#define SUPP_EXNOOP  0
#define SUPP_ALTO    1
#define SUPP_EXEC    2
#define SUPP_FORK    3
#define SUPP_INPIPE  4
#define SUPP_OUTPIPE 5
#define SUPP_IOPIPE  6



/*
 *
 *
 * errno
 *
 *
 */

extern int errno;




/*
 *
 *
 * log message types
 *
 *
 */

typedef enum {
	LOG_ERR,
	LOG_INFO,
	LOG_DBG,
        LOG_STR,
	LOG_MIJI,
        LOG_SLOG_ERR,
        LOG_SLOG_INFO,
        LOG_SLOG_DBG,
	LOG_SLOG_STR,
	LOG_SLOG_MIJI
} logtypeT;



/*
 *
 *
 * utilLog results
 *
 *
 */

typedef struct {
	unsigned str_sz;
	unsigned dbo;
	char *str;
} logbfrT;



	
	
/*
 *
 * 
 * error structures and messages
 *
 *
 */

/*
 * server error messages
 */

typedef enum {
	ERR_SVR_TABLE_START=0,
        ERR_SVR_FOPEN,
	ERR_SVR_FCLOSE,
        ERR_SVR_CFG,
        ERR_SVR_CFG_SYNTAX,
	ERR_SVR_MIJI,
	ERR_SVR_CLNT_NOT_ALLOWD,
	ERR_SVR_INIT,
	ERR_SVR_DEBUG,
	ERR_SVR_VERBOSE,
	ERR_SVR_FWRITE,
	ERR_SVR_FILE_MODE,
	ERR_SVR_CANNOT_BE_ROOT,
	ERR_SVR_TAINTED,
	ERR_SVR_CHMOD,
	ERR_SVR_STAT,
	ERR_SVR_CHOWN,
	ERR_SVR_EXEC_IF,
	ERR_SVR_FORK,
        ERR_SVR_INIT_NET,
	ERR_SVR_TABLE_SIZE
} svc_errT;


#define INIT_SVR_ERR_TBL { \
        NULL, \
	": svr: fopen err: ", \
	": svr: fclose err: ", \
        ": svr: cfg err", \
        ": svr: cfg syntax err", \
	": svr: DANGER!!! Client Authenication Failed, Possible MIJI!!!: ", \
	": svr: DANGER!!! Client Not Allowed, Possible MIJI!!!: ", \
	": svr: server initialization err", \
	": svr: debug: ", \
	": svr: verbose: ", \
	": svr: fwrite err: ", \
	": svr: inv file mode: ", \
	": svr: this code cannot run as root", \
	": svr: path is tainted: ", \
	": svr: chmod err: ", \
	": svr: stat err: ", \
	": svr: chown err: ", \
	": svr: exec_if err: ", \
	": svr: fork err: ", \
	": svr: init network selection mechanism failed: ", \
	NULL \
}




/*
 * client error messages
 */


typedef enum {
        ERR_CLNT_TABLE_START=0,
	ERR_CLNT_USAGE,
        ERR_CLNT_CLNT_CREATE,
	ERR_CLNT_STAT,
	ERR_CLNT_NOT_REG_FILE,
	ERR_CLNT_OFFSET_GT_FSZ,
	ERR_CLNT_LEN_TOO_BIG,
	ERR_CLNT_FOPEN,
	ERR_CLNT_FSEEK,
	ERR_CLNT_MALLOC,
	ERR_CLNT_CALLOC,
	ERR_CLNT_FREAD,
	ERR_CLNT_FCLOSE,
	ERR_CLNT_SVR_NOT_RESPONDING,
	ERR_CLNT_TABLE_SIZE
} clnt_errT;



#define INIT_CLNT_ERR_TBL { \
        NULL, \
        "clnt: usage error", \
        ": clnt: clnt_create failed: ", \
	": clnt: stat err: ", \
	": clnt: not regular file: ", \
	": clnt: offset > filesize: ", \
	": clnt: filesize - offset < len: ", \
	": clnt: fopen err: ", \
	": clnt: fseek err: ", \
	": clnt: malloc err: ", \
	": clnt: calloc err: ", \
	": clnt: fread err: ", \
	": clnt: fclose err: ", \
	": clnt: server not responding", \
	NULL \
}





/*
 * general error messages
 */


typedef enum {
	ERR_GEN_TABLE_START=0,
        ERR_GEN_MALLOC,
        ERR_GEN_CALLOC,
        ERR_GEN_FOPEN,
	ERR_GEN_FCLOSE,
	ERR_GEN_NOENV,
        ERR_GEN_USAGE,
	ERR_GEN_FORK,
	ERR_GEN_OPENDIR,
	ERR_GEN_UTIL_LOG,
	ERR_GEN_STAT,
	ERR_GEN_RENAME,
	ERR_GEN_LOGGER,
	ERR_GEN_TABLE_SIZE
} general_errT;




#define INIT_GEN_ERR_TBL { \
        NULL, \
	": malloc failed: ", \
	": calloc failed: ", \
	": fopen error: ", \
	": fclose error: ", \
	": environment variable not set:", \
	": usage error", \
	": fork error: ", \
	": opendir error: ", \
	": utilLog error", \
	"stat error: ", \
	": rename error: ", \
	": logger error: ", \
	NULL \
}


/*
 * error table structure
 */

typedef const char *err_tblT[];


/*
 * IF_svr allowed clnts 
 */
#define MAXADDRSZ 31
typedef struct allowedClntsElemType {
	char addr[MAXADDRSZ+1];
	char addr_sav[MAXADDRSZ+1];
        char *addr_octet_ptrs[4];
        int  addr_octet_vals[4];
	int uid;
        struct allowedClntsElemType *np;
} allowedClntsElemT; 


/*
 *
 *
 * DBNet Queue Handler Support
 *
 *
 */

typedef enum {
	DBN_EMPTY=0,
	DBN_CLNT,
	DBN_LDM,
	DBN_ALERT,
	DBN_EXEC,
	DBN_PIPE
} DBN_cmdT;


typedef struct {
	unsigned int DBN_VPF_loaded;
	unsigned int DBN_VPF_busy;
	unsigned int DBN_VPF_r2;
	unsigned int DBN_VPF_r3;
	unsigned int DBN_VPF_r4;
	unsigned int DBN_VPF_r5;
	unsigned int DBN_VPF_r6;
	unsigned int DBN_VPF_r7;
} DBN_VPFT;



typedef struct {
	pid_t		VP;
	DBN_VPFT	VPF;
	DBN_cmdT	cmd;
	char		dsitecode[8];
	char		msgcode[8];
	time_t		start;
	time_t		max;
	char		qcmd_path[MAX_PATH+1];
} VCBT;




	
/*
 *
 *
 * function prototypes
 *
 *
 */


/* 
 * util.c
 */

void *utilMalloc(const char *modname, const size_t size, logbfrT *logbfr); 

void *utilCalloc(const char *modname, const size_t nmem, const size_t size, logbfrT *logbfr); 

int utilRename(const char *modname, const char *old, const char *new, logbfrT *logbfr);

int utilLog(const char *modname, const logtypeT logtype, const int *errn, 
            const char *cmsg1, const int *imsg1,
            const char *cmsg2, const int *imsg2,
            const char *cmsg3, const int *imsg3,
            const char *cmsg4, const int *imsg4,
            logbfrT *logbfr);

void utilLogger(const char *logprefix, const char *modname, const int code, logbfrT *logbfr, const unsigned resetlen);



#endif 
