/*
 * @(#)util.c
 *
 * Author: Luis J. Cano
 * Date:   03/12/96
 *
 * Description: utilities
 *
 */

 
#define _POSIX_SOURCE 1

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <sys/types.h>
#include <unistd.h>
#include "support.h"



/* 
 * global
 */

err_tblT err_gen_tbl=INIT_GEN_ERR_TBL;


/*
 * function prototypes
 */

static int updateLogBfr(logbfrT *logbfr, unsigned *dbc, const char *t);




/*
 *
 * utilMalloc
 *
 *
 */


void *utilMalloc(const char *modname, const size_t size, logbfrT *logbfr) {


  const char myname[] = "utilMalloc";

  int errn;
  void *x;

  if ((x=malloc(size))==NULL) {
    errn=ERR_GEN(ERR_GEN_MALLOC); 
    utilLog(myname, ((logbfr==NULL) ? LOG_ERR : LOG_STR), &errn,
          modname, (int *)NULL, 
          err_gen_tbl[ERR_GEN_MALLOC], (int *)NULL,
          "size=", (int *)&size,
          (char *)NULL, (int *)NULL,
          logbfr);
  }
  return x;
} 


/*
 *
 * utilCalloc
 *
 *
 */


void *utilCalloc(const char *modname, const size_t nmem, const size_t size, logbfrT *logbfr) {


  const char myname[] = "utilCalloc";

  int errn;
  void *x;

  if ((x=calloc(nmem, size))==NULL) {
    errn=ERR_GEN(ERR_GEN_CALLOC);
    utilLog(myname, ((logbfr==NULL) ? LOG_ERR : LOG_STR), &errn,
          modname, (int *)NULL,
          err_gen_tbl[ERR_GEN_CALLOC], (int *)NULL,
          "size=", (int *)&size,
          (char *)NULL, (int *)NULL,
          logbfr);
  }
  return x;
}


/*
 *
 * utilRename
 *
 *
 */


int utilRename(const char *modname, const char *old, const char *new, logbfrT *logbfr) {

  const char myname[] = "utilRename";

  int errn, errno_save;

  if (rename(old, new) < 0) {
    errno_save=errno;
    errn=ERR_GEN(ERR_GEN_RENAME);
    utilLog(myname, ((logbfr==NULL) ? LOG_ERR : LOG_STR), &errn,
          modname, (int *)NULL,
          err_gen_tbl[ERR_GEN_RENAME], (int *)NULL,
          strerror(errno_save), (int *)NULL,
          (char *)NULL, (int *)NULL,
          logbfr);
    utilLog(myname, ((logbfr==NULL) ? LOG_ERR : LOG_STR), &errn,
          "old=", (int *)NULL,
          old, (int *)NULL,
          ": new=", (int *)NULL,
          new, (int *)NULL,
          logbfr);
    return errno;
  }
  return IF_SUCCESS;
}


/*
 *
 * utilLog
 *
 *
 */


int utilLog( const char *modname, const logtypeT log_type, const int *errn, 
            const char *cmsg1, const int *imsg1,
            const char *cmsg2, const int *imsg2, 
            const char *cmsg3, const int *imsg3, 
            const char *cmsg4, const int *imsg4, 
            logbfrT *logbfr) {

  unsigned dbc;
  static char token[64];
  int status=IF_SUCCESS;
 
  FILE *stream;

  switch (log_type) {
    case LOG_ERR: case LOG_INFO: case LOG_DBG: case LOG_MIJI:
      if (log_type==LOG_INFO) 
        stream=stdout;
      else
        stream=stderr;
      (void)fprintf(stream, "\n");
      if (errn != NULL)
        (void)fprintf(stream, "%i ", *errn);
      if (modname != NULL)
        (void)fprintf(stream, "%s", modname);
      if (cmsg1 != NULL)
        (void)fprintf(stream, "%s", cmsg1);
      if (imsg1 != NULL)
        (void)fprintf(stream, "%i", *imsg1);
      if (cmsg2 != NULL)
        (void)fprintf(stream, "%s", cmsg2);
      if (imsg2 != NULL)
        (void)fprintf(stream, "%i", *imsg2);
      if (cmsg3 != NULL)
        (void)fprintf(stream, "%s", cmsg3);
      if (imsg3 != NULL)
        (void)fprintf(stream, "%i", *imsg3);
      if (cmsg4 != NULL)
        (void)fprintf(stream, "%s", cmsg4);
      if (imsg4 != NULL)
        (void)fprintf(stream, "%i", *imsg4);
      (void)fflush(stream);
      break;
    case LOG_STR:
      if (logbfr==NULL || logbfr->str_sz<3 || logbfr->str==NULL) {
        status=ERR_GEN_UTIL_LOG; 
        break;
      }
      dbc=logbfr->str_sz;  
      if (*(logbfr->str)!=0x0) 
        logbfr->str_sz=logbfr->dbo;
      else
        logbfr->dbo=logbfr->str_sz=0; 
      if (!updateLogBfr(logbfr, &dbc, (char *)NULL)) {
        status=ERR_GEN_UTIL_LOG;
        break;
      }
      if (errn != NULL) {
        (void)sprintf(token, "%i  ", *errn);
        if (!updateLogBfr(logbfr, &dbc, token)) {
          status=ERR_GEN_UTIL_LOG; 
          break;
        }
      }
      if (modname!=NULL && !updateLogBfr(logbfr, &dbc, modname)) {
        status=ERR_GEN_UTIL_LOG; 
        break;
      }
      if (cmsg1!=NULL && !updateLogBfr(logbfr, &dbc, cmsg1)) {
        status=ERR_GEN_UTIL_LOG;
        break;
      }
      if (imsg1 != NULL) {
        (void)sprintf(token, "%i", *imsg1);
        if (!updateLogBfr(logbfr, &dbc, token)) {
          status=ERR_GEN_UTIL_LOG;
          break;
        }
      }
      if (cmsg2!=NULL && !updateLogBfr(logbfr, &dbc, cmsg2)) {
        status=ERR_GEN_UTIL_LOG;
        break;
      }
      if (imsg2 != NULL) {
        (void)sprintf(token, "%i", *imsg2);
        if (!updateLogBfr(logbfr, &dbc, token)) {
          status=ERR_GEN_UTIL_LOG;
          break; 
        }
      }
      if (cmsg3!=NULL && !updateLogBfr(logbfr, &dbc, cmsg3)) {
        status=ERR_GEN_UTIL_LOG;
        break;
      }
      if (imsg3 != NULL) {
        (void)sprintf(token, "%i", *imsg3);
        if (!updateLogBfr(logbfr, &dbc, token)) {
          status=ERR_GEN_UTIL_LOG;
          break;
        }
      }
      if (cmsg4!=NULL && !updateLogBfr(logbfr, &dbc, cmsg4)) {
        status=ERR_GEN_UTIL_LOG;
        break;
      }
      if (imsg4 != NULL) {
        (void)sprintf(token, "%i", *imsg4);
        if (!updateLogBfr(logbfr, &dbc, token)) {
          status=ERR_GEN_UTIL_LOG;
          break; 
        }
      }
      if (!updateLogBfr(logbfr, &dbc, (char *)NULL)) {
        status=ERR_GEN_UTIL_LOG;
        break;
      }
      logbfr->dbo=logbfr->str_sz;
      logbfr->str_sz=dbc;
      break;
    default:
      status=ERR_GEN_UTIL_LOG;
      break;
  }
  return status; 
}




/*
 *
 * updateLogBfr
 *
 *
 */

static int updateLogBfr(logbfrT *logbfr, unsigned *dbc, 
                        const char *t) {

  unsigned tokensz;
  const char newline[]={ '\n', '\0' };
  char *token;

  token=(t==NULL && (*(logbfr->str+logbfr->str_sz-1) != '\n')) ? 
                      (char *)newline : (char *)t; 
  
  if (token==NULL)
    return 1;
  if ((tokensz=strlen(token))>(*dbc+1)) {
    logbfr->dbo=logbfr->str_sz;
    logbfr->str_sz=*dbc;
    return 0;
  }
  (void)strncpy(logbfr->str+logbfr->str_sz, token, tokensz);
  *dbc-=tokensz;
  logbfr->str_sz+=tokensz;
  *(logbfr->str+logbfr->str_sz)=0x0;
  return 1;
}




/*
 *
 * logger
 *
 */

#define LOGGERSTAGELEN 15
#define RESETLEN(x) ( (x) ? *(logbfr->str)=0x0, logbfr->str_sz=x : x )

void utilLogger(const char *logprefix, const char *modname, const int code, logbfrT *logbfr, const unsigned resetlen) {

  const char myname[] = "utilLogger";

  static char logpath[MAX_PATH+1];
  static char daystage[LOGGERSTAGELEN+1];
  static char timstage[LOGGERSTAGELEN+1];
  static char flgstage[LOGGERSTAGELEN+1];
  time_t now=time((time_t *)NULL);
  static FILE *logfd=NULL;
  static int logprefixoffset=0;
  int errn;
  static pid_t mypid;

  if (logbfr==NULL || logbfr->str==NULL) return;

/*
 * determine log path
 */
  if ( *logpath == 0x0 ) {
    logprefixoffset=strlen(logprefix);
    if (logprefixoffset > MAX_PATH-LOGGERSTAGELEN) {
      errn=ERR_GEN(ERR_GEN_LOGGER);
      utilLog(myname, LOG_ERR, &errn,
        err_gen_tbl[ERR_GEN_LOGGER], (int *)NULL,
        logprefix, (int *)NULL, 
        " size is > MAX_PATH-LOGGERSTAGELEN: ", &logprefixoffset,
        (char *)NULL, (int *)NULL,
        (logbfrT *)NULL);
      RESETLEN(resetlen);
      return;
    }
    (void)strcpy(logpath, logprefix);
    mypid=getpid(); 
  }

/*
 * stage yymmdd and determine if new day
 */
  (void)strftime( daystage, LOGGERSTAGELEN, "%Y%m%d", gmtime(&now) );
  if (strncmp(daystage, flgstage, LOGGERSTAGELEN)!=0) {
    if ((logfd!=NULL) && (fclose(logfd) == EOF)) {
      errn=ERR_GEN(ERR_GEN_FCLOSE);
      utilLog(myname, LOG_ERR, &errn,
        err_gen_tbl[ERR_GEN_FCLOSE], (int *)NULL,
        logpath, (int *)NULL,
        ": logfd: ", (int *)&logfd,
        (char *)NULL, (int *)NULL,
        (logbfrT *)NULL);
    }
    logfd=NULL;
    (void)strncpy(logpath+logprefixoffset, daystage, LOGGERSTAGELEN);
    (void)strncpy(flgstage, daystage, LOGGERSTAGELEN);
  }

/*
 * open the log file (if needed)
 */
  if (logfd==NULL && (logfd=fopen(logpath, "a"))==NULL) {
    errn=ERR_GEN(ERR_GEN_FOPEN);
    utilLog(myname, LOG_ERR, &errn,
      err_gen_tbl[ERR_GEN_FOPEN], (int *)NULL,
      logpath, (int *)NULL, 
      (char *)NULL, (int *)NULL,
      (char *)NULL, (int *)NULL,
      (logbfrT *)NULL);
    RESETLEN(resetlen); 
    return;
  }

/*
 * build timestamp and update log
 */
  (void)strftime( timstage, LOGGERSTAGELEN, "%H%M%S", gmtime(&now) );
  (void)fprintf(logfd, "%s: [%u] %s: %i: %s", timstage, (unsigned)mypid, modname, code, ((*(logbfr->str)=='\n') ? logbfr->str+1 : logbfr->str));
  (void)fflush(logfd);

/* 
 * force the log file to open/close after each log if LOGGERCLOSE
 */ 

#ifdef LOGGERCLOSE 
  if ((logfd!=NULL) && (fclose(logfd) == EOF)) {
    errn=ERR_GEN(ERR_GEN_FCLOSE);
    utilLog(myname, LOG_ERR, &errn,
      err_gen_tbl[ERR_GEN_FCLOSE], (int *)NULL,
      logpath, (int *)NULL,
      ": logfd: ", (int *)&logfd,
      (char *)NULL, (int *)NULL,
      (logbfrT *)NULL);
    }
  logfd=NULL;
#endif

  RESETLEN(resetlen);
  return;
}
