/*
 * @(#)IFP_clnt.c
 *
 * Author: Luis J. Cano
 * Date:   01/18/95
 *
 * Description: InterFace client
 *
 */

#define _RPCGEN_CLNT
#include <rpc/rpc.h>
#include <rpc/pmap_clnt.h>
#include "IFP_rpc.h"

#define _POSIX_SOURCE 1


#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "support.h"

/* 
 * global
 */

err_tblT err_clnt_table=INIT_CLNT_ERR_TBL;


/*
 *
 * 
 * file scope
 *
 *
 *
 */


/* client noop placeholder */
static char     clnt_noop[]={'\0', '\0'};

/* authenicate InterFace */

#ifdef ALERT_AUTHENTICATE
static char	clnt_authenicateInterFace[MAX_AUTHENTICATE_LEN+1];
#else
static char	*clnt_authenicateInterFace=clnt_noop;
#endif

/* remote server */
static char     *remote_host;



/*
 * struct timeval is not posix - might have problems with this
 */

#ifdef IF_CLNT_TIMEOUT
static  struct timeval clnt_timeout_values;
#endif



/* 
 * function prototypes
 */

static int clnt_init(int argc, char *argv[], svr_fparams *svr_fp, logbfrT *logbfr);

static void usage(char *pgm, char *msg, logbfrT *logbfr);

static int set_exec_if(char *exec_if_str);

static void clnt_exit(svr_fparams *svr_fp, CLIENT *cl);




/*
 * 
 *
 * InterFace client
 *
 *
 */



int main(int argc, char *argv[]) {

  CLIENT *cl=NULL;

  static svr_fparams svr_fp;
  svr_result *svr_r=NULL;

  logbfrT logbfr; 
  char    logstr[MAX_LOG_MSG_LEN+1]; 

  int errn;


  logstr[0]=0x0;
  logbfr.str_sz=MAX_LOG_MSG_LEN;
  logbfr.str=logstr;

  if ((errn=clnt_init(argc, argv, &svr_fp, &logbfr))!=IF_SUCCESS) {
    utilLog( (char *)NULL, LOG_INFO, (int *)NULL,
        logbfr.str, (int *)NULL,
        "\n", (int *)NULL,
        (char *)NULL, (int *)NULL,
        (char *)NULL, (int *)NULL,
        (logbfrT *)NULL );
    clnt_exit(&svr_fp, cl);
    exit(EXIT_FAILURE);
  }

  if ((cl=clnt_create(remote_host, IF_PROG, IF_VER, "tcp"))==NULL) {
    errn=ERR_CLNT(ERR_CLNT_CLNT_CREATE);
    utilLog( argv[0], LOG_INFO, &errn,
        err_clnt_table[ERR_CLNT_CLNT_CREATE], (int *)NULL,
	clnt_spcreateerror(remote_host), (int *)NULL,
        (char *)NULL, (int *)NULL,
        (char *)NULL, (int *)NULL,
        (logbfrT *)NULL );
    clnt_exit(&svr_fp, cl);
    exit(EXIT_FAILURE);
  }


#ifdef IF_CLNT_TIMEOUT
  clnt_control(cl, CLSET_TIMEOUT, (char *)&clnt_timeout_values); 
#endif

#if IFP_ENV == 1
  svr_r=interface_svrp_1(&svr_fp, cl);
#elif IFP_ENV == 99
  svr_r=interface_svrp_99(&svr_fp, cl);
#elif IFP_ENV == 100
  svr_r=interface_svrp_100(&svr_fp, cl);
#endif

  if (svr_r==NULL) {
    errn=ERR_CLNT(ERR_CLNT_SVR_NOT_RESPONDING);
    utilLog(argv[0], LOG_INFO, &errn, 
            err_clnt_table[ERR_CLNT_SVR_NOT_RESPONDING], (int *)NULL,
            (char *)NULL, (int *)NULL,
            (char *)NULL, (int *)NULL,
            (char *)NULL, (int *)NULL,
            (logbfrT *)NULL );
     exit(EXIT_FAILURE);
  }

  if (*(svr_r->verbage) != 0x0) 
    utilLog( (char *)NULL, LOG_INFO, (int *)NULL,
	svr_r->verbage, (int *)NULL,
        "\n", (int *)NULL,
        (char *)NULL, (int *)NULL,
        (char *)NULL, (int *)NULL,
        (logbfrT *)NULL );
  clnt_exit(&svr_fp, cl);
  exit(svr_r->code);
/*  exit((svr_r->code==IF_SUCCESS) ? EXIT_SUCCESS : EXIT_FAILURE); */
}







static int clnt_init(int argc, char *argv[], svr_fparams *svr_fp, logbfrT *logbfr) {

  const char myname[]="clnt_init";

/* used by getopt */
  int c;
  extern char *optarg;
  extern int optind;

/* local filename info */
  char *fn=NULL;
  size_t offset=0, len=0;


/* error staging area */
  int errn;
  int errno_save;

/* program info */
  u_long if_ver, if_prog;

/* init server formal parameter list */
  svr_fp->verbose=0;
  svr_fp->debug=0;
  svr_fp->do_this_if=clnt_noop;
  svr_fp->save_pending_file=clnt_noop;
  svr_fp->save_msg_path=clnt_noop;
  svr_fp->save_msg_file_open_flags=clnt_noop;
  svr_fp->save_msg_file_uid=0;
  svr_fp->save_msg_file_gid=0;
  svr_fp->save_msg_file_mode=0;
  svr_fp->exec_if1=SUPP_EXNOOP;
  svr_fp->exec_path1=clnt_noop;
  svr_fp->exec_arg1a=clnt_noop;
  svr_fp->exec_arg1b=clnt_noop;
  svr_fp->exec_arg1c=clnt_noop;
  svr_fp->exec_arg1d=clnt_noop;
  svr_fp->exec_if2=SUPP_EXNOOP;
  svr_fp->exec_path2=clnt_noop;
  svr_fp->exec_arg2a=clnt_noop;
  svr_fp->exec_arg2b=clnt_noop;
  svr_fp->exec_arg2c=clnt_noop;
  svr_fp->exec_arg2d=clnt_noop;

#ifdef ALERT_AUTHENTICATE
  strncpy(clnt_authenicateInterFace, ALERT_AUTHENTICATE, MAX_AUTHENTICATE_LEN);
#endif
  svr_fp->authenicateInterFace=clnt_authenicateInterFace;

  svr_fp->cl_ruid=(int)getuid();
  svr_fp->cl_euid=(int)geteuid();
  svr_fp->cl_rgid=(int)getgid();
  svr_fp->cl_egid=(int)getegid();
  svr_fp->cl_pid=(int)getpid();
  svr_fp->cl_ppid=(int)getppid();
  svr_fp->cl_pgrp=(int)getpgrp();

  svr_fp->msg.msgT_len=0;
  svr_fp->msg.msgT_val=clnt_noop;


#ifdef IF_CLNT_TIMEOUT
  clnt_timeout_values.tv_sec = IF_CLNT_TIMEOUT;
  clnt_timeout_values.tv_usec = 0;
#endif


  while ((c = getopt(argc, argv, "vxr:c:i:e:m:o:l:z:p:f:u:g:d:t:1:B:C:D:E:F:2:I:J:K:L:M:hV")) != EOF) {
    switch (c) {
      case 'v':
        svr_fp->verbose=1;
        break;
      case 'x':
        svr_fp->debug=1;
        break;
      case 'r':
        remote_host=optarg;
        break;
      case 'c':
        if_ver=(u_long)atol(optarg); if_prog=IF_PROG;
        errn=(int)pmap_unset(IF_PROG, if_ver);
        (void)utilLog( argv[0], LOG_STR, (int *)NULL,
		": pmap_unset: ", (int *)&errn,
                ": IF_PROG=", (int *)&if_prog,
                " IF_VER=", (int *)&if_ver,
                (char *)NULL, (int *)NULL,
                logbfr);
        return !IF_SUCCESS;
      case 'i':
        svr_fp->do_this_if=optarg;
        break;
      case 'm':
	fn=optarg;
        break;
      case 'o':
	offset=(size_t)atoi(optarg);
        break;
      case 'l':
	len=(size_t)atoi(optarg);
        break;
      case 'e':
        svr_fp->save_pending_file=optarg;
        break;
      case 'z':
	svr_fp->authenicateInterFace=optarg;
        break;
      case 'p':
	svr_fp->save_msg_path=optarg;
        break;
      case 'f':
	svr_fp->save_msg_file_open_flags=optarg;
        break;
      case 'u':
	svr_fp->save_msg_file_uid=atoi(optarg);
        break;
      case 'g':
	svr_fp->save_msg_file_gid=atoi(optarg);
        break;
      case 'd':
	svr_fp->save_msg_file_mode=atoi(optarg);
        break;
      case 't':
#ifdef IF_CLNT_TIMEOUT
        clnt_timeout_values.tv_sec = atoi(optarg);
#else
        usage(argv[0], "IF_CLNT_TIMEOUT not defined", logbfr);
        return !IF_SUCCESS;
#endif
        break;
      case '1':
	if ((svr_fp->exec_if1=set_exec_if(optarg)) == SUPP_EXNOOP) {
	  usage(argv[0], "inv -1 param", logbfr);
          return !IF_SUCCESS;
        } 
        break;
      case 'B':
	svr_fp->exec_path1=optarg;
        break;
      case 'C':
	svr_fp->exec_arg1a=optarg;
        break;
      case 'D':
	svr_fp->exec_arg1b=optarg;
        break;
      case 'E':
	svr_fp->exec_arg1c=optarg;
        break;
      case 'F':
	svr_fp->exec_arg1d=optarg;
        break;
      case '2':
        if ((svr_fp->exec_if2=set_exec_if(optarg)) == SUPP_EXNOOP) {
          usage(argv[0], "inv -2 param", logbfr);
          return !IF_SUCCESS;
        }
        break;
      case 'I':
	svr_fp->exec_path2=optarg;
        break;
      case 'J':
	svr_fp->exec_arg2a=optarg;
        break;
      case 'K':
	svr_fp->exec_arg2b=optarg;
        break;
      case 'L':
	svr_fp->exec_arg2c=optarg;
        break;
      case 'M':
	svr_fp->exec_arg2d=optarg;
        break;
      case 'h':
        usage(argv[0], "-h help", logbfr);
        return !IF_SUCCESS;
      case 'V':
	if_ver=IF_VER; if_prog=IF_PROG;
	(void)utilLog( argv[0], LOG_STR, (int *)NULL,
		": IF_PROG=", (int *)&if_prog,
		" IF_VER=", (int *)&if_ver,
		(char *)NULL, (int *)NULL,
		(char *)NULL, (int *)NULL,
		logbfr); 
        return !IF_SUCCESS;
      default:
        usage(argv[0], (char *)err_clnt_table[ERR_CLNT_USAGE], logbfr);
        return !IF_SUCCESS;
    }
  }
  if ((argc - optind) > 0) {
    usage(argv[0], (char *)err_clnt_table[ERR_CLNT_USAGE], logbfr);
    return !IF_SUCCESS;
  }



/* ensure remote_host is set */

  if (remote_host==NULL) {
    usage(argv[0], "missing remote host ""-r"" param", logbfr);
    return !IF_SUCCESS;
  }


/* determine msg info */
  if (fn!=NULL) 
  {
  FILE *fd;
  struct stat stat_bfr;
  size_t data_byte_count;

  if (stat(fn, &stat_bfr) != 0) {
    errno_save=errno;
    usage(argv[0], (char *)NULL, logbfr);
    errn=ERR_CLNT(ERR_CLNT_STAT);
    utilLog(myname, LOG_STR, &errn,
            err_clnt_table[ERR_CLNT_STAT], (int *)NULL,
            fn, (int *)NULL,
            ": ", (int *)NULL, 
            strerror(errno_save), (int *)NULL,
            logbfr);
    return !IF_SUCCESS;
  }
  if (!S_ISREG(stat_bfr.st_mode)) {
    usage(argv[0], (char *)NULL, logbfr);
    errn=ERR_CLNT(ERR_CLNT_NOT_REG_FILE);
    utilLog(myname, LOG_STR, &errn,
            err_clnt_table[ERR_CLNT_NOT_REG_FILE], (int *)NULL,
            fn, (int *)NULL,
            (char *)NULL, (int *)NULL,
            (char *)NULL, (int *)NULL,
            logbfr);
    return !IF_SUCCESS;
  }
  if (offset >= (size_t)stat_bfr.st_size) {
    usage(argv[0], (char *)NULL, logbfr);
    errn=ERR_CLNT(ERR_CLNT_OFFSET_GT_FSZ);
    utilLog(myname, LOG_STR, &errn,
            err_clnt_table[ERR_CLNT_OFFSET_GT_FSZ], (int *)NULL,
            fn, (int *)NULL,
            ": offset=", (int *)&offset,
            ": filesz=", (int *)&(stat_bfr.st_size),
            logbfr);
    return !IF_SUCCESS;
  }
  data_byte_count=(size_t)stat_bfr.st_size - offset;
  if ( len==0 )
    len=data_byte_count;
  if ( (data_byte_count < len) || (data_byte_count < 1) ) {
    usage(argv[0], (char *)NULL, logbfr);
    errn=ERR_CLNT(ERR_CLNT_LEN_TOO_BIG);
    utilLog(myname, LOG_STR, &errn,
            err_clnt_table[ERR_CLNT_LEN_TOO_BIG], (int *)NULL,
            fn, (int *)NULL,
            ": filesize-offset=", (int *)&data_byte_count,
            ": len=", (int *)&(len),
            logbfr);
    return !IF_SUCCESS;
  }
  if ((fd=fopen(fn, "r"))==NULL) {
    usage(argv[0], (char *)NULL, logbfr);
    errn=ERR_CLNT(ERR_CLNT_FOPEN);
    utilLog(myname, LOG_STR, &errn,
            err_clnt_table[ERR_CLNT_FOPEN], (int *)NULL,
            fn, (int *)NULL,
            (char *)NULL, (int *)NULL,
            (char *)NULL, (int *)NULL,
            logbfr);
    return !IF_SUCCESS;
  }
  if (fseek(fd, (long)offset, SEEK_SET) != 0) {
    usage(argv[0], (char *)NULL, logbfr);
    errn=ERR_CLNT(ERR_CLNT_FSEEK);
    utilLog(myname, LOG_STR, &errn,
            err_clnt_table[ERR_CLNT_FSEEK], (int *)NULL,
            fn, (int *)NULL,
            (char *)NULL, (int *)NULL,
            (char *)NULL, (int *)NULL,
            logbfr);
    fclose(fd);
    return !IF_SUCCESS;
  }
  if ((svr_fp->msg.msgT_val=malloc(len))==NULL) {
    usage(argv[0], (char *)NULL, logbfr);
    errn=ERR_CLNT(ERR_CLNT_MALLOC);
    utilLog(myname, LOG_STR, &errn,
            err_clnt_table[ERR_CLNT_MALLOC], (int *)NULL,
            fn, (int *)NULL,
            ": len=", (int *)&len,
            (char *)NULL, (int *)NULL,
            logbfr);
    fclose(fd);
    return !IF_SUCCESS;
  }
  if (fread(svr_fp->msg.msgT_val, sizeof(char), len, fd) == EOF) {
    usage(argv[0], (char *)NULL, logbfr);
    errn=ERR_CLNT(ERR_CLNT_FREAD);
    utilLog(myname, LOG_STR, &errn,
            err_clnt_table[ERR_CLNT_FREAD], (int *)NULL,
            fn, (int *)NULL,
            ": len=", (int *)&len,
            (char *)NULL, (int *)NULL,
            logbfr);
    fclose(fd);
    return !IF_SUCCESS;
  }
  if (fclose(fd)==EOF) {
    usage(argv[0], (char *)NULL, logbfr);
    errn=ERR_CLNT(ERR_CLNT_FCLOSE);
    utilLog(myname, LOG_STR, &errn,
            err_clnt_table[ERR_CLNT_FCLOSE], (int *)NULL,
            fn, (int *)NULL,
            ": len=", (int *)&len,
            (char *)NULL, (int *)NULL,
            logbfr);
    return !IF_SUCCESS;
  }
  svr_fp->msg.msgT_len=len;

  } 

/* end of determine msg info */


  return IF_SUCCESS;
}


static int set_exec_if(char *exec_if_str) {

  if (strcmp(exec_if_str, "alto")==0)
    return SUPP_ALTO;
  if (strcmp(exec_if_str, "exec")==0)
    return SUPP_EXEC;
  if (strcmp(exec_if_str, "fork")==0)
    return SUPP_FORK;
  if (strcmp(exec_if_str, "inpipe")==0)
    return SUPP_INPIPE;
  if (strcmp(exec_if_str, "outpipe")==0)
    return SUPP_OUTPIPE;
  if (strcmp(exec_if_str, "iopipe")==0)
    return SUPP_IOPIPE;
  return SUPP_EXNOOP;
}




static void usage(char *pgm, char *msg, logbfrT *logbfr) {

  int errn;
  const char usagemsg[]="\
[vxc:i:e:m:o:l:z:p:f:u:g:d:t:1:B:C:D:E:F:2:I:J:K:L:M:hV] -r svr_host\
\n\t-v    verbose\
\n\t-x    debug\
\n\t-i    do_this_if               do this if file exists on server\
\n\t-m    message_path             local message path\
\n\t-o    message_byte_offset      local message byte offset\
\n\t-l    message_byte_len         local message byte len\
\n\t-e    pending_file             save pending file on server\
\n\t-p    svr_msg_path             save msg path on server\
\n\t-f    svr_msg_file_open_flags  save msg on server file open flags\
\n\t-u    svr_msg_file_uid         save msg on server uid\
\n\t-g    svr_msg_file_gid         save msg on server gid\
\n\t-d    svr_msg_file_mode        save msg on server mode\
\n\t-t    clnt_timeout_val         client timeout value\
\n\t-12   exec_interface           exec|fork|inpipe|outpipe|iopipe|alto\
\n\t-BI   exec_path                execution path on server\
\n\t-CJ   exec_arg1                execution arg1 on server\
\n\t-DK   exec_arg2                execution arg2 on server\
\n\t-EL   exec_arg3                execution arg3 on server\
\n\t-FM   exec_arg4                execution arg4 on server\
\n\t-r    svr_host                 hostname of server\
\n\t-V    version                  client rpc information\
\n\t-h    help\n\
";


  if (logbfr == NULL) {
    (void)fprintf(stderr, "\nusage: %s %s", pgm, usagemsg);
    if (msg != NULL)
      (void)fprintf(stderr, "\n%s\n", msg);
  }
  else {
    errn=ERR_CLNT(ERR_CLNT_USAGE);
    utilLog("usage: pgmname ", LOG_STR, &errn,
        usagemsg, (int *)NULL,
        (msg!=NULL) ? "\n": (char *)NULL, (int *)NULL,
        msg, (int *)NULL,
        (msg!=NULL) ? "\n": (char *)NULL, (int *)NULL,
        logbfr );
  }
  return;
}



static void clnt_exit(svr_fparams *svr_fp, CLIENT *cl) {
  if (cl != NULL) 
    clnt_destroy(cl);

}
