/*
 * @(#)IFP_svr.c
 *
 * Author: Luis J. Cano
 * Date:   01/18/95
 *
 * Description: InterFace Server
 *
 */

#define _RPCGEN_SVC
#include <rpc/rpc.h>
#include "IFP_rpc.h"

#define _POSIX_SOURCE 1

#include <string.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <unistd.h>
#include <signal.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include "support.h"

/*
 * PARANOID mode
 */
const int paranoid=PARANOID; 

/*
 * macros
 */
#define DBNROOT_ENV "DBNROOT"
#define PENDING  "/pending/"
#define PENDINGH "/pending/."
#define FILE_FLAGS_DEFAULT "w"
#define IFS_trace 1

/*
 *
 * filescope 
 *
 */

/* 
 * err table
 */
static err_tblT err_svr_table=INIT_SVR_ERR_TBL;

/*
 * logger path
 */
static char loggerpath[MAX_PATH+1];

/*
 * debug and verbose
 */
static int debug; 
static int verbose;

/*
 * init flag
 */
static int inited=0;

/*
 * allow table and current clnt
 */
static allowedClntsElemT *allowedClnts;
#define CLNTNETSIZE 63
static char currentClnt[CLNTNETSIZE+1];
static int initallow=1;

/*
 * dbn root
 */
static char *dbn_root=NULL;
static char default_dbn_root[]="/usr/DBNet";



/*
 * function prototypes
 */

static int svr_save_msg( char *msg, unsigned len,
                         char *path, char *flags,
                         logbfrT *logbfr );

static int svr_set_fmode( char *path,
                          uid_t uid, gid_t gid, int mode,
                          logbfrT *logbfr );

static int svr_save_pending( char *msg, unsigned len,
                             char *pending,
                             logbfrT *logbfr );

static int exec_funcs(int exec_if,
                      char *exec_path,
                      char *exec_arga,
                      char *exec_argb,
                      char *exec_argc,
                      char *exec_argd,
                      logbfrT *logbfr );

static void build_argv(char *argv[], char *path, char *arga, char *argb, char *argc, char *argd);

static int do_this_if(char *path, logbfrT *logbfr);

static void die(int s);

static void hup(int s);

/* 
 *
 * InterFace Server entry point
 *
 */

#if defined(linux2) || defined(linux22) || defined(sunos5) || defined(hpux11)
#define SVR_INTERFACE_PROD interface_svrp_1_svc
#define SVR_INTERFACE_DEV interface_svrp_99_svc
#define SVR_INTERFACE_NPDS interface_svrp_100_svc
#else
#define SVR_INTERFACE_PROD interface_svrp_1
#define SVR_INTERFACE_DEV interface_svrp_99
#define SVR_INTERFACE_NPDS interface_svrp_100
#endif


#if IFP_ENV == 1
svr_result *SVR_INTERFACE_PROD(svr_fparams *fparams, struct svc_req *SVR_req) {
#elif IFP_ENV == 99
svr_result *SVR_INTERFACE_DEV(svr_fparams *fparams, struct svc_req *SVR_req) {
#elif IFP_ENV == 100
svr_result *SVR_INTERFACE_NPDS(svr_fparams *fparams, struct svc_req *SVR_req) {
#endif

  const char myname[]="InterFace_svrp";

  static logbfrT logbfr, logbfrv;
  static char logstr[MAX_LOG_MSG_LEN+1];
  static char logstrv[MAX_LOG_VERB_LEN+1];
  static svr_result svr_res;
  static int errn, *rtn;
  const char logprefix[]="/log/IFS.log.";
  static char allowpath[MAX_PATH+1];
  const char allowpathfile[]="/table/.a";
  static FILE *allowfd;
  static allowedClntsElemT **nextAllowed;
  static allowedClntsElemT stageAllowed;
  static char addr_stage[MAXADDRSZ+1];
  static int uid_stage;
  
  int i, j, k, l, found=0;

#if defined(sunos5) || defined(hpux11)
#include <netconfig.h>
#include <netdir.h>
  struct netbuf *netbufp; 
  struct netconfig *nconf;
  char *uaddr;
#else
  struct sockaddr_in *sa_in=svc_getcaller(SVR_req->rq_xprt); 
#endif

  logstr[0]=0x0;
  logstrv[0]=0x0;
  logbfr.str=logstr;
  logbfrv.str=logstrv;
  logbfr.str_sz=MAX_LOG_MSG_LEN;
  logbfrv.str_sz=MAX_LOG_VERB_LEN;
  svr_res.verbage=logstr;
  rtn = &svr_res.code;


/*
 * initialize
 */

  if (!inited) {

/*
 * setup dbn_root
 */
    dbn_root=getenv(DBNROOT_ENV);
    if (dbn_root==NULL) {
      /*  (void)fprintf(stderr, "%s environment variable %s not set!: defaulting to %s\n", 
          myname, DBNROOT_ENV, default_dbn_root); */
      dbn_root=default_dbn_root;
    }

/*
 * setup logger path
 */
    (void)strcpy(loggerpath, dbn_root);
    (void)strcat(loggerpath, (*(dbn_root+strlen(dbn_root)-1)=='/') ? logprefix+1 : logprefix);

/*
 * not allowed to run as root
 */
   if (geteuid() == 0 || getuid() == 0) {
      errn=ERR_SVR(ERR_SVR_CANNOT_BE_ROOT);
      (void)utilLog( myname, LOG_STR, &errn,
               err_svr_table[ERR_SVR_CANNOT_BE_ROOT], (int *)NULL,
               (char *)NULL, (int *)NULL,
               (char *)NULL, (int *)NULL,
               (char *)NULL, (int *)NULL,
               &logbfr );
       utilLogger(loggerpath, myname, ERR_SVR_CANNOT_BE_ROOT, &logbfr, 0);
       exit(EXIT_FAILURE);
    }

/*
 * setup allow path
 */
    (void)strcpy(allowpath, dbn_root);
    (void)strcat(allowpath, (*(dbn_root+strlen(dbn_root)-1)=='/') ? allowpathfile+1 : allowpathfile);
   
/*
 * all done
 */
    inited=1;
  }

/*
 * init allow table
 */
  if (initallow) {
    int initrtn;

    while (allowedClnts!=NULL) {
      nextAllowed=&(allowedClnts->np);
      free(allowedClnts);
      allowedClnts=*nextAllowed;
    }
    allowfd=fopen(allowpath, "r");
    if (allowfd == NULL) {
      errn=ERR_SVR(ERR_SVR_FOPEN);
      (void)utilLog( myname, LOG_STR, &errn,
               err_svr_table[ERR_SVR_FOPEN], (int *)NULL,
               "cannot open allow file: ", (int *)NULL,
               allowpath, (int *)NULL,
               (char *)NULL, (int *)NULL,
               &logbfr );
       utilLogger(loggerpath, myname, ERR_SVR_FOPEN, &logbfr, 0);
       exit(EXIT_FAILURE);
    } 
    nextAllowed=&allowedClnts;
    while ( initallow ) {
      initrtn=fscanf(allowfd, "%s %i", addr_stage, &uid_stage);
      if (initrtn == 2) {
        if ((*nextAllowed = utilCalloc(myname, 1, sizeof(allowedClntsElemT), (logbfrT *)NULL)) == NULL)
          exit(EXIT_FAILURE);
        strcpy((*nextAllowed)->addr, addr_stage);
        (*nextAllowed)->uid = uid_stage;
        strcpy((*nextAllowed)->addr_sav, (*nextAllowed)->addr);
        k=strlen((*nextAllowed)->addr_sav);
        for (i=0, j=0, l=0; i<k && l<4; j++) {
          if ((*nextAllowed)->addr_sav[j] == '.' || l==3) {
            if (l != 3)
              (*nextAllowed)->addr_sav[j] = 0x0;
            (*nextAllowed)->addr_octet_ptrs[l] = &(*nextAllowed)->addr_sav[i];
            i=j+1;
            (*nextAllowed)->addr_octet_vals[l] = atoi((*nextAllowed)->addr_octet_ptrs[l]);
            l++;
          }
        }
      nextAllowed=&((*nextAllowed)->np);
      }  
      else {
        initallow=0;
        (void)fclose(allowfd);
      }
    }
  }


/*
 * setup signals
 */
    signal(SIGINT, die);
    signal(SIGQUIT, die);
    signal(SIGTERM, die);
    signal(SIGHUP, hup);

/*
 *
 * authenicate
 *
 */

/*
 * get client hostname
 */

#if defined(sunos5) || defined(hpux11)
  /* Get Clients network address */
  if ((netbufp=svc_getrpccaller(SVR_req->rq_xprt)) == (struct netbuf *)NULL) {
    errn=ERR_SVR(ERR_SVR_INIT_NET);
    (void)utilLog( myname, LOG_STR, &errn,
             err_svr_table[ERR_SVR_INIT_NET], (int *)NULL,
             ": netbufp=svc_getrpccaller(SVR_req->rq_xprt) failed", (int *)NULL,
             (char *)NULL, (int *)NULL,
             (char *)NULL, (int *)NULL,
             &logbfr );
    utilLogger(loggerpath, myname, ERR_SVR_INIT_NET, &logbfr, MAX_LOG_MSG_LEN);
    svr_res.code=1;
    return &svr_res;
  }

  /* Network Selection */
  if ((nconf = getnetconfigent(SVR_req->rq_xprt->xp_netid)) == NULL) { 
    errn=ERR_SVR(ERR_SVR_INIT_NET);
    (void)utilLog( myname, LOG_STR, &errn,
             err_svr_table[ERR_SVR_INIT_NET], (int *)NULL,
             ": nconf = getnetconfigent(", (int *)NULL,
             (char *)SVR_req->rq_xprt->xp_netid, (int *)NULL,
             ") failed", (int *)NULL,
             &logbfr );
    utilLogger(loggerpath, myname, ERR_SVR_INIT_NET, &logbfr, MAX_LOG_MSG_LEN);
    svr_res.code=1;
    return &svr_res;
  }

  /* 
   * Convert the transport-specific address into universal address notation. 
   */
  if ((uaddr = taddr2uaddr(nconf, netbufp)) == NULL) {
    freenetconfigent(nconf);
    errn=ERR_SVR(ERR_SVR_INIT_NET);
    (void)utilLog( myname, LOG_STR, &errn,
             err_svr_table[ERR_SVR_INIT_NET], (int *)NULL,
             ": uaddr = taddr2uaddr(nconf, netbufp) failed", (int *)NULL,
             (char *)NULL, (int *)NULL,
             (char *)NULL, (int *)NULL,
             &logbfr );
    utilLogger(loggerpath, myname, ERR_SVR_INIT_NET, &logbfr, MAX_LOG_MSG_LEN);
    svr_res.code=1;
    return &svr_res;
  }
  for (i=0, j=0, k=0; i<CLNTNETSIZE && j<4; i++) {
    switch(j) {
      case 0: case 1: case 2:
        currentClnt[k++]=uaddr[i];
        if (uaddr[i] == '.') {
          j++;
        }
        break;
      case 3:
        if (isdigit(uaddr[i])) {
          currentClnt[k++]=uaddr[i];
        } else {
          currentClnt[k++]=0x0;
          j++;
        }
        break;
    }
  }
  freenetconfigent(nconf);
  free(uaddr);

/*  errn=ERR_SVR(ERR_SVR_DEBUG);
  (void)utilLog(myname, LOG_STR, &errn,
        err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
        " : netbufp->maxlen = ", (int *)&(netbufp->maxlen),
        " : netbufp->len = ", (int *)&(netbufp->len),
        (char *)NULL, (int *)NULL,
        &logbfr);
*/

#else
  (void)sprintf(currentClnt, "%s", (char *)inet_ntoa(sa_in->sin_addr)); 
#endif
  

/*
 * log request from client
 */
  (void)utilLog( myname, LOG_STR, (int *)NULL,
     ": request fm: ", (int *)NULL,
     currentClnt, (int *)NULL,
     ": paranoid mode: ", (int *)&paranoid, 
     (char *)NULL, (int *)NULL,
     &logbfr );
  (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
       "cl_ruid = ", &fparams->cl_ruid,
     ": cl_euid = ", &fparams->cl_euid,
     ": cl_rgid = ", &fparams->cl_rgid,
     ": cl_egid = ", &fparams->cl_egid,
     &logbfr );
  (void)utilLog( (char *)NULL, LOG_STR, (int *)NULL,
        "cl_pid = ", &fparams->cl_pid,
     ": cl_ppid = ", &fparams->cl_ppid,
     ": cl_pgrp = ", &fparams->cl_pgrp,
     (char *)NULL, (int *)NULL,
     &logbfr );

/*  utilLogger(loggerpath, myname, IF_SUCCESS, &logbfr, MAX_LOG_MSG_LEN); */


#ifdef ALERT_AUTHENTICATE
  if (strcmp(ALERT_AUTHENTICATE, fparams->authenicateInterFace)!=0) {
    errn=ERR_SVR(ERR_SVR_MIJI);
    (void)utilLog( myname, LOG_STR, &errn,
             err_svr_table[ERR_SVR_MIJI], (int *)NULL,
             currentClnt, (int *)NULL,
             ": invalid auth=", (int *)NULL,
             fparams->authenicateInterFace, (int *)NULL,
             &logbfr );
    utilLogger(loggerpath, myname, ERR_SVR_MIJI, &logbfr, MAX_LOG_MSG_LEN);
    svr_res.code=0;
    return &svr_res;

  }
#endif
 
/*
 * is this client allowed to talk to me?
 */ 
  nextAllowed=&allowedClnts;
  while ((*nextAllowed!=NULL) && !found) {
    strcpy(stageAllowed.addr_sav, currentClnt );
    k=strlen(stageAllowed.addr_sav);
    for (i=0, j=0, l=0; i<k && l<4; j++) {
      if ((stageAllowed.addr_sav[j] == '.') || l==3) {
        if (l != 3)
          stageAllowed.addr_sav[j] = 0x0;
        stageAllowed.addr_octet_ptrs[l] = &stageAllowed.addr_sav[i];
        i=j+1;
        stageAllowed.addr_octet_vals[l] = atoi(stageAllowed.addr_octet_ptrs[l]);
        l++;
      }
    }
    if ( 
         ( ( (*nextAllowed)->uid                == 0 )   ||
           ( (*nextAllowed)->uid                == fparams->cl_ruid ) )                &&
         ( ( (*nextAllowed)->uid                == 0 )   ||
           ( (*nextAllowed)->uid                == fparams->cl_euid ) )                &&
         ( ( (*nextAllowed)->addr_octet_vals[0] == 0 )   ||
           ( (*nextAllowed)->addr_octet_vals[0] == stageAllowed.addr_octet_vals[0] ) ) &&
         ( ( (*nextAllowed)->addr_octet_vals[1] == 0 )   ||
           ( (*nextAllowed)->addr_octet_vals[1] == stageAllowed.addr_octet_vals[1] ) ) &&
         ( ( (*nextAllowed)->addr_octet_vals[2] == 0 )   ||
           ( (*nextAllowed)->addr_octet_vals[2] == stageAllowed.addr_octet_vals[2] ) ) &&
         ( ( (*nextAllowed)->addr_octet_vals[3] == 0 )   ||
           ( (*nextAllowed)->addr_octet_vals[3] == stageAllowed.addr_octet_vals[3] ) )  
                                                                                         )
      found=1;
    else
      nextAllowed=&(*nextAllowed)->np;
  }
 
  if (!found) {
    errn=ERR_SVR(ERR_SVR_CLNT_NOT_ALLOWD);
    (void)utilLog( myname, LOG_STR, &errn,
             err_svr_table[ERR_SVR_CLNT_NOT_ALLOWD], (int *)NULL,
             currentClnt, (int *)NULL,
             (char *)NULL, (int *)NULL,
             (char *)NULL, (int *)NULL,
             &logbfr );
    utilLogger(loggerpath, myname, ERR_SVR_CLNT_NOT_ALLOWD, &logbfr, MAX_LOG_MSG_LEN);
    svr_res.code=0;
    return &svr_res;

  }


/* debug */
  debug=fparams->debug;
  verbose=fparams->verbose;

/*  errn=ERR_SVR(ERR_SVR_DEBUG);
  (void)utilLog(myname, LOG_STR, &errn,
        err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
        "entry", (int *)NULL,
        ": paranoid mode: ", (int *)&paranoid,
        (char *)NULL, (int *)NULL,
        &logbfr); */

  if (verbose || debug) {
    errn=ERR_SVR(ERR_SVR_VERBOSE);
    (void)utilLog(myname, LOG_STR, &errn,
      err_svr_table[ERR_SVR_VERBOSE], (int *)NULL,
      "entry", (int *)NULL,
      ": paranoid mode: ", (int *)&paranoid,
      (char *)NULL, (int *)NULL,
      &logbfrv);
  }

/*
 * do this if the following file exists on server:
 *  do_this_if();
 */
  if (*(fparams->do_this_if)!=0x0) {
    if (IFS_trace) {
      errn = ERR_SVR(ERR_SVR_DEBUG);
      (void)utilLog( myname, LOG_STR, &errn,
             err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
             "calling do_this_if", (int *)NULL,
             (char *)NULL, (int *)NULL,
             (char *)NULL, (int *)NULL,
             &logbfr);
    }
    *rtn=do_this_if(fparams->do_this_if, &logbfr);
    if (*rtn != IF_SUCCESS) {
      utilLogger(loggerpath, myname, *rtn, &logbfr, 0);
      return &svr_res;
    }
  }


/*
 * call the following if there is a msg from the client: 
 *   svr_save_msg()
 *   svr_save_pending()
 *
 */

  if (fparams->msg.msgT_len > 0) {

   /* call svr_save_msg */
    if (IFS_trace) {
      errn = ERR_SVR(ERR_SVR_DEBUG);
      (void)utilLog( myname, LOG_STR, &errn,
             err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
             "calling svr_save_msg", (int *)NULL,
             (char *)NULL, (int *)NULL,
             (char *)NULL, (int *)NULL,
             &logbfr);
    }
    if (!paranoid) {
      *rtn=svr_save_msg(fparams->msg.msgT_val, 
                       (unsigned)fparams->msg.msgT_len,
                       fparams->save_msg_path,
                       fparams->save_msg_file_open_flags,
                       &logbfr);
      if (*rtn != IF_SUCCESS) {
        utilLogger(loggerpath, myname, *rtn, &logbfr, 0);
        return &svr_res;
      }
    }

 
    /* call svr_save_pending */
    if (IFS_trace) {
      errn = ERR_SVR(ERR_SVR_DEBUG);
      (void)utilLog( myname, LOG_STR, &errn,
               err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
               "calling svr_save_pending", (int *)NULL,
               (char *)NULL, (int *)NULL,
               (char *)NULL, (int *)NULL,
               &logbfr);
    }
    *rtn=svr_save_pending(fparams->msg.msgT_val,
                     (unsigned)fparams->msg.msgT_len,
                     fparams->save_pending_file,
                     &logbfr);
    if (*rtn != IF_SUCCESS) {
      utilLogger(loggerpath, myname, *rtn, &logbfr, 0);
      return &svr_res;
    }

  } /* end of calling functions when there is a msg from the client */
  


/*
 * call the following if the client has indicated a path on the server: 
 *    svr_set_fmode() 
 *
 */

  if (*(fparams->save_msg_path)!=0x0) {

    /* call svr_set_fmode */
    if (IFS_trace) {
      errn = ERR_SVR(ERR_SVR_DEBUG);
      (void)utilLog( myname, LOG_STR, &errn,
               err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
               "calling svr_set_fmode", (int *)NULL,
               (char *)NULL, (int *)NULL,
               (char *)NULL, (int *)NULL,
               &logbfr);
    }
    if (!paranoid) {
      *rtn=svr_set_fmode( fparams->save_msg_path,
                          (uid_t)fparams->save_msg_file_uid,
                          (gid_t)fparams->save_msg_file_gid,
                          fparams->save_msg_file_mode,
                          &logbfr );
      if (*rtn != IF_SUCCESS) {
        utilLogger(loggerpath, myname, *rtn, &logbfr, 0);
        return &svr_res;
      }
    }  

  } /* end of calling functions when client has indicated a path on the server */


/*
 * call exec functions:
 *    exec_funcs()
 *
 */

  if (fparams->exec_if1 != SUPP_EXNOOP && *(fparams->exec_path1) != 0x0) {
    if (IFS_trace) {
      errn = ERR_SVR(ERR_SVR_DEBUG);
      (void)utilLog( myname, LOG_STR, &errn,
               err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
               "calling exec_funcs for exec_if1: ", (int *)&fparams->exec_if1,
               ": ", (int *)NULL,
               fparams->exec_path1, (int *)NULL,
               &logbfr);
      (void)utilLog( "arg1,arg2: ", LOG_STR, &errn,
               fparams->exec_arg1a, (int *)NULL,
               ", ", (int *)NULL,
               fparams->exec_arg1b, (int *)NULL,
               ", ", (int *)NULL,
               &logbfr);
      (void)utilLog( "arg3,arg4: ", LOG_STR, &errn,
               fparams->exec_arg1c, (int *)NULL,
               ", ", (int *)NULL,
               fparams->exec_arg1d, (int *)NULL,
               ", ", (int *)NULL,
               &logbfr);
    }
    if (!paranoid) {
      *rtn=exec_funcs(fparams->exec_if1,
                      fparams->exec_path1,
                      fparams->exec_arg1a,
                      fparams->exec_arg1b,
                      fparams->exec_arg1c,
                      fparams->exec_arg1d,
                      &logbfr );
      if (*rtn != 0) {
        utilLogger(loggerpath, myname, *rtn, &logbfr, 0);
        return &svr_res;
      }
    } 
  }


  if (fparams->exec_if2 != SUPP_EXNOOP  && *(fparams->exec_path2) != 0x0) {
    if (IFS_trace) {
      errn = ERR_SVR(ERR_SVR_DEBUG);
      (void)utilLog( myname, LOG_STR, &errn,
               err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
               "calling exec_funcs for exec_if2: ", (int *)&fparams->exec_if2,
               ": ", (int *)NULL,
               fparams->exec_path2, (int *)NULL,
               &logbfr);
      (void)utilLog( "arg1,arg2: ", LOG_STR, &errn,
               fparams->exec_arg2a, (int *)NULL,
               ", ", (int *)NULL,
               fparams->exec_arg2b, (int *)NULL,
               ", ", (int *)NULL,
               &logbfr);
      (void)utilLog( "arg3,arg4: ", LOG_STR, &errn,
               fparams->exec_arg2c, (int *)NULL,
               ", ", (int *)NULL,
               fparams->exec_arg2d, (int *)NULL,
               ", ", (int *)NULL,
               &logbfr);
    }
    if (!paranoid) {
      *rtn=exec_funcs(fparams->exec_if2,
                      fparams->exec_path2,
                      fparams->exec_arg2a,
                      fparams->exec_arg2b,
                      fparams->exec_arg2c,
                      fparams->exec_arg2d,
                      &logbfr );
      if (*rtn != 0) {
        utilLogger(loggerpath, myname, *rtn, &logbfr, 0);
        return &svr_res;
      }
    }

  } /* end of calling exec functions */


/*
 * exit server
 */
  if (verbose) {
    errn=ERR_SVR(ERR_SVR_VERBOSE);
    (void)utilLog(myname, LOG_STR, &errn,
       err_svr_table[ERR_SVR_VERBOSE], (int *)NULL,
       "exit", (int *)NULL,
       (char *)NULL, (int *)NULL,
       (char *)NULL, (int *)NULL,
       &logbfrv);
  }

  errn=ERR_SVR(ERR_SVR_DEBUG);
  (void)utilLog(myname, LOG_STR, &errn,
       err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
       "exit", (int *)NULL,
       (char *)NULL, (int *)NULL,
       (char *)NULL, (int *)NULL,
       &logbfr);

  utilLogger(loggerpath, myname, *rtn, &logbfr, ((debug) ? 0 : MAX_LOG_MSG_LEN));
  if (!debug && verbose)
    svr_res.verbage=logstrv;
  return &svr_res;

}

  

static int svr_save_msg( char *msg, unsigned len, 
                         char *path, char *flags,
                         logbfrT *logbfr ) {

  const char myname[]="svr_save_msg";

  FILE *fd;
  int errn;

  if (len<1 || *path==0x0) {
    if (IFS_trace) {
       errn=ERR_SVR(ERR_SVR_DEBUG);
       (void)utilLog(myname, LOG_STR, &errn,
         err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
         "len<1 || *path==0x0: len=", (int *)&len,
         (char *)NULL, (int *)NULL,
         (char *)NULL, (int *)NULL,
         logbfr);
    }
    return IF_SUCCESS;
  }
 
  if ((fd=fopen(path, (*flags==0x0) ? FILE_FLAGS_DEFAULT : flags)) == NULL) {
    errn=ERR_SVR(ERR_SVR_FOPEN);
    (void)utilLog(myname, LOG_STR, &errn,
            err_svr_table[ERR_SVR_FOPEN], (int *)NULL,
            path, (int *)NULL,
            ": flags=", (int *)NULL,
            (*flags==0x0) ? FILE_FLAGS_DEFAULT : flags, (int *)NULL,
            logbfr);
    return !IF_SUCCESS;
  }
  else if (IFS_trace || verbose) {
    errn=ERR_SVR(ERR_SVR_VERBOSE);
    (void)utilLog(myname, LOG_STR, &errn,
            err_svr_table[ERR_SVR_VERBOSE], (int *)NULL,
            path, (int *)NULL,
            ": flags=", (int *)NULL,
            (*flags==0x0) ? FILE_FLAGS_DEFAULT : flags, (int *)NULL,
            logbfr);
  }

  if (fwrite(msg, sizeof(char), len, fd) != len) {
    errn=ERR_SVR(ERR_SVR_FWRITE);
    (void)utilLog(myname, LOG_STR, &errn,
            err_svr_table[ERR_SVR_FWRITE], (int *)NULL,
            path, (int *)NULL,
            ": len=", (int *)&len,
            (char *)NULL, (int *)NULL,
            logbfr);
    if (fclose(fd)==EOF) {
      errn=ERR_SVR(ERR_SVR_FCLOSE);
      (void)utilLog(myname, LOG_STR, &errn,
              err_svr_table[ERR_SVR_FCLOSE], (int *)NULL,
              path, (int *)NULL,
              ": len=", (int *)&len,
              (char *)NULL, (int *)NULL,
              logbfr);
    }
    return !IF_SUCCESS;
  }
  
  if (fclose(fd)==EOF) {
    errn=ERR_SVR(ERR_SVR_FCLOSE);
    (void)utilLog(myname, LOG_STR, &errn,
            err_svr_table[ERR_SVR_FCLOSE], (int *)NULL,
            path, (int *)NULL,
            ": len=", (int *)&len,
            (char *)NULL, (int *)NULL,
            logbfr);
    return !IF_SUCCESS;
  }
  return IF_SUCCESS;
}


static int svr_save_pending( char *msg, unsigned len,
                             char *pending,
                             logbfrT *logbfr ) {
  const char myname[]="svr_save_pending";

  static char pending_path[MAX_PATH_LEN+1];
  static char pendingh_path[MAX_PATH_LEN+1];
  static int pending_dir_len, pendingh_dir_len;
  int i, errn, rtn;


  if (len<1 || *pending==0x0) {
    if (IFS_trace) {
       errn=ERR_SVR(ERR_SVR_DEBUG);
       (void)utilLog(myname, LOG_STR, &errn,
         err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
         "len<1 || *pending==0x0: len=", (int *)&len,
         (char *)NULL, (int *)NULL,
         (char *)NULL, (int *)NULL,
         logbfr);
    }
    return IF_SUCCESS;
  }

/*
 * init pending_path(s)
 */
  if (*pending_path == 0x0) {
    (void)strcpy(pending_path, dbn_root);
    (void)strcat(pending_path, PENDING);
    pending_dir_len=strlen(pending_path);
  }
  if (*pendingh_path == 0x0) {
    (void)strcpy(pendingh_path, dbn_root);
    (void)strcat(pendingh_path, PENDINGH);
    pendingh_dir_len=strlen(pendingh_path);
  } 
  *(pending_path+pending_dir_len)=0x0;
  *(pendingh_path+pendingh_dir_len)=0x0;


/*
 * untaint
 */
  for (i=0; pending[i] != 0x0; i++) {
    if (pending[i]=='/') {
      errn=ERR_SVR(ERR_SVR_TAINTED);
      (void)utilLog( myname, LOG_STR, &errn,
               err_svr_table[ERR_SVR_TAINTED], (int *)NULL,
               pending, (int *)NULL,
               (char *)NULL, (int *)NULL,
               (char *)NULL, (int *)NULL,
               logbfr );
      return !IF_SUCCESS;
    }
  }

/*
 * build pending_path(s)
 */
  (void)strcat(pending_path+pending_dir_len, pending);
  (void)strcat(pendingh_path+pendingh_dir_len, pending);

/*
 * save the msg
 */

  rtn=svr_save_msg( msg, len, pendingh_path,  FILE_FLAGS_DEFAULT, logbfr);
  if (rtn==IF_SUCCESS) 
    rtn=svr_set_fmode(pendingh_path, (uid_t)0, (gid_t)0, (int)664, logbfr);
  if (rtn==IF_SUCCESS) 
    rtn=utilRename(myname, pendingh_path, pending_path, logbfr); 
  return rtn;
}


static int svr_set_fmode( char *path, 
                          uid_t uid, gid_t gid, int mode,
                          logbfrT *logbfr ) {
  const char myname[]="svr_set_fmode";

  const mode_t tbl[3][8]= {
     {        0,
              S_IXOTH,
              S_IWOTH,
              S_IWOTH | S_IXOTH,
              S_IROTH,
              S_IROTH | S_IXOTH,
              S_IROTH | S_IWOTH,
              S_IROTH | S_IWOTH | S_IXOTH   },
     {        0,
              S_IXGRP,
              S_IWGRP,
              S_IWGRP | S_IXGRP,
              S_IRGRP,
              S_IRGRP | S_IXGRP,
              S_IRGRP | S_IWGRP,
              S_IRGRP | S_IWGRP | S_IXGRP   },
      {       0,
              S_IXUSR,
              S_IWUSR,
              S_IWUSR | S_IXUSR,
              S_IRUSR,
              S_IRUSR | S_IXUSR,
              S_IRUSR | S_IWUSR,
              S_IRUSR | S_IWUSR | S_IXUSR   }
  };

  int errn, errno_save;
  static char mode_str[32];
  mode_t determined_mode;
  int i, mode_str_len;
  short other_byte=0, group_byte=0, user_byte=0, which_mode_byte=0;
  struct stat s;

  if (*path==0x0) {
    if (IFS_trace) {
       errn=ERR_SVR(ERR_SVR_DEBUG);
       (void)utilLog(myname, LOG_STR, &errn,
         err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
         "*path==0x0", (int *)NULL,
         (char *)NULL, (int *)NULL,
         (char *)NULL, (int *)NULL,
         logbfr);
    }
    return IF_SUCCESS;
  }
 
/*
 * set mode 
 */

  if (mode != 0) {

    (void)sprintf(mode_str, "%i", mode);
    mode_str_len=strlen(mode_str);
    for (i=0; i<mode_str_len && which_mode_byte<3; i++, which_mode_byte++) {
      *(mode_str+mode_str_len-i)=0x0;
      switch(which_mode_byte) {
        case 0:
          other_byte=atoi(mode_str+mode_str_len-i-1);
          break;
        case 1:
          group_byte=atoi(mode_str+mode_str_len-i-1);
          break;
        case 2:
          user_byte=atoi(mode_str+mode_str_len-i-1);
          break;
      }
    }
   
    if (other_byte > 7 || group_byte > 7 || user_byte > 7) {
      errn=ERR_SVR(ERR_SVR_FILE_MODE);
      (void)utilLog( myname, LOG_STR, &errn,
               err_svr_table[ERR_SVR_FILE_MODE], (int *)NULL,
               "other=", (int *)&other_byte,
               ": group=", (int *)&group_byte,
               ": user=", (int *)&user_byte,
               logbfr);
       return !IF_SUCCESS;
    }
    determined_mode=tbl[0][other_byte] | tbl[1][group_byte] | tbl[2][user_byte];
    if (chmod(path, determined_mode) < 0) {
      errno_save=errno;
      errn=ERR_SVR(ERR_SVR_CHMOD); 
      utilLog(myname, LOG_STR, &errn,
            err_svr_table[ERR_SVR_CHMOD], (int *)NULL,
            path, (int *)NULL,
            ": ", (int *)&determined_mode,
            strerror(errno_save), (int *)NULL,
            logbfr);
      return !IF_SUCCESS;
    }
 

    if (IFS_trace) {
      errn=ERR_SVR(ERR_SVR_VERBOSE);
      (void)utilLog(myname, LOG_STR, &errn,
             err_svr_table[ERR_SVR_VERBOSE], (int *)NULL,
            "file mode determined=", (int *)&determined_mode,
            (char *)NULL, (int *)NULL,
            (char *)NULL, (int *)NULL,
            logbfr);
    }

  } /* end of set mode */



/*
 * set uid gid
 */

  if ( uid != 0 || gid != 0 ) {
    if (stat(path, &s) < 0) {
      errno_save=errno;
      errn=ERR_SVR(ERR_SVR_STAT);
      utilLog(myname, LOG_STR, &errn,
            err_svr_table[ERR_SVR_STAT], (int *)NULL,
            path, (int *)NULL,
            ": ", (int *)NULL,
            strerror(errno_save), (int *)NULL,
            logbfr);
      return !IF_SUCCESS;
    }
    if (chown(path, ((uid != 0) ? uid : s.st_uid), ((gid != 0) ? gid : s.st_gid)) < 0) { 
      errno_save=errno;
      errn=ERR_SVR(ERR_SVR_CHOWN);
      utilLog(myname, LOG_STR, &errn,
            err_svr_table[ERR_SVR_CHOWN], (int *)((uid != 0) ? &uid : &s.st_uid),
            path, (int *)((gid != 0) ? &gid : &s.st_gid),
            ": ", (int *)NULL,
            strerror(errno_save), (int *)NULL,
            logbfr);
      return !IF_SUCCESS;
    }

  } /* end of set uid gid */


  return IF_SUCCESS;

}

static int exec_funcs(int exec_if,
                      char *exec_path,
                      char *exec_arga,
                      char *exec_argb,
                      char *exec_argc,
                      char *exec_argd,
                      logbfrT *logbfr ) {
  
  const char myname[]="exec_funcs";

  int errn, errno_save, status=0;
  pid_t cpid;

  switch (exec_if) {
    case SUPP_EXNOOP:
      return IF_SUCCESS;
    case SUPP_ALTO:
    case SUPP_INPIPE:
    case SUPP_OUTPIPE:
    case SUPP_IOPIPE:
      errn=ERR_SVR(ERR_SVR_EXEC_IF);
      utilLog(myname, LOG_STR, &errn,
            err_svr_table[ERR_SVR_EXEC_IF], (int *)&exec_if,
            ": currently not supported", (int *)NULL,
            (char *)NULL, (int *)NULL,
            (char *)NULL, (int *)NULL,
            logbfr);
      return !IF_SUCCESS;
    case SUPP_EXEC:
    case SUPP_FORK:
      if ((cpid=fork()) < 0) {
        errno_save=errno;
        errn=ERR_SVR(ERR_SVR_FORK);
        utilLog(myname, LOG_STR, &errn,
            err_svr_table[ERR_SVR_FORK], (int *)&exec_if,
            (char *)NULL, (int *)NULL,
            ": ", (int *)NULL,
            strerror(errno_save), (int *)NULL,
            logbfr);
        return !IF_SUCCESS;
      }
      if (cpid==0) {          /* child process */
        static char *exec_argv[6];
         
        build_argv(exec_argv, exec_path, exec_arga, exec_argb, exec_argc, exec_argd);
        if (execvp(exec_path, exec_argv)<0) {

           static logbfrT logbfr_child;
           static char logstr_child[MAX_LOG_MSG_LEN+1];
           logstr_child[0]=0x0;
           logbfr_child.str=logstr_child;
           logbfr_child.str_sz=MAX_LOG_MSG_LEN;

           errno_save=errno;
           errn=ERR_SVR(ERR_SVR_FORK);
           utilLog("child", LOG_STR, &errn,
               err_svr_table[ERR_SVR_FORK], (int *)NULL,
               exec_path, (int *)NULL,
               ": ", (int *)NULL,
               strerror(errno_save), (int *)NULL,
               &logbfr_child);     
           utilLogger(loggerpath, myname, errno_save, &logbfr_child, 0);
        }
        exit(EXIT_FAILURE);             /* child process error */
      }
      while (wait(&status) != cpid) /* wait for the child */
           ;
      errn=ERR_SVR(ERR_SVR_DEBUG);
      (void)utilLog(myname, LOG_STR, &errn,
            err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
            ": ", (int *)NULL,
            exec_path, (int *)NULL,
            ": status from child: ", &status,
            logbfr);
      status=WEXITSTATUS(status); 
      (void)utilLog(myname, LOG_STR, &errn,
            err_svr_table[ERR_SVR_DEBUG], (int *)NULL,
            ": ", (int *)NULL,
            exec_path, (int *)NULL,
            ": WEXITSTATUS(status): ", &status,
            logbfr);
      return status;
    default:
      errn=ERR_SVR(ERR_SVR_EXEC_IF);
      utilLog(myname, LOG_STR, &errn,
            err_svr_table[ERR_SVR_EXEC_IF], (int *)&exec_if,
            (char *)NULL, (int *)NULL,
            (char *)NULL, (int *)NULL,
            (char *)NULL, (int *)NULL,
            logbfr);
      return !IF_SUCCESS;
  }
}

static void build_argv(char *argv[], char *path, char *arga, char *argb, char *argc, char *argd) {

  int i,j;

  argv[0]=path;
  for (j=0 ,i=1; j<5; j++) {
    switch (j) {
      case 0:
        if (*arga != 0x0) {
          argv[i]=arga;
          i++;
        }
        break;
      case 1:
       if (*argb != 0x0) { 
          argv[i]=argb;
          i++;
       } 
       break;
      case 2:
        if (*argc != 0x0) {
          argv[i]=argc; 
          i++;
        }
        break;
      case 3:
       if (*argd != 0x0) {
          argv[i]=argd; 
          i++;
       }
       break;
      case 4:
       argv[i]=NULL; 
       break;
    }
  }
}

static int do_this_if(char *path, logbfrT *logbfr) {

  const char myname[]="do_this_if";
  
  int errn, errno_save;
  struct stat s;

  if (stat(path, &s) < 0) {
    errno_save=errno;
    errn=ERR_SVR(ERR_SVR_STAT);
    utilLog(myname, LOG_STR, &errn,
          err_svr_table[ERR_SVR_STAT], (int *)NULL,
          path, (int *)NULL,
          ": ", (int *)NULL,
          strerror(errno_save), (int *)NULL,
          logbfr);
    return !IF_SUCCESS;
  }

  return IF_SUCCESS;
}

static void die(int s) {
  svc_unregister(IF_PROG, IF_VER);
  exit(EXIT_SUCCESS);
}

static void hup(int s) {
  initallow=1;
}
